@@ -1,4 +1,4 @@
-/* Copyright (C) 2003,2004 MySQL AB
+/* Copyright (C) 2004 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -49,11 +49,13 @@ extern "C" {
  * Enums
  */
 
+// When this type is changed then also the error mapper in myx_backup.c must be adjusted!
 typedef enum myx_lib_error
 {
     MYX_NO_ERROR = 0, MYX_ERROR_CANT_OPEN_FILE, MYX_ERROR_CANT_CONNECT_TO_INSTANCE, MYX_XML_PARSE_ERROR,
     MYX_XML_NO_VALID_DOCUMENT, MYX_XML_EMPTY_DOCUMENT, MYX_SQL_ERROR, MYX_STOP_EXECUTION, MYX_ALLOC_CHANGE_ERROR,
-    MYX_OBJECT_NOT_FOUND, MYX_CANT_READ_FROM_FILE, MYX_CHARSET_CONVERSION_ERROR,MYX_CHARSET_WRONG_CHARSET_SPECIFIED
+    MYX_OBJECT_NOT_FOUND, MYX_CANT_READ_FROM_FILE, MYX_CHARSET_CONVERSION_ERROR,MYX_CHARSET_WRONG_CHARSET_SPECIFIED,
+    MYX_MEMORY_LIMIT_EXCEEDED, MYX_OUT_OF_MEMORY
 } MYX_LIB_ERROR;
 
 typedef enum myx_user_connection_type
@@ -205,6 +207,8 @@ typedef struct {
   unsigned int not_null;
 } MYX_SCHEMA_TABLE_COLUMN;
 
+typedef MYX_SCHEMA_TABLE_COLUMN MYX_SCHEMA_VIEW_COLUMN;
+
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_TABLE_INDEX_COLUMN
 typedef struct {
   char *column_name;
@@ -226,21 +230,21 @@ typedef struct {
 
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_TABLE_STATUS
 typedef struct {
-  unsigned char *table_name;
-  unsigned char *table_type;
-  unsigned char *row_format;
-  unsigned char *rows;
-  unsigned char *avg_row_length;
-  unsigned char *data_length;
-  unsigned char *max_data_length;
-  unsigned char *index_length;
-  unsigned char *data_free;
-  unsigned char *auto_increment;
-  unsigned char *create_time;
-  unsigned char *update_time;
-  unsigned char *check_time;
-  unsigned char *create_options;
-  unsigned char *comment;
+  char *table_name;
+  char *table_type;
+  char *row_format;
+  char *rows;
+  char *avg_row_length;
+  char *data_length;
+  char *max_data_length;
+  char *index_length;
+  char *data_free;
+  char *auto_increment;
+  char *create_time;
+  char *update_time;
+  char *check_time;
+  char *create_options;
+  char *comment;
 
   unsigned int indexes_num;
   MYX_TABLE_INDEX *indexes;
@@ -249,17 +253,62 @@ typedef struct {
   MYX_SCHEMA_TABLE_COLUMN *columns;
 } MYX_TABLE_STATUS;
 
+//typedef MYX_TABLE_STATUS MYX_VIEW_STATUS;
+
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_SCHEMA_TABLE_STATUS
 typedef struct {
   unsigned int schema_tables_num;
   MYX_TABLE_STATUS *schema_tables;
 } MYX_SCHEMA_TABLE_STATUS;
 
-  
+//typedef MYX_SCHEMA_TABLE_STATUS MYX_SCHEMA_VIEW_STATUS;
+
+// -----------------------------------------------------------------------------------------------------------
+// View Status
+
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_VIEW_STATUS
+typedef struct 
+{
+  char *view_name;
+  char *comment;
+  unsigned int flags;
+} MYX_VIEW_STATUS;
+
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_SCHEMA_VIEW_STATUS
+typedef struct 
+{
+  unsigned int schema_views_num;
+  MYX_VIEW_STATUS *schema_views;
+} MYX_SCHEMA_VIEW_STATUS;
+
+// -----------------------------------------------------------------------------------------------------------
+// Schema entiry Status
+
+typedef enum
+{
+  MYX_ENTITY_TABLE = 1,
+  MYX_ENTITY_VIEW,
+  MYX_ENTITY_PROC,
+  MYX_ENTITY_FUNC
+} MYX_ENTITY_TYPE;
+
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_ENTITY_STATUS
+typedef struct 
+{
+  MYX_ENTITY_TYPE entity_type;
+  void *entity;
+} MYX_ENTITY_STATUS;
+
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_SCHEMA_ENTITY_STATUS
+typedef struct 
+{
+  unsigned int schema_entities_num;
+  MYX_ENTITY_STATUS *schema_entities;
+} MYX_SCHEMA_ENTITY_STATUS;
+
 // -----------------------------------------------------------------------------------------------------------
 // Catalog / Schema Info
 
-
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_SCHEMA_INDICES
 typedef struct {
   unsigned int indices_num;
@@ -360,13 +409,13 @@ typedef struct {
 
 typedef struct myx_trans_text
 {
-    char *id;
+    unsigned char *id;
     char *trans;
 } MYX_TRANS_TEXT;
 
 typedef struct myx_trans_textgroup
 {
-    char *id;
+    unsigned char *id;
     unsigned int texts_num;
     MYX_TRANS_TEXT *texts;
 } MYX_TRANS_TEXTGROUP;
@@ -436,6 +485,7 @@ typedef struct
 typedef enum
 {
   MYX_QT_SELECT,
+  MYX_QT_SELECT_INTO_OUTFILE,
   MYX_QT_UPDATE,
   MYX_QT_INSERT,
   MYX_QT_DELETE,
@@ -500,7 +550,8 @@ typedef enum
 {
   MYX_RSCT_INTEGER= 0, MYX_RSCT_FLOAT, MYX_RSCT_STRING, MYX_RSCT_DATE, MYX_RSCT_TIME, MYX_RSCT_DATETIME,
   MYX_RSCT_BLOB, MYX_RSCT_TEXT,
-  MYX_RSCT_ENUM, MYX_RSCT_SET
+  MYX_RSCT_ENUM, MYX_RSCT_SET,
+  MYX_RSCT_DECIMAL
 } MYX_RS_COLUMN_TYPE;
 
 #define MYX_RSCT_NEEDS_QUOTE(t) (t != MYX_RSCT_INTEGER && t != MYX_RSCT_FLOAT)
@@ -649,6 +700,8 @@ typedef struct
 
   struct MYX_RS_ACTIONS *actions;
 
+  bigint memory_used;
+  
   double query_time;
   double fetch_time;
 } MYX_RESULTSET;
@@ -965,7 +1018,8 @@ typedef enum {
   MYX_TE_COL_STRING,
   MYX_TE_COL_INTEGER,
   MYX_TE_COL_FLOAT,
-  MYX_TE_COL_BOOL
+  MYX_TE_COL_BOOL,
+  MYX_TE_COL_DATE
 } MYX_TE_COLUMN_TYPE;
 
 
@@ -1076,6 +1130,7 @@ MYX_PUBLIC_FUNC bigint myx_mysql_affected_rows(MYSQL *mysql);
 MYX_PUBLIC_FUNC char * myx_get_mysql_full_version(MYSQL *mysql);
 MYX_PUBLIC_FUNC int myx_get_mysql_major_version(MYSQL *mysql);
 MYX_PUBLIC_FUNC int myx_get_mysql_minor_version(MYSQL *mysql);
+MYX_PUBLIC_FUNC int myx_get_mysql_release(MYSQL *mysql);
 MYX_PUBLIC_FUNC int mysql_version_is_later_or_equal_than(MYSQL *mysql,
                                                          int major_version, int minor_version);
 MYX_PUBLIC_FUNC int mysql_full_version_is_later_or_equal_than(MYSQL *mysql,
@@ -1119,13 +1174,17 @@ MYX_PUBLIC_FUNC int myx_get_available_server_features(MYSQL *mysql, MYX_AVAILABL
 MYX_PUBLIC_FUNC MYX_CATALOGS * myx_get_catalogs(MYSQL *mysql);
 MYX_PUBLIC_FUNC MYX_SCHEMA_TABLES * myx_get_schema_tables(MYSQL *mysql, const char *catalog_name, const char *schema_name);
 MYX_PUBLIC_FUNC MYX_SCHEMA_INDICES * myx_get_schema_indices(MYSQL *mysql, const char *catalog_name, const char *schema_name);
-MYX_PUBLIC_FUNC MYX_SCHEMA_STORED_PROCEDURES * myx_get_schema_sps(MYSQL *mysql, const char *catalog_name, const char *schema_name);
 MYX_PUBLIC_FUNC MYX_SCHEMA_TABLE_STATUS * myx_get_schema_table_status(MYSQL *mysql, const char *catalog_name, const char *schema_name);
+MYX_PUBLIC_FUNC MYX_SCHEMA_VIEW_STATUS * myx_get_schema_view_status(MYSQL *mysql, const char *catalog_name, const char *schema_name);
+MYX_PUBLIC_FUNC MYX_SCHEMA_STORED_PROCEDURES * myx_get_schema_sps(MYSQL *mysql, const char *catalog_name, const char *schema_name);
+MYX_PUBLIC_FUNC MYX_SCHEMA_ENTITY_STATUS * myx_get_schema_entity_status(MYSQL *mysql, const char *catalog_name, const char *schema_name);
 MYX_PUBLIC_FUNC int myx_free_catalogs(MYX_CATALOGS *catalogs);
 MYX_PUBLIC_FUNC int myx_free_schema_tables(MYX_SCHEMA_TABLES *schema_tables);
 MYX_PUBLIC_FUNC int myx_free_schema_indices(MYX_SCHEMA_INDICES *schema_indices);
-MYX_PUBLIC_FUNC int myx_free_schema_sps(MYX_SCHEMA_STORED_PROCEDURES *schema_sps);
+MYX_PUBLIC_FUNC int myx_free_schema_entity_status(MYX_SCHEMA_ENTITY_STATUS *schema_tables);
 MYX_PUBLIC_FUNC int myx_free_schema_table_status(MYX_SCHEMA_TABLE_STATUS *schema_tables);
+MYX_PUBLIC_FUNC int myx_free_schema_view_status(MYX_SCHEMA_VIEW_STATUS *schema_tables);
+MYX_PUBLIC_FUNC int myx_free_schema_sps(MYX_SCHEMA_STORED_PROCEDURES *schema_sps);
 
 //Application options functions
 MYX_PUBLIC_FUNC int myx_store_application_options(MYX_APPLICATION_OPTIONS *options, char *filename);
@@ -1166,6 +1225,9 @@ MYX_PUBLIC_FUNC MYX_SYN_SUGGESTIONS* myx_lookup_word(MYX_SYN* syn,const char *wo
 MYX_PUBLIC_FUNC int myx_free_syn_suggestions(MYX_SYN_SUGGESTIONS *s);
 
 // query functions
+MYX_PUBLIC_FUNC void myx_mysql_limit_resultset_size(MYSQL *mysql, unsigned int limitMB);
+MYX_PUBLIC_FUNC unsigned int myx_mysql_get_resultset_size_limit(MYSQL *mysql);
+
 MYX_PUBLIC_FUNC int myx_identifier_needs_quotes(const char *name);
 MYX_PUBLIC_FUNC MYX_RESULTSET * myx_query_execute(MYSQL *mysql, const char *sql, int enforce_editable,
   MYX_STRINGLIST *params, MYX_LIB_ERROR *error_code, void *user_data,
@@ -1186,6 +1248,8 @@ MYX_PUBLIC_FUNC int myx_query_pack_resultset(MYX_RESULTSET *resultset);
 MYX_PUBLIC_FUNC int myx_query_discard_actions(MYX_RESULTSET *resultset);
 MYX_PUBLIC_FUNC int myx_query_free_action_errors(MYX_RS_ACTION_ERRORS *errors);
 
+MYX_PUBLIC_FUNC int myx_get_unix_newline_count(const char *str);
+
 MYX_PUBLIC_FUNC MYX_Q_TYPE myx_query_type(const char *sql);
 MYX_PUBLIC_FUNC char * myx_query_add_column_to_sql(MYSQL *mysql, const char *catalog, const char *schema, 
                                                    const char *table, const char *column, char *sqlcmd, 
@@ -1232,6 +1296,17 @@ MYX_PUBLIC_FUNC char * myx_dbm_make_script_from_sps(MYSQL *mysql, const char *ca
 MYX_PUBLIC_FUNC MYX_LIB_ERROR myx_dbm_create_view(MYSQL *mysql, const char *catalog, const char *schema,
                                                   const char *view_name, const char *select_sql);
 
+MYX_PUBLIC_FUNC MYX_LIB_ERROR myx_dbm_drop_sp(MYSQL *mysql, 
+                                              const char *catalog_name, 
+                                              const char *schema_name, 
+                                              const char *sp_name, 
+                                              MYX_SCHEMA_STORED_PROCEDURE_TYPE sp_type);
+
+MYX_PUBLIC_FUNC MYX_LIB_ERROR myx_dbm_drop_view(MYSQL *mysql, 
+                                                const char *catalog_name, 
+                                                const char *schema_name, 
+                                                const char *view_name);
+
 MYX_PUBLIC_FUNC MYX_DBM_VIEW_DATA * myx_dbm_get_view_data(MYSQL *mysql, const char *catalog, const char *schema,
                                                          const char *view);
   
@@ -1260,7 +1335,6 @@ MYX_PUBLIC_FUNC int myx_free_charsets(MYX_DBM_CHARSETS *obj);
 MYX_PUBLIC_FUNC MYX_INTL_FILE * myx_new_intl_file(const char *filename, const char *charset, MYX_LIB_ERROR *error);
 MYX_PUBLIC_FUNC void myx_free_intl_file(MYX_INTL_FILE *intl_file);
 MYX_PUBLIC_FUNC bigint myx_read_char_from_intl_file(MYX_INTL_FILE *intl_file, int *bytes_read, MYX_LIB_ERROR *error);
-MYX_PUBLIC_FUNC char * myx_detect_charset_from_sql_file(const char * filename);
 
 /* Others */
 MYX_PUBLIC_FUNC MYX_IMAGE_FORMAT myx_guess_image_format(const char *data, unsigned int length);
