@@ -91,7 +91,7 @@ char* hex_decode(const char *hex_str, int *ret_str_len)
   }
   result[i]= 0;
 
-  return result;
+  return (char*)result;
 }
 
 /* This functions transforms an arbitrary sequence of bytes
@@ -402,7 +402,11 @@ char *str_g_subst(const char *str, const char *search, const char *replace)
   char *res;
   const char *bptr, *ptr;
   
-  g_return_val_if_fail(str != NULL && *str, g_strdup(str));
+  g_return_val_if_fail(str != NULL, g_strdup(str));
+  if(str[0] == '\0')
+  {
+    return g_strdup(str);
+  }
   g_return_val_if_fail(search != NULL && *search, g_strdup(str));
   g_return_val_if_fail(replace != NULL, g_strdup(str));
 
@@ -685,6 +689,99 @@ char *str_right(char *dest, char *src, unsigned int len)
     return dest;
 }
 
+char *str_align_left(const char *txt, unsigned int width, char linechar)
+{
+  char *dst= g_malloc(width + 1);
+  unsigned int len= (unsigned int)strlen(txt);
+  unsigned int i;
+
+  if (len > width)
+    len= width;
+
+  memset(dst, linechar, width);
+  dst[width]= 0;
+
+  for (i= 0; i < len; i++)
+    dst[i]= txt[i];
+
+  return dst;
+}
+
+char *str_align_right(const char *txt, unsigned int width, char linechar)
+{
+  char *dst= g_malloc(width + 1);
+  unsigned int len= (unsigned int)strlen(txt);
+  unsigned int i;
+
+  if (len > width)
+    len= width;
+
+  memset(dst, linechar, width);
+  dst[width]= 0;
+
+  for (i= 0; i < len; i++)
+    dst[i + width - len]= txt[i];
+
+  return dst;
+}
+
+char *str_align_center(const char *txt, unsigned int width, char linechar)
+{
+  char *dst= g_malloc(width + 1);
+  unsigned int i;
+  unsigned int len= (unsigned int)strlen(txt);
+  unsigned int p= div(width, 2).quot - div(len, 2).quot;
+
+  if (len > width)
+    len= width;
+
+  memset(dst, linechar, width);
+  dst[width]= 0;
+
+  for (i= 0; i < len; i++)
+    dst[p + i]= txt[i];
+ 
+  return dst;
+}
+
+char *auto_line_break(const char *txt, unsigned int width, char sep)
+{
+  char *dst= g_malloc((width + 2) * 80);
+  unsigned int i, o= 0, p= 0, w= 0, l= (unsigned int)strlen(txt);
+
+  for (i= 0; i < l;)
+  {
+    w++;
+
+    if (w > width)
+    {
+#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
+      dst[p + o]= '\r';
+      dst[p + o + 1]= '\n';
+
+      o+= 1;
+#else
+      dst[p + o]= '\n';
+#endif
+      i= p + 1;
+      w= 0;
+    }
+    else
+    {
+      dst[i + o]= txt[i];
+
+      if (txt[i] == sep)
+        p= i;
+
+        i++;
+    }
+  }
+
+  dst[i + o]= 0;
+
+  return dst;
+}
+
 int sub_str_count(char *search_str, const char *src)
 {
     unsigned int i;
@@ -717,7 +814,10 @@ int str_endswith(const char *str, const char *substr)
 
 int str_beginswith(const char *str, const char *substr)
 {
-  return strncmp(str, substr, strlen(substr))==0;
+  if (str)
+    return strncmp(str, substr, strlen(substr))==0;
+  else
+    return 0;
 }
 
 
@@ -1030,7 +1130,73 @@ static const char *get_cpu_type_name(int cpu_type, int cpu_subtype)
   switch (cpu_type)
   {
     case CPU_TYPE_I386:
-      return "ix86";
+      switch (cpu_subtype)
+      {
+        case CPU_SUBTYPE_386:
+          return "80386";
+        case CPU_SUBTYPE_486:
+          return "80486";
+        case CPU_SUBTYPE_486SX:
+          return "80486SX";
+        case CPU_SUBTYPE_PENT:
+          return "Pentium";
+        case CPU_SUBTYPE_PENTPRO:
+          return "Pentium Pro";
+        case CPU_SUBTYPE_PENTII_M3:
+          return "Pentium II";
+        case CPU_SUBTYPE_PENTII_M5:
+          return "Pentium II";
+#ifdef CPU_SUBTYPE_CELERON
+        case CPU_SUBTYPE_CELERON:
+          return "Celeron";
+#endif
+#ifdef CPU_SUBTYPE_CELERON_MOBILE
+        case CPU_SUBTYPE_CELERON_MOBILE:
+          return "Celeron Mobile";
+#endif
+#ifdef CPU_SUBTYPE_PENTIUM_3
+        case CPU_SUBTYPE_PENTIUM_3:
+          return "Pentium III";
+#endif
+#ifdef CPU_SUBTYPE_PENTIUM_3_M
+        case CPU_SUBTYPE_PENTIUM_3_M:
+          return "Pentium III M";
+#endif
+#ifdef CPU_SUBTYPE_PENTIUM_3_XEON
+        case CPU_SUBTYPE_PENTIUM_3_XEON:
+          return "Pentium III Xeon";
+#endif
+#ifdef CPU_SUBTYPE_PENTIUM_M
+        case CPU_SUBTYPE_PENTIUM_M:
+          return "Pentium M";
+#endif
+#ifdef CPU_SUBTYPE_PENTIUM_4
+        case CPU_SUBTYPE_PENTIUM_4:
+          return "Pentium 4";
+#endif
+#ifdef CPU_SUBTYPE_PENTIUM_4_M
+        case CPU_SUBTYPE_PENTIUM_4_M:
+          return "Pentium 4M";
+#endif
+#ifdef CPU_SUBTYPE_ITANIUM
+        case CPU_SUBTYPE_ITANIUM:
+          return "Itanium";
+#endif
+#ifdef CPU_SUBTYPE_ITANIUM_2
+        case CPU_SUBTYPE_ITANIUM_2:
+          return "Itanium 2";
+#endif
+#ifdef CPU_SUBTYPE_XEON
+        case CPU_SUBTYPE_XEON:
+          return "Xeon";
+#endif
+#ifdef CPU_SUBTYPE_XEON_MP
+        case CPU_SUBTYPE_XEON_MP:
+          return "Xeon MP";
+#endif
+        default:
+          return "x86";
+      }
       break;
     case CPU_TYPE_POWERPC:
       switch (cpu_subtype)
@@ -1059,6 +1225,8 @@ static const char *get_cpu_type_name(int cpu_type, int cpu_subtype)
           return "PowerPC G4";
         case CPU_SUBTYPE_POWERPC_970:
           return "PowerPC G5";
+        default:
+          return "PowerPC";
       }
       break;
   }
@@ -1072,7 +1240,6 @@ static int _get_hardware_info(char **cpu, char **clock, int *cpu_count, unsigned
   int mib[2];
   size_t length;
   int tmp;
-  uint64_t memsize;
   char *mclass;
   int cpu_type, cpu_subtype;
   
@@ -1106,23 +1273,11 @@ static int _get_hardware_info(char **cpu, char **clock, int *cpu_count, unsigned
   mib[1]= HW_NCPU;
   length= sizeof(cpu_count);
   if (sysctl(mib, 2, cpu_count, &length, NULL, 0) < 0)
-    cpu_count= 1;
+    *cpu_count= 1;
 
   // get memory size
-  mib[0]= CTL_HW;
-  mib[1]= HW_MEMSIZE;
-  length= sizeof(memsize);
-  if (sysctl(mib, 2, &memsize, &length, NULL, 0) < 0)
-  {
-    mib[0]= CTL_HW;
-    mib[1]= HW_PHYSMEM;
-    length= sizeof(tmp);
-    sysctl(mib, 2, &tmp, &length, NULL, 0);
-    *mem_kb= tmp/1024;
-  }
-  else
-    *mem_kb= memsize/1024;
-  
+  *mem_kb= get_physical_memory_size()/1024;
+
   return 0;
 }
 
@@ -1159,37 +1314,8 @@ static int _get_hardware_info(char **cpu, char **clock, int *cpu_count, unsigned
   }
   fclose(proc);
   
-  
-  // fetch physical memory info from /proc/meminfo
-  proc= fopen("/proc/meminfo", "r");
-  if (proc) 
-  {
-    char *ptr, *end;
-    
-    *mem_kb= 0;
-    while (fgets(line, sizeof(line), proc))
-    {
-        if (strncasecmp(line, "MemTotal:", sizeof("MemTotal:")-1)==0)
-        {
-          char *line_end= line+strlen(line);
-          ptr= strchr(line, ':')+1;
-          while (*ptr && *ptr==' ') ptr++;
-          end= strchr(ptr, ' ');
-          if (end)
-            *end= 0;
-          if (end < line_end)
-            end++;
-          if (strstr(end, "mB") || strstr(end, "MB"))
-            *mem_kb= strtoul(str_trim(ptr), NULL, 10)*1024;
-          else if (strstr(end, "kB") || strstr(end, "KB"))
-            *mem_kb= strtoul(str_trim(ptr), NULL, 10);
-          else
-            *mem_kb= strtoul(str_trim(ptr), NULL, 10)/1024;
-          break;
-        }
-    }
-    fclose(proc);
-  }
+  *mem_kb= get_physical_memory_size();
+
   return 0;
 }
 #endif
@@ -1362,6 +1488,80 @@ int copy_file(const char *orig_file, const char *new_file)
 #endif
 
 
+
+bigint get_physical_memory_size()
+{
+#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
+  MEMORYSTATUS memstat;
+  
+  GlobalMemoryStatus(&memstat);
+  
+  return memstat.dwTotalPhys;
+#elif defined(__APPLE__)
+  uint64_t mem64;
+  int mib[2];
+  int mem32;
+  size_t length;
+  mib[0]= CTL_HW;
+  mib[1]= HW_MEMSIZE;
+  length= sizeof(mem64);
+  if (sysctl(mib, 2, &mem64, &length, NULL, 0) < 0)
+  {
+    mib[0]= CTL_HW;
+    mib[1]= HW_PHYSMEM;
+    length= sizeof(mem32);
+    sysctl(mib, 2, &mem32, &length, NULL, 0);
+    mem64= mem32;
+  }
+  
+  return mem64;
+#else
+  //long long mem64;
+  FILE *proc;
+  bigint mem64;
+  mem64= 0;
+  // fetch physical memory info from /proc/meminfo
+  proc= fopen("/proc/meminfo", "r");
+  if (proc) 
+  {
+    char line[1024];
+    char *ptr, *end;
+    
+    while (fgets(line, sizeof(line), proc))
+    {
+      if (strncasecmp(line, "MemTotal:", sizeof("MemTotal:")-1)==0)
+      {
+        char *line_end= line+strlen(line);
+        ptr= strchr(line, ':')+1;
+        while (*ptr && *ptr==' ') ptr++;
+        end= strchr(ptr, ' ');
+        if (end)
+          *end= 0;
+        if (end < line_end)
+          end++;
+        if (strstr(end, "gB") || strstr(end, "GB"))
+          mem64= strtoul(str_trim(ptr), NULL, 10)*1024*1024*1024LL;
+        else if (strstr(end, "mB") || strstr(end, "MB"))
+          mem64= strtoul(str_trim(ptr), NULL, 10)*1024*1024LL;
+        else if (strstr(end, "kB") || strstr(end, "KB"))
+          mem64= strtoul(str_trim(ptr), NULL, 10)*1024LL;
+        else
+          mem64= strtoul(str_trim(ptr), NULL, 10);
+        break;
+      }
+    }
+    fclose(proc);
+  }
+  else
+  {
+    g_warning("Memory stats retrieval not implemented for this system");
+  }
+  return mem64;
+
+#endif
+}
+
+
 bigint get_file_size(const char *filename)
 {
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
@@ -1451,14 +1651,17 @@ char * get_value_from_text_ex_opt(const char *txt, int txt_length,
     pcre_exp= pcre_compile(regexpr, PCRE_CASELESS, &error_str, &erroffset, NULL);
     if (pcre_exp)
     {
-      if ( (rc= pcre_exec(pcre_exp, NULL, txt, txt_length, 0, 
+      if ((rc= pcre_exec(pcre_exp, NULL, txt, txt_length, 0, 
                           options_for_exec, o_vector, O_VECTOR_COUNT) ) > 0)
       {
-        pcre_get_substring(txt, o_vector, rc, substring_nr, &ret_val);
+        if (o_vector[substring_nr * 2] != -1)
+        {
+          pcre_get_substring(txt, o_vector, rc, substring_nr, &ret_val);
 
-        value= g_strdup(ret_val);
+          value= g_strdup(ret_val);
 
-        pcre_free_substring((char*)ret_val);
+          pcre_free_substring((char*)ret_val);
+        }
       }
 
       pcre_free(pcre_exp);
@@ -1479,6 +1682,19 @@ char * get_value_from_text(const char *txt, int txt_length, const char *regexpr)
   return get_value_from_text_ex(txt, txt_length, regexpr, 1);
 }
 
+int strlist_g_indexof(const char **list, const char *value)
+{
+  int i= 0;
+  while(list[i] != NULL)
+  {
+    if(strcmp(list[i], value) == 0)
+    {
+      return i;
+    }
+  }
+  return -1;
+}
+
 void strlist_g_append(char ***list, char *value)
 {
   unsigned int i= 0;
@@ -1498,6 +1714,18 @@ void strlist_g_append(char ***list, char *value)
   }
 }
 
+void strlist_g_append_or_replace(char ***list, char *value)
+{
+  int i= strlist_g_indexof(*list, value);
+  if(i < 0)
+  {
+    strlist_g_append(list, value);
+  }
+  else
+  {
+    (*list)[i]= value;
+  }
+}
 
 const char *strfindword(const char *str, const char *word)
 {
@@ -1856,19 +2084,25 @@ char *unescape_html_entities(const char *str)
 
 
 //taken from glib
-int myx_mkdir(const char *filename, int mode)
+int myx_mkdir(const char *filename, int mode, int *error_no)
 {
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
   wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
-  int retval = _wmkdir (wfilename);
-  int save_errno = errno;
+  int retval= _wmkdir (wfilename);
+  int save_errno= errno;
+
+  *error_no= errno;
 
   g_free (wfilename);
   
-  errno = save_errno;
+  errno= save_errno;
   return retval;
 #else
-  return mkdir (filename, mode);
+  int retval= mkdir (filename, mode);
+
+  *error_no= errno;
+
+  return retval;
 #endif
 }
 
