@@ -544,6 +544,8 @@ MYX_DBM_TABLE_DATA * myx_dbm_retrieve_table_data(MYSQL *mysql, MYX_DBM_DATATYPES
 
       for(column= table_data->columns; (row= mysql_fetch_row(res))!=NULL; column++)
       {
+        column->default_value_is_null= 1;
+
         for (i= 0; i < num_fields; i++)
         {
           if (strcmp(fields[i].name, "Field") == 0)
@@ -583,6 +585,8 @@ MYX_DBM_TABLE_DATA * myx_dbm_retrieve_table_data(MYSQL *mysql, MYX_DBM_DATATYPES
           }
           else if (strcmp(fields[i].name, "Default") == 0)
           {
+            column->default_value_is_null= row[i] ? 0 : 1;
+
             column->default_value= myx_convert_dbstr_utf8(mysql, row[i]);
           }
           else if (strcmp(fields[i].name, "Extra")==0)
@@ -600,12 +604,15 @@ MYX_DBM_TABLE_DATA * myx_dbm_retrieve_table_data(MYSQL *mysql, MYX_DBM_DATATYPES
           {
             column->collation= myx_convert_dbstr_utf8(mysql, row[i]);
             column->charset= g_strdup(column->collation);
-            for(j= 0; j<256 && column->charset[j]; j++)
+            if (column->charset)
             {
-              if(column->charset[j] == '_')
+              for(j= 0; j<256 && column->charset[j]; j++)
               {
-                column->charset[j]= 0;
-                break;
+                if(column->charset[j] == '_')
+                {
+                  column->charset[j]= 0;
+                  break;
+                }
               }
             }
           }
@@ -983,6 +990,18 @@ MYX_DBM_CHARSETS * myx_dbm_retrieve_charsets(MYSQL *mysql, MYX_LIB_ERROR *error_
   return charsets;
 }
 
+#ifdef notused
+static int string_contains_only(const char* str, char c)
+{
+	if(*str == '\0')
+	{
+		return 0;
+	}
+	while(*str++ == c);
+	return *str == '\0';
+}
+#endif
+
 char * dbm_get_sql_column_create_code(MYX_DBM_COLUMN_DATA *column, MYX_DBM_SERVER_VERSION *version, int pk_clause)
 {
   char *col_def, *tmp;
@@ -1016,10 +1035,33 @@ char * dbm_get_sql_column_create_code(MYX_DBM_COLUMN_DATA *column, MYX_DBM_SERVE
   if(column->not_null)
     col_def= str_g_append(col_def, " NOT NULL");
 
-  if(!STR_EMPTY(column->default_value))
+  if(!column->default_value_is_null && column->default_value /*&& column->default_value[0]*/)
+                                                             // this caused problems settings empty string def val
   {
-    col_def= str_g_append(col_def, " DEFAULT ");
-    col_def= str_g_append(col_def, column->default_value);
+    tmp = col_def;
+    if((column->datatype_pointer) && (column->datatype_pointer->group == MYX_DBM_DTG_NUMERIC))
+    {
+      if (strcmp(column->default_value, "") != 0)
+        col_def= g_strdup_printf("%s DEFAULT %s", col_def, column->default_value);
+      else
+        col_def= g_strdup_printf("%s DEFAULT 0", col_def);
+    }
+    else if((column->datatype_pointer) && (column->datatype_pointer->group == MYX_DBM_DTG_DATETIME))
+    {
+      // starting with 5.x MySQL server doesn't accept '' as default datetime value
+      if (strcmp(column->default_value, "") != 0)
+      {
+        col_def= g_strdup_printf("%s DEFAULT '%s'", col_def, column->default_value);
+      }
+      else
+      {
+        col_def= g_strdup_printf("%s DEFAULT 0", col_def);
+      }
+    }
+    else
+      col_def= g_strdup_printf("%s DEFAULT '%s'", col_def, column->default_value);
+
+    g_free(tmp);
   }
 
   if(column->auto_inc)
@@ -1070,7 +1112,7 @@ char * myx_dbm_get_sql_index_create_code(MYX_DBM_INDEX_DATA *index, MYX_DBM_SERV
     tmp= NULL;
 
   //Index type
-  if((version->major_version>=4) && (version->minor_version>=1))
+  if((version->major_version>=4) && (version->minor_version>=1) && (index->index_kind != MYX_DBM_IK_FULLTEXT))
   {
     if(index->index_type == MYX_DBM_IT_BTREE)
       tmp2= g_strdup("USING BTREE");
@@ -1196,17 +1238,35 @@ char * myx_dbm_get_sql_option_create_code(MYX_DBM_TABLE_DATA *table, MYX_DBM_SER
   char *sql= NULL;
   char *tmp;
 
-  //Engine
-  switch (table->table_engine)
+  if((version->major_version >= 5) || ((version->major_version >= 4) && (version->minor_version >= 1)))
   {
-    case MYX_DBM_TE_INNODB: sql= str_g_append(sql, "TYPE = InnoDB" _br); break;
-    case MYX_DBM_TE_MEMORY: sql= str_g_append(sql, "TYPE = MEMORY" _br); break;
-    case MYX_DBM_TE_MERGE:  sql= str_g_append(sql, "TYPE = MERGE" _br);  break;
-    case MYX_DBM_TE_BDB:    sql= str_g_append(sql, "TYPE = BDB" _br);    break;
-    case MYX_DBM_TE_ISAM:   sql= str_g_append(sql, "TYPE = ISAM" _br);   break;
-    case MYX_DBM_TE_NDB:    sql= str_g_append(sql, "TYPE = NDB" _br);    break;
-    //default: sql= g_strdup(""); break;
-    default: sql= str_g_append(sql, "TYPE = MYISAM" _br); break;
+    //Engine
+    switch (table->table_engine)
+    {
+      case MYX_DBM_TE_INNODB: sql= str_g_append(sql, "ENGINE = InnoDB" _br); break;
+      case MYX_DBM_TE_MEMORY: sql= str_g_append(sql, "ENGINE = MEMORY" _br); break;
+      case MYX_DBM_TE_MERGE:  sql= str_g_append(sql, "ENGINE = MERGE" _br);  break;
+      case MYX_DBM_TE_BDB:    sql= str_g_append(sql, "ENGINE = BDB" _br);    break;
+      case MYX_DBM_TE_ISAM:   sql= str_g_append(sql, "ENGINE = ISAM" _br);   break;
+      case MYX_DBM_TE_NDB:    sql= str_g_append(sql, "ENGINE = NDB" _br);    break;
+      //default: sql= g_strdup(""); break;
+      default: sql= str_g_append(sql, "ENGINE = MYISAM" _br); break;
+    }
+  }
+  else  // 4.0 or earlier
+  {
+    //Engine
+    switch (table->table_engine)
+    {
+      case MYX_DBM_TE_INNODB: sql= str_g_append(sql, "TYPE = InnoDB" _br); break;
+      case MYX_DBM_TE_MEMORY: sql= str_g_append(sql, "TYPE = MEMORY" _br); break;
+      case MYX_DBM_TE_MERGE:  sql= str_g_append(sql, "TYPE = MERGE" _br);  break;
+      case MYX_DBM_TE_BDB:    sql= str_g_append(sql, "TYPE = BDB" _br);    break;
+      case MYX_DBM_TE_ISAM:   sql= str_g_append(sql, "TYPE = ISAM" _br);   break;
+      case MYX_DBM_TE_NDB:    sql= str_g_append(sql, "TYPE = NDB" _br);    break;
+      //default: sql= g_strdup(""); break;
+      default: sql= str_g_append(sql, "ENGINE = MYISAM" _br); break;
+    }
   }
 
   //auto_inc
@@ -1384,8 +1444,9 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
   MYX_DBM_COLUMN_DATA *column, *ex_column;
   MYX_DBM_INDEX_DATA *index, *ex_index;
   MYX_DBM_FK_DATA *fk, *ex_fk;
+  unsigned int collation_added= 0;
 
-#define NAMECMP(a,b) (a&&b?strcasecmp(a,b):(a?-1:1))
+#define NAMECMP(a,b) (a&&b?strcmp(a,b):(a?-1:1))
 
   *error_code= MYX_NO_ERROR;
 
@@ -1463,10 +1524,10 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
 
         //Check DefaultValue
         //if there is no default value
-        if (STR_EMPTY(column->default_value))
+        if (STR_EMPTY(column->default_value) || column->default_value_is_null)
         {
           //and the default value is different than 0, "", 0-datetime and 0-date
-          if((ex_column->default_value) &&
+          if((ex_column->default_value || !column->default_value_is_null) &&
             (strcmp2(ex_column->default_value, "0") != 0) &&
             (strcmp2(ex_column->default_value, "") != 0) &&
             (strcmp2(ex_column->default_value, "0000-00-00 00:00:00") != 0) &&
@@ -1592,10 +1653,14 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
         {
           tmp2= myx_dbm_get_sql_column_create_code_no_pk(column, version);
 
-          if(name_changed)
+          if(name_changed) 
+          {
             tmp= g_strdup_printf("," _br " CHANGE COLUMN `%s` %s", column->original_name, tmp2);
+          }
           else
+          {
             tmp= g_strdup_printf("," _br " MODIFY COLUMN %s", tmp2);
+          }
 
           alter_spec= str_g_append(alter_spec, tmp);
 
@@ -1610,8 +1675,31 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
       char previous_column_name[256];
       strcpy(previous_column_name, "");
 
-      if(existing_table->columns_num > 0)
-        strcpy(previous_column_name, existing_table->columns[existing_table->columns_num-1].name);
+      /*
+      // check whether the name of the last existing column was changed
+      // and use the new name in the AFTER clause if so
+      for(j= existing_table->columns_num - 1; j >= 0; j--)
+      {
+        //j= existing_table->columns_num-1;
+        for(i= 0; i < altered_table->columns_num; i++)
+        {
+          if(NAMECMP(altered_table->columns[i].original_name, existing_table->columns[j].name) == 0)
+          {
+            strcpy(previous_column_name, altered_table->columns[i].name);
+            j= -1;
+            break;
+          }
+        }
+        if((int)j == -1) 
+        {
+          break;
+        }
+      }
+//      if(j != -1)
+//      {
+//        strcpy(previous_column_name, existing_table->columns[j].name);
+//      }
+    */
 
       //Add new columns
       for(i= 0; i < altered_table->columns_num; i++)
@@ -1632,42 +1720,66 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
         {
           tmp2= myx_dbm_get_sql_column_create_code(column, version);
 
-          if(existing_table->columns_num == 0)
+          //if(existing_table->columns_num == 0)
+          if(existing_table->columns_num == 0 || strcmp(previous_column_name,"") == 0)
             tmp= g_strdup_printf("," _br " ADD COLUMN %s FIRST", tmp2);
           else
             tmp= g_strdup_printf("," _br " ADD COLUMN %s AFTER `%s`", tmp2, previous_column_name);
 
-          strcpy(previous_column_name, column->name);
+          //strcpy(previous_column_name, column->name);
 
           alter_spec= str_g_append(alter_spec, tmp);
           g_free(tmp);
           g_free(tmp2);
         }
+        //++
+        strcpy(previous_column_name, column->name);
       }
     }
 
     //Check count of primary key cols
-    for(i= 0; i<altered_table->indices_num; i++)
+    if(altered_table->indices_num > 0)
     {
-      if(NAMECMP(altered_table->indices[i].name, "PRIMARY") == 0)
+      for(i= 0; i<altered_table->indices_num; i++)
       {
-        found= 0;
-        for(j= 0; j<existing_table->indices_num; j++)
+        if(NAMECMP(altered_table->indices[i].name, "PRIMARY") == 0)
         {
-          if(NAMECMP(existing_table->indices[j].name, "PRIMARY") == 0)
+          found= 0;
+          for(j= 0; j<existing_table->indices_num; j++)
           {
-            found= 1;
-            if(altered_table->indices[i].columns_num != existing_table->indices[j].columns_num)
-              new_pk= 1;
+            if(NAMECMP(existing_table->indices[j].name, "PRIMARY") == 0)
+            {
+              found= 1;
+              if(altered_table->indices[i].columns_num != existing_table->indices[j].columns_num)
+                new_pk= 1;
 
-            break;
+              break;
+            }
           }
-        }
 
-        if(!found)
+          if(!found)
+            new_pk= 1;
+            
+          break;
+        }
+      }
+    }
+    else  // altered table has no pks
+    {
+      for(i= 0; i<existing_table->indices_num; i++)
+      {
+        if(NAMECMP(existing_table->indices[i].name, "PRIMARY") == 0)
+        {
+          // existing table had PKs - need to DROP PRIMARY KEY (below)
           new_pk= 1;
-          
-        break;
+          // remote column attrs
+          for(i= 0; i<altered_table->columns_num; i++)
+          {
+            altered_table->columns[i].primary_key= 0;
+            //altered_table->columns[i].auto_inc= 0;
+          }
+          break;
+        }
       }
     }
 
@@ -1676,8 +1788,33 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
       if(ex_pk)
         alter_spec= str_g_append(alter_spec, "," _br " DROP PRIMARY KEY");
 
+      // use index instead of PK flag (bugfix for #7136)
       tmp3= NULL;
-      for(i= 0; i<altered_table->columns_num; i++)
+      for(i= 0; i<altered_table->indices_num; i++)
+      {
+        if(NAMECMP(altered_table->indices[i].name, "PRIMARY") == 0)
+        {
+          for(j= 0; j<altered_table->indices[i].columns_num; j++)
+          {
+            if(!tmp3)
+              tmp2= g_strdup_printf("`%s`", altered_table->indices[i].columns[j].name);
+            else
+              tmp2= g_strdup_printf(", `%s`", altered_table->indices[i].columns[j].name);
+
+            if (altered_table->indices[i].columns[j].len && altered_table->indices[i].columns[j].len[0])
+              tmp2= str_g_append_and_free(tmp2, g_strdup_printf("(%s)", altered_table->indices[i].columns[j].len));
+
+            tmp3= str_g_append(tmp3, tmp2);
+            g_free(tmp2);
+          }
+
+          if(!found)
+            new_pk= 1;
+            
+          break;
+        }
+      }
+      /*for(i= 0; i<altered_table->columns_num; i++)
       {
         if(altered_table->columns[i].primary_key)
         {
@@ -1688,7 +1825,7 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
           tmp3= str_g_append(tmp3, tmp2);
           g_free(tmp2);
         }
-      }
+      }*/
 
       if (tmp3)
       {
@@ -1802,7 +1939,7 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
       for(j= 0; j<altered_table->fks_num; j++)
         if(NAMECMP(ex_fk->name, altered_table->fks[j].name) == 0)
         {
-          fk= altered_table->fks+i;
+          fk= altered_table->fks+j;
           break;
         }
 
@@ -1883,13 +2020,13 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
     if(existing_table->table_engine != altered_table->table_engine)
       switch (altered_table->table_engine)
       {
-      case MYX_DBM_TE_INNODB: alter_options= str_g_append(alter_options, _br "TYPE = InnoDB"); break;
-      case MYX_DBM_TE_MEMORY: alter_options= str_g_append(alter_options, _br "TYPE = MEMORY"); break;
-      case MYX_DBM_TE_MERGE:  alter_options= str_g_append(alter_options, _br "TYPE = MERGE");  break;
-      case MYX_DBM_TE_BDB:    alter_options= str_g_append(alter_options, _br "TYPE = BDB");    break;
-      case MYX_DBM_TE_ISAM:   alter_options= str_g_append(alter_options, _br "TYPE = ISAM");   break;
-      case MYX_DBM_TE_NDB:    alter_options= str_g_append(alter_options, _br "TYPE = NDB");    break;
-      default:                alter_options= str_g_append(alter_options, _br "TYPE = MYISAM"); break;
+      case MYX_DBM_TE_INNODB: alter_options= str_g_append(alter_options, _br "ENGINE = InnoDB"); break;
+      case MYX_DBM_TE_MEMORY: alter_options= str_g_append(alter_options, _br "ENGINE = MEMORY"); break;
+      case MYX_DBM_TE_MERGE:  alter_options= str_g_append(alter_options, _br "ENGINE = MERGE");  break;
+      case MYX_DBM_TE_BDB:    alter_options= str_g_append(alter_options, _br "ENGINE = BDB");    break;
+      case MYX_DBM_TE_ISAM:   alter_options= str_g_append(alter_options, _br "ENGINE = ISAM");   break;
+      case MYX_DBM_TE_NDB:    alter_options= str_g_append(alter_options, _br "ENGINE = NDB");    break;
+      default:                alter_options= str_g_append(alter_options, _br "ENGINE = MYISAM"); break;
       }
 
     //auto_inc
@@ -1908,25 +2045,43 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
       alter_options= str_g_append_and_free(alter_options, g_strdup_printf(_br "DELAY_KEY_WRITE = %i",
                                                       altered_table->delay_key_write));
 
-    //Charset
+    //charset
     if((strcmp3(existing_table->charset, altered_table->charset)!=0) &&
       !STR_EMPTY(altered_table->charset))
     {
       alter_options= str_g_append_and_free(alter_options, g_strdup_printf(_br "CHARACTER SET %s",
                                                       altered_table->charset));
+
+      collation_added= 1;
     }
 
-    if ((strcmp3(existing_table->collation, altered_table->collation)!=0) &&
-      !STR_EMPTY(altered_table->collation))
-      if(((version->major_version >= 5) || ((version->major_version >= 4) && (version->minor_version >= 1))))
+    //collation
+    if ((strcmp3(existing_table->collation, altered_table->collation) != 0) &&
+      !STR_EMPTY(altered_table->collation) &&
+      ((version->major_version >= 5) || ((version->major_version >= 4) && (version->minor_version >= 1))))
+    {
+      // if the charset has not been added yet, add it now
+      if ((collation_added == 0) && !STR_EMPTY(altered_table->charset))
       {
-        if(!(strcmp2(existing_table->charset, altered_table->charset)!=0))
-          alter_options= str_g_append_and_free(alter_options, g_strdup_printf(_br "CHARACTER SET %s",
-                                                      altered_table->charset));
+        alter_options= str_g_append_and_free(alter_options, g_strdup_printf(_br "CHARACTER SET %s",
+                                                    altered_table->charset));
 
-        alter_options= str_g_append_and_free(alter_options, g_strdup_printf(" COLLATE %s", 
-                                                        altered_table->collation));
+        collation_added= 1;
       }
+      else if ((collation_added == 0) && STR_EMPTY(altered_table->charset))
+      {
+        alter_options= str_g_append_and_free(alter_options, g_strdup_printf(_br "CHARACTER SET Default"));
+
+        collation_added= 1;
+      }
+
+
+      if (collation_added == 1)
+      {
+        alter_options= str_g_append_and_free(alter_options, g_strdup_printf(" COLLATE %s", 
+                                                      altered_table->collation));
+      }
+    }
 
     //comment
     if(strcmp3(existing_table->comment, altered_table->comment)!=0)
@@ -2176,6 +2331,53 @@ MYX_DBM_STORED_PROCEDURE_DATA *myx_dbm_get_sp_data(MYSQL *mysql, const char *cat
   return spdata;
 }
 
+// return 0 when OK
+MYX_LIB_ERROR myx_dbm_drop_sp(MYSQL *mysql, const char *catalog_name, const char *schema_name, const char *sp_name, MYX_SCHEMA_STORED_PROCEDURE_TYPE sp_type)
+{
+  const char *q;
+  char *query;
+  
+  MYX_LIB_ERROR error;
+
+  if (check_exact_mysql_version(mysql, 5, 0, 0))
+  {
+    if (sp_type == MSPT_PROCEDURE)
+      q= "DROP PROCEDURE `%s`";
+    else 
+      q= "DROP FUNCTION `%s`";
+    
+    query= g_strdup_printf(q, sp_name);
+  }
+  else
+  {
+    if (sp_type == MSPT_PROCEDURE)
+      q= "DROP PROCEDURE `%s`.`%s`";
+    else 
+      q= "DROP FUNCTION `%s`.`%s`";
+    
+    query= g_strdup_printf(q, schema_name, sp_name);
+  }
+
+  myx_query_execute_direct(mysql, query, &error);
+  g_free(query);
+  
+  return error;
+}
+
+// return 0 when OK
+MYX_LIB_ERROR myx_dbm_drop_view(MYSQL *mysql, const char *catalog_name, const char *schema_name, const char *view_name)
+{
+  char *query;
+  MYX_LIB_ERROR error;
+
+  query= g_strdup_printf("DROP VIEW IF EXISTS `%s`.`%s`", schema_name, view_name);
+
+  myx_query_execute_direct(mysql, query, &error);
+  g_free(query);
+  
+  return error;
+}
+
 
 void myx_dbm_free_sp_data(MYX_DBM_STORED_PROCEDURE_DATA *sp_data)
 {
@@ -2210,6 +2412,15 @@ static const char * view_show_create[]=
 static const char ** view_show_create_end=
                  view_show_create + sizeof(view_show_create)/sizeof(char*);
 
+static const char * view_show_create_new[]=
+{
+  "View",       // 0 (col 1)
+  "Create View"// 1 (col 2)
+};
+static const char ** view_show_create_new_end=
+                 view_show_create_new + sizeof(view_show_create_new)/sizeof(char*);
+
+
 MYX_DBM_VIEW_DATA *myx_dbm_get_view_data(MYSQL *mysql, const char *catalog, const char *schema,
                                          const char *view)
 {
@@ -2235,8 +2446,12 @@ MYX_DBM_VIEW_DATA *myx_dbm_get_view_data(MYSQL *mysql, const char *catalog, cons
         num_fields= mysql_num_fields(res);
         fields= mysql_fetch_fields(res);
 
-        build_field_subst(view_show_create,view_show_create_end,
-                          fields,fields+num_fields,fi);
+        if (mysql_full_version_is_later_or_equal_than(mysql, 5, 0, 3))
+          build_field_subst(view_show_create_new,view_show_create_new_end,
+                            fields,fields+num_fields,fi);
+        else
+          build_field_subst(view_show_create,view_show_create_end,
+                            fields,fields+num_fields,fi);
 
         vdata= g_new0(MYX_DBM_VIEW_DATA, 1);
         vdata->catalog= g_strdup(catalog);
@@ -2313,7 +2528,7 @@ char *myx_dbm_make_script_from_sps(MYSQL *mysql, const char *catalog_name,
 
 
   script= g_strdup("DELIMITER $$\n\n"
-    "// -----------------------------------------------------------------------------\n\n");
+    "-- -----------------------------------------------------------------------------\n\n");
   // create a big script from the sorted list of SPs
   for (i= 0; i < spcount; i++)
   {
@@ -2322,10 +2537,10 @@ char *myx_dbm_make_script_from_sps(MYSQL *mysql, const char *catalog_name,
     sptype= spdata[i]->sp_type==MSPT_FUNCTION?"FUNCTION":"PROCEDURE";
     
     script= str_g_append_and_free(script,
-                                  g_strdup_printf("DROP %s IF EXISTS `%s`.`%s`$$\n", sptype, schema_name,
+                                  g_strdup_printf("DROP %s IF EXISTS `%s`.`%s` $$\n", sptype, schema_name,
                                                   spdata[i]->name));
-    script= str_g_append_and_free(script, g_strdup_printf("%s$$\n\n", spdata[i]->definition));
-    script= str_g_append(script, "// -----------------------------------------------------------------------------\n\n");
+    script= str_g_append_and_free(script, g_strdup_printf("%s $$\n\n", spdata[i]->definition));
+    script= str_g_append(script, "-- -----------------------------------------------------------------------------\n\n");
   }
 
   script= str_g_append(script, "DELIMITER ;");
@@ -2340,6 +2555,39 @@ char *myx_dbm_make_script_from_sps(MYSQL *mysql, const char *catalog_name,
   return script;
 }
 
+// create table t1 ...
+// will become
+// create table `schema`.`t1` 
+static char *add_schema_name_to_stmt(const char *unquoted_schema_name, 
+                                     const char *name, // entity name
+                                     const char *sql)  // sql stmt
+{
+  char *name_start= strstr(sql, name);
+  char *stmt_start;
+  char *retval;
+  char quote_char= '`';
+  
+  if(!name_start)
+  {
+    return g_strdup(sql);
+  }
+  if((name_start[-1] == '`') || (name_start[-1] == '"')) // quoted name
+  {
+    name_start -= 1;
+	quote_char= name_start[0];
+  }
+  if(name_start[-1] == '.') // already have schema name added?
+  {
+    return g_strdup(sql);
+  }
+  stmt_start= g_malloc((gulong) (name_start - sql) + 1);
+  strncpy(stmt_start, sql, name_start - sql);
+  stmt_start[name_start - sql]= '\0';
+  retval= g_strdup_printf("%s %c%s%c.%s", stmt_start, quote_char, unquoted_schema_name, quote_char, name_start);
+  g_free(stmt_start);
+  return retval;
+}
+
 static const char * show_create_table[]=
 {
   "Database",        // 0 (col 0)
@@ -2360,6 +2608,10 @@ char * myx_dbm_get_create_sql(MYSQL *mysql, const char *catalog_name, const char
   MYSQL_FIELD *fields;
   unsigned int num_fields;
   char *sql= NULL;
+  char *sql2= NULL;
+
+  if ((object_type == MYX_DBM_OT_SCHEMA) && (!mysql_version_is_later_or_equal_than(mysql, 4, 1)))
+    return g_strdup_printf("CREATE DATABASE `%s`", schema_name);
 
   if ((object_type != MYX_DBM_OT_PROCEDURE) && (object_type !=MYX_DBM_OT_FUNCTION))
   {
@@ -2415,6 +2667,7 @@ char * myx_dbm_get_create_sql(MYSQL *mysql, const char *catalog_name, const char
     myx_dbm_free_sp_data(sp_data);
   }
 
-  return sql;
+  sql2= add_schema_name_to_stmt(schema_name, name, sql);
+  g_free(sql);
+  return sql2;
 }
-
