@@ -187,7 +187,7 @@ static void csv_exporter_columns(MYX_TABLE_EXPORTER_INFO *self, const char *cons
   unsigned int i;
   for (i= 0; i < self->columns_num[0]; i++)
   {
-    if(self->columns[0][i].ctype == MYX_TE_COL_STRING)
+    if(self->columns[0][i].ctype == MYX_TE_COL_STRING || self->columns[0][i].ctype == MYX_TE_COL_DATE)
     {
       char *s;
       
@@ -512,7 +512,6 @@ static void xml_exporter_table_setup(MYX_TABLE_EXPORTER_INFO *self, unsigned int
 
 static void xml_exporter_table_header(MYX_TABLE_EXPORTER_INFO *self)
 {
-  unsigned int i;
   MYX_XML_EXPORTER_INFO* xi= (MYX_XML_EXPORTER_INFO*)self->priv;
 
   self->current_level++;
@@ -524,6 +523,10 @@ static void xml_exporter_table_header(MYX_TABLE_EXPORTER_INFO *self)
 
     xi->dtd_written= 1;
     
+	// this is a "simple DTD" solution of #11830
+	fprintf(file, "<!ELEMENT field (#PCDATA) >\n<!ATTLIST field name CDATA #REQUIRED >\n<!ELEMENT row (field)+ >\n<!ELEMENT ROOT (row)+ >");
+
+	/*	
     fprintf(file, "<!ELEMENT ROOT (%s)+>\n", self->option_values[1]);
 
     fprintf(file, "\t<!ELEMENT %s (", self->option_values[1]);
@@ -543,7 +546,9 @@ static void xml_exporter_table_header(MYX_TABLE_EXPORTER_INFO *self)
 
       g_free(s);
     }
+	*/
     fclose(file);
+	
   }
   
   if (self->current_level > 1)
@@ -703,7 +708,7 @@ static void excel_exporter_table_header(MYX_TABLE_EXPORTER_INFO *self)
 
   //Set width for text columns
   for(i= 0; i<self->columns_num[0]; i++)
-    if(self->columns[0][i].ctype == MYX_TE_COL_STRING)
+    if(self->columns[0][i].ctype == MYX_TE_COL_STRING || self->columns[0][i].ctype == MYX_TE_COL_DATE)
     {
       fprintf(self->file_stream,"   <Column ss:Index=\"%d\" ss:AutoFitWidth=\"0\" ss:Width=\"110\"/>", i+1);
     }
@@ -732,7 +737,8 @@ static void excel_exporter_columns(MYX_TABLE_EXPORTER_INFO *self, const char *co
     "String",//MYX_TE_COL_STRING
     "Number",//MYX_TE_COL_INTEGER
     "Number",//MYX_TE_COL_FLOAT
-    "Number"//MYX_TE_COL_BOOL
+    "Number"//MYX_TE_COL_BOOL,
+    "String",//MYX_TE_COL_DATE
   };
   
   fprintf(self->file_stream, "   <Row>\n");
@@ -741,9 +747,11 @@ static void excel_exporter_columns(MYX_TABLE_EXPORTER_INFO *self, const char *co
     if(text[i])
     {
       char *s= escape_xml_entities(text[i]);
+      if (s == NULL)
+        s = "";
 
-      fprintf(self->file_stream, "    <Cell><Data ss:Type=\"%s\">%s</Data></Cell>\n",
-              excel_types[self->columns[0][i].ctype], s);
+      fprintf(self->file_stream, "    <Cell><Data ss:Type=\"%s\">%s</Data></Cell>\n", 
+        excel_types[self->columns[0][i].ctype], s);
 
       g_free(s);
     }
@@ -768,6 +776,143 @@ static void excel_exporter_end(MYX_TABLE_EXPORTER_INFO *self)
 }
 
 
+// ------------------------------------------------------------
+// MacOS PropertyList exporter
+// ------------------------------------------------------------
+
+
+static MYX_TABLE_EXPORTER_INFO *plist_exporter_init();
+static void plist_exporter_free(MYX_TABLE_EXPORTER_INFO *info);
+static int plist_exporter_setup(MYX_TABLE_EXPORTER_INFO *self, const char *filename);
+static void plist_exporter_begin(MYX_TABLE_EXPORTER_INFO *self, const char *text);
+static void plist_exporter_table_setup(MYX_TABLE_EXPORTER_INFO *self, unsigned int columns_num, MYX_TE_COLUMN *columns);
+static void plist_exporter_table_header(MYX_TABLE_EXPORTER_INFO *self);
+static void plist_exporter_columns(MYX_TABLE_EXPORTER_INFO *self, const char *const*text);
+static void plist_exporter_table_footer(MYX_TABLE_EXPORTER_INFO *self);
+static void plist_exporter_end(MYX_TABLE_EXPORTER_INFO *self);
+
+static MYX_TE_OPTION plist_exporter_options[]= 
+{
+  {"Old style proplist format", "0", "Use the old, NEXTSTEP style property list file format.", MYX_TEOP_BOOL},
+  {NULL, NULL, NULL, 0}
+};
+
+
+MYX_TABLE_EXPORTER PLIST_TABLE_EXPORTER= 
+{
+  "PLIST",
+  "Generates a Property List File",
+  "PropertyList File",
+  "plist",
+  plist_exporter_init,
+  plist_exporter_free,
+  plist_exporter_setup,
+  plist_exporter_begin,
+  plist_exporter_table_setup,
+  plist_exporter_table_header,
+  plist_exporter_columns,
+  plist_exporter_table_footer,
+  plist_exporter_end,
+  
+  plist_exporter_options
+};
+
+
+static MYX_TABLE_EXPORTER_INFO *plist_exporter_init()
+{
+  MYX_TABLE_EXPORTER_INFO *info;
+  
+  info= make_exporter_info(&PLIST_TABLE_EXPORTER);
+    
+  return info;
+}
+
+
+static void plist_exporter_free(MYX_TABLE_EXPORTER_INFO *self)
+{
+  free_exporter_info(self);
+}
+
+
+static int plist_exporter_setup(MYX_TABLE_EXPORTER_INFO *self, const char *filename)
+{
+  self->file_stream= myx_fopen(filename, "w+");
+  if (!self->file_stream)
+    return -1;
+  
+  return 0;
+}
+
+
+static void plist_exporter_begin(MYX_TABLE_EXPORTER_INFO *self, const char *text)
+{
+  fprintf(self->file_stream, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
+  fprintf(self->file_stream, "<!DOCTYPE plist PUBLIC \"-//Apple Computer//DTD PLIST 1.0//EN\" \"http://www.apple.com/DTDs/PropertyList-1.0.dtd\">\n");
+  fprintf(self->file_stream, "<plist version=\"1.0\">\n");
+}
+
+
+static void plist_exporter_table_setup(MYX_TABLE_EXPORTER_INFO *self, unsigned int columns_num, MYX_TE_COLUMN *columns)
+{
+  add_tabledef(self, columns_num, columns);
+}
+
+
+static void plist_exporter_table_header(MYX_TABLE_EXPORTER_INFO *self)
+{
+  fprintf(self->file_stream, "<array>\n");
+}
+
+
+static void plist_exporter_columns(MYX_TABLE_EXPORTER_INFO *self, const char *const*text)
+{
+  unsigned int i;
+  MYX_TE_COLUMN *columns= self->columns[0];
+  
+  fprintf(self->file_stream, "    <dict>\n");
+  for (i= 0; i < self->columns_num[0]; i++)
+  {
+    if(text[i])
+    {
+      char *s= escape_xml_entities(text[i]);
+      fprintf(self->file_stream, "        <key>%s</key>\n", columns[i].name);      
+      switch (columns[i].ctype)
+      {
+        case MYX_TE_COL_STRING:   
+          fprintf(self->file_stream, "        <string>%s</string>\n", s); 
+          break;
+        case MYX_TE_COL_INTEGER:  
+          fprintf(self->file_stream, "        <integer>%s</integer>\n", s);
+          break;
+        case MYX_TE_COL_FLOAT:
+          fprintf(self->file_stream, "        <real>%s</real>\n", s);
+          break;
+        case MYX_TE_COL_BOOL:
+          fprintf(self->file_stream, "        <%s/>\n", 
+                  ((strcasecmp(s,"true")==0 || strcasecmp(s, "1")==0)) ? "true" : "false");
+          break;
+        case MYX_TE_COL_DATE:
+          fprintf(self->file_stream, "        <date>%s</date>\n", s);
+          break;
+      }      
+      g_free(s);
+    }
+  }
+  
+  fprintf(self->file_stream, "    </dict>\n");
+}
+
+
+static void plist_exporter_table_footer(MYX_TABLE_EXPORTER_INFO *self)
+{
+  fprintf(self->file_stream, "</array>\n");
+}
+
+
+static void plist_exporter_end(MYX_TABLE_EXPORTER_INFO *self)
+{
+  fprintf(self->file_stream, "</plist>\n");
+}
 
 // ============================================================
 
@@ -775,7 +920,8 @@ static MYX_TABLE_EXPORTER *exporters[]= {
   &CSV_TABLE_EXPORTER,
   &HTML_TABLE_EXPORTER,
   &XML_TABLE_EXPORTER,
-  &EXCEL_TABLE_EXPORTER
+  &EXCEL_TABLE_EXPORTER,
+  &PLIST_TABLE_EXPORTER
 };
 
 
@@ -838,3 +984,4 @@ MYX_STRINGLIST *myx_get_table_export_formats()
   return sl;
 }
 
+
