@@ -49,6 +49,7 @@ typedef struct {
   MYX_QUERY_CALLBACK pre_query_hook;
   MYX_QUERY_CALLBACK post_query_hook;
   void *client_data;
+  unsigned int resultset_memory_limit; // 0 for no limit, > 0 for xMB of limit
   short minor_version;
   short major_version;
   short patchlevel;
@@ -186,7 +187,17 @@ void oi_ansi_quotes(ADVANCED_OPTION_DESCRIPTION * opt,
                     const char * value,
                     MYSQL_REAL_CONNECT_PARAMETERS * options)
 {
-  options->ansi_quotes= value ? toupper(*value) == 'Y' : -1;
+  if (value)
+  {
+    switch (toupper(*value))
+    {
+      case 'Y': options->ansi_quotes= 1; break;
+      case 'N': options->ansi_quotes= 0; break;
+      default:  options->ansi_quotes= -1; break;
+    }
+  }
+  else
+    options->ansi_quotes= -1;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -350,6 +361,23 @@ MYX_MYSQL *myx_mysql_get_private(MYSQL *mysql)
 }
 
 
+unsigned int myx_mysql_get_resultset_size_limit(MYSQL *mysql)
+{
+  MYX_MYSQL *data= myx_mysql_get_private(mysql);
+  if (data)
+    return data->resultset_memory_limit;
+  return 0;
+}
+
+
+void myx_mysql_limit_resultset_size(MYSQL *mysql, unsigned int limitMB)
+{
+  MYX_MYSQL *data= myx_mysql_get_private(mysql);
+  if (data)
+    data->resultset_memory_limit= limitMB;
+}
+
+
 const char *server_args[] =
 {
   "app_dummy",
@@ -511,13 +539,12 @@ static void modify_sql_mode(MYSQL *mysql, int ansi_quotes)
       }
     }
 
-    query= g_strdup_printf("SET SESSION sql_mode='%s'", current_mode);
+    query= g_strdup_printf("SET SESSION sql_mode='%s'", current_mode ? current_mode : "");
     mysql_query(mysql, query);
     g_free(query);
   }
 }
 
-
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief prepare a connection to a MySQL server
     @ingroup bridge_to_MySQL_API
@@ -559,22 +586,10 @@ int myx_connect_to_instance(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
 #if !(defined(__WIN__) || defined(_WIN32) || defined(_WIN64))
   if (!connect_options.unix_socket)
   {
-    gchar *out= NULL, *err= NULL;
-//    gint est; can't request exist status because when it's being called
-//    in the GRT, it will have SIGCHLD caught by the JVM and g_spawn needs it
-    if (g_spawn_command_line_sync("mysql_config --socket",
-                                  &out, &err, NULL,
-                                  NULL) && out)
-    {
-      char *ptr= strchr(out, '\n');
-      if (ptr)
-      {
-        *ptr= 0;
-        unix_socket= g_strdup(out);
-      }
-    }
-    g_free(out);
-    g_free(err);
+    if (access("/tmp/mysql.sock", F_OK) == 0)
+      unix_socket= g_strdup("/tmp/mysql.sock");
+    else if (access("/var/lib/mysql/mysql.sock", F_OK) == 0)
+      unix_socket= g_strdup("/var/lib/mysql/mysql.sock");
   }
 #endif
   
@@ -582,11 +597,14 @@ int myx_connect_to_instance(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
   //of mysql_real_connect are in utf8
   //mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "utf8");
 
+#if MYSQL_VERSION_ID >= 40103
   if (myx_mysql_get_private(mysql)->embedded)
     mysql_options(mysql, MYSQL_OPT_USE_EMBEDDED_CONNECTION, (char *)1);
   else
     mysql_options(mysql, MYSQL_OPT_USE_REMOTE_CONNECTION, (char *)1);
+#endif
 
+  mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "utf8");
   if (!mysql_real_connect(mysql, user_conn->hostname, user_conn->username,
                           user_conn->password, user_conn->schema,
                           user_conn->port, 
@@ -873,7 +891,7 @@ void myx_get_mysql_version(MYSQL *mysql)
     const char *version;
     int major= 0, minor= 0, pl= 0;
     version= mysql_get_server_info(mysql);
-    if ( sscanf(version, "%i.%i.%i%*s", &major, &minor, &pl) != 3)
+    if ( version && sscanf(version, "%i.%i.%i%*s", &major, &minor, &pl) < 3)
     {
     }
     priv->major_version= major;
@@ -920,6 +938,25 @@ int myx_get_mysql_minor_version(MYSQL *mysql)
   return priv->minor_version;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+/** @brief get release number of mysql server
+    @ingroup bridge_to_MySQL_API
+
+    @param mysql connection handler
+    @return release number of connection
+
+    calls myx_get_mysql_version() and returns minor_version
+*//////////////////////////////////////////////////////////////////////////////
+MYX_PUBLIC_FUNC int myx_get_mysql_release(MYSQL *mysql)
+{
+  MYX_MYSQL *priv;
+  myx_get_mysql_version(mysql);
+  
+  priv= myx_mysql_get_private(mysql);
+
+  return priv->patchlevel;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief get information about the server machine
     @ingroup Host_information_management
