@@ -44,7 +44,7 @@ static void free_option_group_content(MYX_OPTION_GROUP *option_group);
 
 ///////////////////////////////////////////////////////////////////////////////
 /** @ brief loads application options from an XML file
-    @param filename path to a xml-file with stored application options
+    @param filename Path to an xml file containing the application options (utf-8 encoded).
 
     @param error_code returned error code, <BR>
            possible values are:
@@ -70,15 +70,22 @@ MYX_APPLICATION_OPTIONS* myx_get_application_options(char *filename,
   MYX_APPLICATION_OPTIONS *app_options;
   MYX_OPTION_GROUP * group;
 
+  char* local_filename;
   *error_code= MYX_NO_ERROR;
 
-  if (! file_exists(filename) )
+  if (!(local_filename= g_filename_from_utf8(filename, -1, NULL, NULL, NULL)))
+  {
+    *error_code= MYX_CHARSET_CONVERSION_ERROR;
+    return NULL;
+  };
+
+  if (!file_exists(local_filename))
   {
     *error_code= MYX_ERROR_CANT_OPEN_FILE;
     return NULL;
   }
 
-  if(file_size(filename) > MAX_VALID_OPTION_FILE_SIZE)
+  if(file_size(local_filename) > MAX_VALID_OPTION_FILE_SIZE)
   {
     *error_code= MYX_CANT_READ_FROM_FILE;
     return NULL;
@@ -111,7 +118,7 @@ MYX_APPLICATION_OPTIONS* myx_get_application_options(char *filename,
   app_options= g_malloc(sizeof(MYX_APPLICATION_OPTIONS));
 
   /*count the number of group_elements*/
-  app_options->option_groups_num= get_child_count(root,"group");
+  app_options->option_groups_num= get_child_count(root,(xmlChar*)"group");
 
   app_options->option_groups= 
     app_options->option_groups_num==0 ? NULL :
@@ -121,10 +128,10 @@ MYX_APPLICATION_OPTIONS* myx_get_application_options(char *filename,
   for (group= app_options->option_groups, cur= root->children; cur != NULL;
        cur= cur->next)
   {
-    if (xmlStrcmp(cur->name, "group"))
+    if (xmlStrcmp(cur->name, (xmlChar*)"group"))
       continue;
 
-    group->name= xmlGetProp(cur, "name");
+    group->name= (char*)xmlGetProp(cur, (xmlChar*)"name");
     get_name_value_pairs(cur, group);
     group++;
   }
@@ -150,22 +157,22 @@ int myx_store_application_options(MYX_APPLICATION_OPTIONS *options,
 
   if (options == NULL || filename == NULL) return -1;
 
-  doc= xmlNewDoc("1.0");
+  doc= xmlNewDoc((xmlChar*)"1.0");
 
   root_node= doc->children= xmlNewDocRawNode(doc,NULL,
-                                             "application_options", NULL);
+                                             (xmlChar*)"application_options", NULL);
 
   groups_end= options->option_groups + options->option_groups_num;
   for (group= options->option_groups; group!=groups_end; group++)
   {
-    group_node= xmlNewTextChild(root_node, NULL, "group", NULL);
-    xmlNewProp(group_node, "name", group->name);
+    group_node= xmlNewTextChild(root_node, NULL, (xmlChar*)"group", NULL);
+    xmlNewProp(group_node, (xmlChar*)"name", (xmlChar*)group->name);
     pairs_end= group->name_value_pairs + group->name_value_pairs_num;
     for (pair= group->name_value_pairs; pair!=pairs_end; pair++)
     {
-      name_value_node= xmlNewTextChild(group_node, NULL, "property", NULL);
-      xmlNewProp(name_value_node, "name", pair->name);
-      xmlNewProp(name_value_node, "value", pair->value);
+      name_value_node= xmlNewTextChild(group_node, NULL, (xmlChar*)"property", NULL);
+      xmlNewProp(name_value_node, (xmlChar*)"name", (xmlChar*)pair->name);
+      xmlNewProp(name_value_node, (xmlChar*)"value", (xmlChar*)pair->value);
     }
   }
   
@@ -234,7 +241,7 @@ static void get_name_value_pairs(xmlNodePtr node,
   MYX_NAME_VALUE_PAIR * pair;
 
   /*count the number of name-value pairs*/
-  option_group->name_value_pairs_num= get_child_count(node,"property");
+  option_group->name_value_pairs_num= get_child_count(node,(xmlChar*)"property");
 
   option_group->name_value_pairs = 
     option_group->name_value_pairs_num <= 0 ? NULL :
@@ -244,11 +251,11 @@ static void get_name_value_pairs(xmlNodePtr node,
   for (pair= option_group->name_value_pairs, cur= node->children; cur;
        cur= cur->next)
   {
-    if (xmlStrcmp(cur->name, "property") ) /*ignore wrong elements*/
+    if (xmlStrcmp(cur->name, (xmlChar*)"property") ) /*ignore wrong elements*/
       continue;
 
-    pair->name= xmlGetProp(cur, "name");
-    pair->value= xmlGetProp(cur, "value");
+    pair->name= (char*)xmlGetProp(cur, (xmlChar*)"name");
+    pair->value= (char*)xmlGetProp(cur, (xmlChar*)"value");
     pair++;
   }
 }
