@@ -39,15 +39,25 @@ static xmlXPathContextPtr context; /* used by all XPath operations*/
 
 // forward declarations
 static xmlChar *obscure(const xmlChar* str);
-static xmlChar *retrieve_password(xmlChar *password,
+static xmlChar *retrieve_password(MYX_USER_CONNECTION *conn, xmlChar *password,
                                   enum myx_password_storage_type storage_type);
-static xmlChar* save_password(xmlChar *password,
+static xmlChar* save_password(MYX_USER_CONNECTION *conn, xmlChar *password,
                               enum myx_password_storage_type storage_type);
 static 
   void parse_user_connection(xmlDocPtr doc, xmlNodePtr user_conn,
                              MYX_USER_CONNECTION *out,
                              enum myx_password_storage_type pwd_storage_type);
 
+
+// needed because we want to avoid calling OSX framework stuff from here
+// (thus avoiding the extra linkage)
+static char*(*os_password_store_function)(const char *host,
+                                          const char *username,
+                                          const char *password)= NULL;
+static char*(*os_password_retrieve_function)(const char *host,
+                                             const char *username,
+                                             const char *password_data)= NULL;
+
 /*
  * Public functions
  */
@@ -62,7 +72,7 @@ static
 *//////////////////////////////////////////////////////////////////////////////
 int get_xpath_int_value(xmlDocPtr doc, const char *xpath)
 {
-  xmlXPathObjectPtr result= xmlXPathEval(xpath, context);
+  xmlXPathObjectPtr result= xmlXPathEval((xmlChar*)xpath, context);
   xmlNodeSetPtr node_set= result->nodesetval;
   int res= atoi_and_free(xmlNodeListGetString(doc,
                                               node_set->nodeTab[0]->children,
@@ -87,7 +97,7 @@ MYX_USER_CONNECTIONS * new_user_connections()
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-/** @brief load predefined connection stored in xml-file
+/** @brief load predefined connection stored in xml-file (utf-8 encoded)
     @ingroup Connection_management
 
     @param filename path to xml-file with stored connections
@@ -119,13 +129,20 @@ MYX_USER_CONNECTIONS* myx_load_user_connections(const char *filename,
   xmlNodeSetPtr node_set;
   xmlXPathObjectPtr result;
   enum myx_password_storage_type pwd_storage_type;
+  char* local_filename;
 
   *error_code= MYX_NO_ERROR;
 
-  if (! file_exists(filename) )
+  if (!(local_filename= g_filename_from_utf8(filename, -1, NULL, NULL, NULL)))
+  {
+    *error_code= MYX_CHARSET_CONVERSION_ERROR;
+    return NULL;
+  };
+
+  if (!file_exists(local_filename))
     return new_user_connections();
 
-  if ( !(doc= myx_xmlParseFile(filename)) )
+  if (!(doc = myx_xmlParseFile(filename)))
   {
     *error_code= MYX_XML_PARSE_ERROR;
     return NULL;
@@ -155,7 +172,7 @@ MYX_USER_CONNECTIONS* myx_load_user_connections(const char *filename,
     get_xpath_int_value(doc,"/user_connections/password_storage_type");
 
   /*retrieve the user_connections*/
-  result= xmlXPathEval("/user_connections/user_connection", context);
+  result= xmlXPathEval((xmlChar*)"/user_connections/user_connection", context);
   node_set= result->nodesetval;
 
   count= node_set->nodeNr;
@@ -204,13 +221,13 @@ int myx_store_user_connections(MYX_USER_CONNECTIONS* user_connections,
 
   if (user_connections == NULL) return -1;
 
-  doc= xmlNewDoc("1.0");
+  doc= xmlNewDoc((xmlChar*)"1.0");
   root_node= doc->children= xmlNewDocRawNode(doc,NULL,
-                                             "user_connections", NULL);
+                                             (xmlChar*)"user_connections", NULL);
 
-  NewTextChild_int_content(root_node,NULL,"last_connection",
+  NewTextChild_int_content(root_node,NULL,(xmlChar*)"last_connection",
                            user_connections->last_connection);
-  NewTextChild_int_content(root_node,NULL,"password_storage_type",
+  NewTextChild_int_content(root_node,NULL,(xmlChar*)"password_storage_type",
                            pwd_storage_type);
 
   ucon= user_connections->user_connections;
@@ -219,28 +236,28 @@ int myx_store_user_connections(MYX_USER_CONNECTIONS* user_connections,
   {
     char *tmp;
 
-    ucon_node= xmlNewTextChild(root_node, NULL, "user_connection", NULL);
-#define STRNULL(s) (s)?(s):""
-    xmlNewTextChild(ucon_node, NULL, "connection_name",
-                    STRNULL(ucon->connection_name));
-    xmlNewTextChild(ucon_node, NULL, "username", ucon->username);
-    xmlNewTextChild(ucon_node, NULL, "hostname", ucon->hostname);
-    NewTextChild_int_content(ucon_node, NULL, "port", ucon->port);
-    xmlNewTextChild(ucon_node, NULL, "schema", ucon->schema);
+    ucon_node= xmlNewTextChild(root_node, NULL, (xmlChar*)"user_connection", NULL);
+#define STRNULL(s) ((s)?(s):"")
+    xmlNewTextChild(ucon_node, NULL, (xmlChar*)"connection_name",
+                    (xmlChar*)STRNULL(ucon->connection_name));
+    xmlNewTextChild(ucon_node, NULL, (xmlChar*)"username", (xmlChar*)ucon->username);
+    xmlNewTextChild(ucon_node, NULL, (xmlChar*)"hostname", (xmlChar*)ucon->hostname);
+    NewTextChild_int_content(ucon_node, NULL, (xmlChar*)"port", ucon->port);
+    xmlNewTextChild(ucon_node, NULL, (xmlChar*)"schema", (xmlChar*)ucon->schema);
 
-    node= xmlNewTextChild(ucon_node, NULL, "advanced_options", NULL);
+    node= xmlNewTextChild(ucon_node, NULL, (xmlChar*)"advanced_options", NULL);
     for (j=0; j < (int)ucon->advanced_options_num; j++)
-      xmlNewTextChild(node, NULL, "advanced_option",ucon->advanced_options[j]);
+      xmlNewTextChild(node, NULL, (xmlChar*)"advanced_option",(xmlChar*)ucon->advanced_options[j]);
 
-    xmlNewTextChild(ucon_node,NULL,"storage_path",STRNULL(ucon->storage_path));
-    xmlNewTextChild(ucon_node,NULL,"notes", STRNULL(ucon->notes));
+    xmlNewTextChild(ucon_node,NULL,(xmlChar*)"storage_path",(xmlChar*)STRNULL(ucon->storage_path));
+    xmlNewTextChild(ucon_node,NULL,(xmlChar*)"notes", (xmlChar*)STRNULL(ucon->notes));
 #undef STRNULL
-    NewTextChild_int_content(ucon_node, NULL, "connection_type",
+    NewTextChild_int_content(ucon_node, NULL, (xmlChar*)"connection_type",
                              ucon->connection_type);
-    NewTextChild_int_content(ucon_node,NULL,"storage_type",ucon->storage_type);
+    NewTextChild_int_content(ucon_node,NULL,(xmlChar*)"storage_type",ucon->storage_type);
 
-    tmp= save_password(ucon->password, pwd_storage_type);
-    xmlNewTextChild(ucon_node, NULL, "password", tmp);
+    tmp= (char*)save_password(ucon, (unsigned char*)ucon->password, pwd_storage_type);
+    xmlNewTextChild(ucon_node, NULL, (xmlChar*)"password", (xmlChar*)tmp);
     g_free(tmp);
   }
 
@@ -394,7 +411,7 @@ static xmlChar *obscure(const xmlChar* str)
   }
   else
   {
-    gulong len= (gulong)strlen(str);
+    gulong len= (gulong)strlen((char*)str);
     xmlChar *result= (xmlChar*)g_malloc(len+1);
     xmlChar *res_pos= result;
     xmlChar *res_end= result + len;
@@ -405,6 +422,19 @@ static xmlChar *obscure(const xmlChar* str)
   }
 }
 
+
+void set_os_specific_password_functions(char*(*store_func)(const char *host,
+                                                           const char *username,
+                                                           const char *password),
+                                        char*(*retrieve_func)(const char *host,
+                                                              const char *username,
+                                                              const char *password_data))
+{
+  os_password_store_function= store_func;
+  os_password_retrieve_function= retrieve_func;
+}
+
+
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief Crypt uprotected password
@@ -490,12 +520,12 @@ static xmlChar* win_store_password(xmlChar *password)
 
     calls hex_decode() and obscure() to retrieve encrypted password <BR>
 *//////////////////////////////////////////////////////////////////////////////
-static xmlChar *retrieve_password(xmlChar *password,
+static xmlChar *retrieve_password(MYX_USER_CONNECTION *conn, xmlChar *password,
                                   enum myx_password_storage_type storage_type)
 {
   xmlChar *tmp, *tmp2, *plaintext_password;
 
-  if (!password) return NULL;
+  if (!password && storage_type != MYX_PASSWORD_OS_SPECIFIC) return NULL;
 
   switch (storage_type)
   {
@@ -506,11 +536,11 @@ static xmlChar *retrieve_password(xmlChar *password,
     {
       //since the old hex_encode function produced a single 0 at
       //the end of the str (instead of two), remove it
-      unsigned int password_len= (unsigned int)strlen(password);
+      unsigned int password_len= (unsigned int)strlen((char*)password);
       if (password_len % 2 != 0 && password[password_len-1] == '0')
         password[password_len-1]= 0;
     }
-    tmp= hex_decode(password, NULL);
+    tmp= (xmlChar*)hex_decode((char*)password, NULL);
     tmp2= obscure(tmp);
     plaintext_password= xmlStrdup(tmp2);
     g_free(tmp);
@@ -521,15 +551,26 @@ static xmlChar *retrieve_password(xmlChar *password,
 
   case MYX_PASSWORD_OS_SPECIFIC:
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
+    if (!password) return NULL;
     tmp= win_load_password(password);
     plaintext_password= xmlStrdup(tmp);
     g_free(tmp);
     return plaintext_password;
+#else
+    if (os_password_retrieve_function)
+    {
+      xmlChar *tmp;
+      char *inst;
+      inst= g_strdup_printf("%s:%i", conn->hostname, conn->port);
+      tmp= (xmlChar*)(*os_password_retrieve_function)(inst, conn->username, (char*)password);
+      g_free(inst);
+      return tmp;
+    }
 #endif
     break;
   }
 
-  return password;
+  return xmlStrdup(password);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -540,7 +581,7 @@ static xmlChar *retrieve_password(xmlChar *password,
 
     calls obscure() and hex_encode() to encrypt password
 *//////////////////////////////////////////////////////////////////////////////
-static xmlChar* save_password(xmlChar *password,
+static xmlChar* save_password(MYX_USER_CONNECTION *conn, xmlChar *password,
                               enum myx_password_storage_type storage_type)
 {
   if (!password || !(*password)) return NULL;
@@ -548,18 +589,28 @@ static xmlChar* save_password(xmlChar *password,
   switch (storage_type)
   {
   case MYX_PASSWORD_NOT_STORED: return NULL;     break;
-  case MYX_PASSWORD_PLAINTEXT:  return g_strdup(password); break;
+  case MYX_PASSWORD_PLAINTEXT:  return xmlStrdup(password); break;
 
   case MYX_PASSWORD_OBSCURED:
     {
       xmlChar *tmp= obscure(password);
-      xmlChar *encrypted_password= hex_encode(tmp,-1);
+      xmlChar *encrypted_password= (xmlChar*)hex_encode((char*)tmp,-1);
       g_free(tmp);
       return encrypted_password;
     }
   case MYX_PASSWORD_OS_SPECIFIC:
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
     return win_store_password(password);
+#else
+    if (os_password_store_function)
+    {
+      xmlChar *tmp;
+      char *inst;
+      inst= g_strdup_printf("%s:%i", conn->hostname, conn->port);
+      tmp= (xmlChar*)(*os_password_store_function)(inst, conn->username, (char*)password);
+      g_free(inst);
+      return tmp;
+    }
 #endif
     break;
   }
@@ -587,7 +638,7 @@ static
   xmlXPathObjectPtr result;
   xmlNodeSetPtr node_set;
 
-  assert( !xmlStrcmp(user_conn->name, "user_connection") );
+  assert( !xmlStrcmp(user_conn->name, (xmlChar*)"user_connection") );
   memset(out, 0, sizeof(MYX_USER_CONNECTION));
   for (cur= user_conn->children; cur != NULL; cur= cur->next)
   {
@@ -603,19 +654,19 @@ static
                          (int*)&out->connection_type);
     try_to_get_int_field(doc,cur,"storage_type",   (int*)&out->storage_type);
 
-    if ( !xmlStrcmp(cur->name, "password") )
+    if ( !xmlStrcmp(cur->name, (xmlChar*)"password") )
     {
       xmlChar *tmp;
       if (out->password)
         xmlFree(out->password);
       tmp= xmlNodeListGetString(doc, cur->children,1);
-      out->password= retrieve_password(tmp, pwd_storage_type);
+      out->password= (char*)retrieve_password(out, tmp, pwd_storage_type);
       xmlFree(tmp);
     }
   }
 
   context->node= user_conn;
-  result= xmlXPathEval("advanced_options/advanced_option", context);
+  result= xmlXPathEval((xmlChar*)"advanced_options/advanced_option", context);
   node_set= result->nodesetval;
   out->advanced_options_num= !node_set ? 0 : node_set->nodeNr;
   if (out->advanced_options_num == 0)
@@ -627,7 +678,7 @@ static
     out->advanced_options= (char**)g_malloc(sizeof(char*) * node_set->nodeNr);
     for (i=0; i< (int)out->advanced_options_num; i++)
     {
-      out->advanced_options[i]=
+      out->advanced_options[i]= (char*)
         xmlNodeListGetString(doc, node_set->nodeTab[i]->children, 1);
     }
   }
