@@ -23,11 +23,10 @@
 #ifndef __GC_CANVAS_H__
 #define __GC_CANVAS_H__
 
-#include "myx_gc_base.h"
-
-#include "myx_gc_utilities.h"
-#include "myx_gc_layer.h"
 #include "myx_gc_view.h"
+#include "myx_gc_font_manager.h"
+#include "myx_gc_svgparser.h"
+#include "myx_gc_figure_parser.h"
 
 //----------------------------------------------------------------------------------------------------------------------
 
@@ -36,44 +35,13 @@ class CFigureInstance;
 class CLayer;
 class CGenericCanvas;
 class CGCView;
-
-//----------------- CFigureInstanceEnumerator --------------------------------------------------------------------------
-
-/**
- * The CFigureInstanceEnumerator class is for quick access to all figure instances on all (common) layers.
- * Enumeration happens depth-first. That means for each layer first all instances are enumerated before the next layer
- * is taken.
- */
-class GENERIC_CANVAS_API CFigureInstanceEnumerator
-{
-private:
-  CGenericCanvas* FCanvas;
-  CLayers::iterator FLayerIterator;
-  CFigureInstances::iterator FFigureInstanceIterator;
-public:
-  CFigureInstanceEnumerator(CGenericCanvas* Canvas);
-
-  virtual bool __cdecl hasNext(void);
-  virtual CFigureInstance* __cdecl next(void);
-  virtual void __cdecl release(void);
-  virtual void __cdecl reset(void);
-};
+class CConnection;
 
 //----------------- CGenericCanvas -------------------------------------------------------------------------------------
 
+/** States the canvas can enter. */
 #define GC_STATE_PENDING_ACTIVATION 0x0001
 
-class CCanvasListener: public CGCListener
-{
-  friend class CGenericCanvas;
-protected:
-  CGenericCanvas* canvas;
-public:
-  virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason);
-  virtual void __cdecl onDestroy(CGCBase* object);
-  virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* message);
-};
-
 /**
  * CGenericCanvas is the main class of the library and is the base for all further functionality (e.g. it creates and 
  * maintains the model). Instances are created via the exported CreateGenericCanvas function (if called from non C++ 
@@ -88,67 +56,75 @@ class GENERIC_CANVAS_API CGenericCanvas: public CGCBase
   friend class CGCBase;
   friend class CGCModel;
   friend class CFeedbackLayer;
-  friend class CFigureInstanceEnumerator;
 private:
+  class CCanvasListener: public CGCListener
+  {
+    friend class CGenericCanvas;
+  protected:
+    CGenericCanvas* canvas;
+  public:
+    virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TActionType& actionType);
+    virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason);
+    virtual void __cdecl onDestroy(CGCBase* object);
+    virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* message);
+  };
+
   wstring FName;
-  GCContext FContext;                  // The always active OpenGL rendering cntext.
+  GCContext FContext;                  // The OpenGL rendering cntext (platform specific).
   CGCModel* FModel;                    // The model this canvas is controlling.
   CLayers FLayers;                     // A list of layers currently in this canvas.
-  CFeedbackLayer* FFeedbackLayer;      // The selection layer is special.
   CViews FViews;                       // A list of views that can be used by this canvas.
   CGCView* FCurrentView;               // The currently active set of layers.
   unsigned int FStates;               // Persistent storage for certain states in the canvas.
   CCanvasListener FListener;           // Listener used to have the canvas listen to changes and error messages from children.
+#ifdef _WINDOWS
+  CRITICAL_SECTION FLock;
+#else
   GStaticRecMutex FLock;               // A lock to synchronize access to canvas objects.
+#endif
+  TOglExtensions FExtensions;          // A list of the support state of certain OpenGL extensions.
 protected:
-  void clearBuffers();
+  void clearBuffers(void);
+  void determineExtensions(void);
   void lock(void);
   void unlock(void);
 public:
-  CGenericCanvas(GCContext Context, wstring Name);
+  CGenericCanvas(GCContext Context, wstring name);
   virtual ~CGenericCanvas(void);
 
-  virtual void __cdecl addLayer(CLayer* Layer);
-  virtual TGCError __cdecl addLayoutsFromFile(const char* FileName);
-  virtual TGCError __cdecl addStylesFromFile(const char* FileName);
-  virtual void __cdecl addToSelection(CFigureInstance* Instance);
+  virtual void __cdecl addLayer(CLayer* layer);
+  virtual TGCError __cdecl addLayoutsFromFile(const char* filename);
+  virtual TGCError __cdecl addStylesFromFile(const char* filename, const char* variables);
+  TGCError addStylesFromFile(const char* filename, map<string, string>& variables);
+  virtual TGCError __cdecl addStyleFromDefinition(const char* definition);
   virtual void __cdecl checkError(void);
   virtual void __cdecl clearContent(void);
   virtual void __cdecl clearLayouts(void);
-  virtual void __cdecl clearSelection(void);
   virtual void __cdecl clearStyles(void);
+  virtual CConnection* __cdecl createConnection(CFigure* endPoint1, CFigure* endPoint2);
   virtual CFigure* __cdecl createFigure(const char* type, const char* class_);
-  virtual CLayer* __cdecl createLayer(const char* Name, bool AddToCurrentView);
-  virtual CGCView* __cdecl createView(const char* Name);
+  virtual CLayer* __cdecl createLayer(const char* name, bool addToCurrentView);
+  virtual CGCView* __cdecl createView(const char* name);
   virtual void __cdecl currentView(CGCView* View);
   virtual CGCView* __cdecl currentView(void);
-  virtual bool __cdecl doAction(CFigureInstance* Instance, const float X, const float Y);
-  virtual CFigureInstanceEnumerator* __cdecl getFigureInstanceEnumerator(void);
-  CGCModel* getModel(void) { return FModel; }; // For special use only. Speaking MVC pattern, the view should not access the model!
-  virtual TGCDirection __cdecl getSelectionInfo(const float X, const float Y);
-  virtual void __cdecl invalidateSelectionBounds(CFigureInstance* Instance);
-  virtual CLayer* __cdecl layerByName(const char* Name);
+  CGCModel* getModel(void) { return FModel; }; // For special use only. Speaking MVC pattern, the viewer should not access the model!
+  virtual CLayer* __cdecl layerByName(const char* name);
   virtual TGCVariant __cdecl property(const char* name, unsigned int index);
-  virtual void __cdecl property(const char* name, unsigned int index, const TGCVariant& value);
+  virtual void __cdecl property(const char* name, unsigned int index, TGCVariant value);
   virtual void __cdecl refresh(void);
-  virtual void __cdecl removeFromSelection(CFigureInstance* Instance);
-  virtual void __cdecl removeLayer(CLayer* Layer);
+  virtual void __cdecl removeLayer(CLayer* layer);
+  virtual void __cdecl removeStyle(const char* name);
   virtual void __cdecl removeView(CGCView* View);
   virtual void __cdecl render(void);
-  virtual void __cdecl resizeFiguresStart(int X, int Y, TGCDirection Direction);
-  virtual void __cdecl resizeFiguresStop(void);
-  virtual void __cdecl resizeFiguresTo(int X, int Y);
-  virtual void __cdecl rubberbandResize(int X, int Y, TRBSelectionAction Action);
-  virtual void __cdecl rubberbandStart(TRubberbandStyle Style, int X, int Y, bool ClearSelection);
-  virtual void __cdecl rubberbandStop(void);
-  virtual void __cdecl showSelection(bool Visible);
-  virtual CGCView* __cdecl viewByName(const char* Name);
+  virtual void __cdecl setTexturePath(const char* path);
+  bool supportsExtension(TOglExtension extension);
+  virtual CGCView* __cdecl viewByName(const char* name);
 };
 
 //----------------------------------------------------------------------------------------------------------------------
 
 // Factory function to create a generic canvas. This function is exported and must be used by the viewer implementations
 // to actually create a canvas instance. This is the only way to get hold of a generic canvas instance for non-C++ languages.
-extern "C" GENERIC_CANVAS_API CGenericCanvas* CreateGenericCanvas(GCContext Context, char* Name);
+extern "C" GENERIC_CANVAS_API CGenericCanvas* CreateGenericCanvas(GCContext Context, char* name);
 
 #endif // __GC_CANVAS_H__
