@@ -26,36 +26,24 @@
 #include "myx_gc_style.h"
 #include "myx_gc_layer.h"
 #include "myx_gc_layout.h"
+#include "myx_gc_font_manager.h"
 
 //----------------------------------------------------------------------------------------------------------------------
 
-class CGenericCanvas;
+class CGCView;
 class CGCModel;
 class CLayer;
 class CFigureInstance;
 class CLayouter;
+class CFigureController;
 
 //----------------- Figure templates -----------------------------------------------------------------------------------
 
-const string DefaultLayout = "column";      // The layout to be used for figure elements without any given layout.
-const string DefaultResize = "false";       // By default figure elements cannot be resized.
-
 class CFigureElementTemplate;
 class CCaptionElementTemplate;
 class CFigureElement;
 class CCaptionElement;
 
-class CStyleListener: public CGCListener
-{
-  friend class CFigureElementTemplate;
-protected:
-  CFigureElementTemplate* template_;
-public:
-  virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason);
-  virtual void __cdecl onDestroy(CGCBase* sender);
-  virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* message);
-};
-
 /** 
  * A figure element is one detail in a figure template and so also in a figure. There can be a hierarchy of figure
  * elements to form complex figures.
@@ -64,9 +52,22 @@ class CFigureElementTemplate
 {
   friend class CFigureElement;
 private:
+  class CStyleListener: private CGCListener
+  {
+    friend class CFigureElementTemplate;
+  protected:
+    CFigureElementTemplate* template_;
+  public:
+    virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TActionType& action) { };
+    virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason);
+    virtual void __cdecl onDestroy(CGCBase* sender);
+    virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* message);
+  };
+
+  wstring FId;                         // The XML element id.
   wstring FKey;                        // An identifier that can be used to search for this template.
   TFigureElementLayout FLayout;        // How are the element's children laid out.
-  bool FResizable;                     // Indicates whether this element can be resized.
+  TFigureElementResize FResizeMode;    // Indicates how this element can be resized.
   TBoundingBox FBoundingBox;           // The box that includes this and all child elements.
   CElementTemplateList FChildren;      // A list of child elements attached to this element.
   CGCStyle* FStyle;                    // The style to be used for this element.
@@ -76,18 +77,20 @@ private:
   CCaptionElementTemplate* FCaption;   // An optional caption for this element.
   TConstraints FConstraints;           // Size constraints for the element. A value of -1 means no constraint.
   TOccurence FOccurence;               // Number of times this element may occur.
-  CStyleListener FListener;
+  CStyleListener FListener;            // A listener for changes in connected elements.
+  CActions FActions;                   // Associated actions for the element.
 protected:
   void computeBoundingBox(void);
 public:
-  CFigureElementTemplate(wstring key);
+  CFigureElementTemplate(wstring id, wstring key);
   virtual ~CFigureElementTemplate(void);
 
+  void addAction(const TAction& action);
   void addChild(CFigureElementTemplate* Child);
   void freeNotification(CGCBase* object);
   CFigureElementTemplate* getListElement(void);
-  void initialize(TFigureElementLayout Layout, bool Resizable, CGCStyle* style, const TConstraints& Constraints,
-    TOccurence Occurence);
+  void initialize(TFigureElementLayout Layout, TFigureElementResize resizeMode, CGCStyle* style, 
+    const TConstraints& Constraints, TOccurence Occurence);
   bool isList(void);
   wstring key(void) { return FKey; };
   TOccurence occurence(void) { return FOccurence; };
@@ -123,6 +126,7 @@ private:
   CFigureElementTemplate* FParent;     // The parent element of this element (if at all).
   GLfloat FColor[4];                   // The text color.
   bool FHasColor;                      // True, if this caption has an own color.
+  bool FWrapText;
   TConstraints FConstraints;           // Size constraints for the element. A value of -1 means no constraint.
   float FOffsetX;                      // Horizontal fine tuning offset.
   float FOffsetY;                      // Vertical fine tuning offset.
@@ -130,7 +134,8 @@ public:
   CCaptionElementTemplate(wstring key);
 
   void initialize(wstring Text, float X, float Y, string FontFamily, int FontSize, int Weight, string FontStyle, 
-    TAlignment HorzontalAlignment, TAlignment VerticalAlignment, GLubyte* Color, const TConstraints& Constraints);
+    TAlignment HorzontalAlignment, TAlignment VerticalAlignment, GLubyte* Color, const TConstraints& Constraints,
+    bool WrapText);
   wstring key(void) { return FKey; };
 };
 
@@ -154,21 +159,38 @@ public:
   wstring layoutClass(void) { return FClass; };
   wstring type(void) { return FType; };
   virtual TGCVariant __cdecl property(const char* name, unsigned int index);
-  virtual void __cdecl property(const char* name, unsigned int index, const TGCVariant& value);
+  virtual void __cdecl property(const char* name, unsigned int index, TGCVariant value);
 };
 
 //----------------------------------------------------------------------------------------------------------------------
 
+#define ALFREDO_TEXT
+
 /**
  * Instance for figure elements and captions.
  */
-class CCaptionElement: public CGCBase
+class CCaptionElement: public CGraphicElement
 {
   friend class CFigureElement;
 private:
+  struct CTextLineLayout
+  {
+    float xOffset;
+    float yOffset;
+    float width;
+    wstring text;
+  };
+  typedef vector<CTextLineLayout> CTextLayout;
+  
   GLuint FList;                        // The display list for that text.
   wstring FText;                       // The text of the caption.
-  wstring FDisplayText;                // The text that is actually displayed (might be shortend with ellipsis).
+#ifdef ALFREDO_TEXT
+  CTextLayout FTextLayout;             // The text that is actually displayed (might be shortend with ellipsis).
+#else
+  wstring FDisplayText;                // The text that is actually displayed (might be shortend with ellipsis).^M
+  float FAlignOffsetX;                 // Horizontal text alignment offset.^M
+  float FAlignOffsetY;                 // Vertical text alignment offset.^M
+#endif
   string FFontFamily;                  // The font to be used for output (e.g. Arial, Verdana)
   int FFontSize;                       // Font size in points at 100% zoom.
   int FActualFontSize;                 // Actual font size in points, adjusted by current zoom.
@@ -176,121 +198,141 @@ private:
   string FFontStyle;                   // normal or italic
   TAlignment FHorizontalAlignment;     // Left, center, right
   TAlignment FVerticalAlignment;       // Top, center, bottom
-  TBoundingBox FBounds;                // The box that includes the text.
   CFigureElement* FParent;             // The parent element of this element (if at all).
-  bool FDirty;                         // True if anything changed in the element.
   GLfloat FColor[4];                   // The text color.
   bool FHasColor;                      // True, if this caption has an own color.
-  float FAlignOffsetX;                 // Horizontal text alignment offset.
-  float FAlignOffsetY;                 // Vertical text alignment offset.
+  bool FWrapText;                      // True if the text should be wrapped around.
   float FOffsetX;                      // Horizontal fine tuning offset given by the template.
   float FOffsetY;                      // Vertical fine tuning offset given by the template.
   TConstraints FConstraints;           // Size constraints for the element. A value of -1 means no constraint.
   TBidiMode FBidiMode;                 // Determines text directionality.
+
+  TBoundingBox breakupLine(const wstring &text, float availableWidth,
+                           float ellipsisWidth,
+                           wstring &result, wstring &leftover);
+  CTextLayout createTextLayout(float width, float height, float ellipsisWidth, const wstring &text);
+
 public:
   CCaptionElement(CGenericCanvas* canvas);
   virtual ~CCaptionElement(void);
 
   void applyAlignment(void);
-  TBoundingBox bounds(void) { return FBounds; };
-  void makeDirty(void);
+  virtual void __cdecl makeDirty(TGCChangeReason reason = GC_CHANGE_NONE);
   virtual TGCVariant __cdecl property(const char* name, unsigned int index);
-  virtual void __cdecl property(const char* name, unsigned int index, const TGCVariant& value);
+  virtual void __cdecl property(const char* name, unsigned int index, TGCVariant value);
   void render(void);
   void text(wstring newText);
-  void validate(void);
+  virtual void __cdecl validate(void);
   bool zoomChanged(float ZoomFactor);
 };
 
-class CElementListener: public CGCListener
-{
-  friend class CFigureElement;
-protected:
-    CFigureElement* element;
-public:
-  virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason);
-  virtual void __cdecl onDestroy(CGCBase* sender);
-  virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* message);
-};
-
-class GENERIC_CANVAS_API CFigureElement: public CGCBase
+class GENERIC_CANVAS_API CFigureElement: public CGraphicElement
 {
   friend class CCaptionElement;
 private:
+  class CElementListener: private CGCListener
+  {
+    friend class CFigureElement;
+  protected:
+      CFigureElement* element;
+  public:
+    virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TActionType& actionType);
+    virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason);
+    virtual void __cdecl onDestroy(CGCBase* sender);
+    virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* message);
+  };
+
   float FTranslation[3];          // Determines the location of the element after layouting.
   TFigureElementLayout FLayout;   // How are the element's children laid out.
-  bool FResizable;                // Indicates whether this element can be resized.
-  TBoundingBox FBounds;           // The box that includes this and all child elements.
+  TFigureElementResize FResizeMode;  // Indicates how this element can be resized.
   CElementList FChildren;         // A list of child elements attached to this element.
   CGCStyle* FStyle;               // The style to be used for this element.
   CFigureElement* FParent;        // The parent element of this element (if at all).
-  CFigure* FFigure;               // The figure this element belongs to if not a child element of another element.
-                                  // FParent and FFigure are mutual exclusive. Only one of both (or none) can be set.
+  CFigure* FFigure;               // The figure this element belongs to.
   CCaptionElement* FCaption;      // An optional caption for this element.
   CLayouter* FLayouter;           // The class that computes the layout.
-  bool FDirty;                    // True if anything changed in the element.
   bool FExpanded;                 // True if child elements are visible.
   TConstraints FConstraints;      // Size constraints for the element. A value of -1 means no constraint.
   CElementListener FListener;
   CFigureElementTemplate* FTemplate; // Reference to the template part for this element.
+  float FForcedWidth;             // >= 0 if a certain width is assigned.
+  float FForcedHeight;            // >= 0 if a certain height is assigned.
+  float FStyleScaleX;             // Horizontal scale factor in case the style (if any) does not have the same size as the element.
+  float FStyleScaleY;             // Dito for y direction.
 protected:
   void computeBoundingBox(void);
+  TVertex computeOffset(CFigureElement* child);
+  TActionType doAction(TAction* action);
 public:
   CFigureElement(CFigureElementTemplate* aTemplate, CGenericCanvas* canvas);
   virtual ~CFigureElement(void);
 
   CFigureElement* addSubElement(void);
-  TBoundingBox bounds(void) { return FBounds; };
+  bool canResize(bool vertical);
   CElementList* children(void) { return &FChildren; };
   static CFigureElement* createFromTemplate(CGenericCanvas* canvas, CFigure* Owner, CFigureElementTemplate* Template);
-  bool doAction(CFigureInstance* Instance, const float X, const float Y);
-  CFigure* figure(void);
+  CFigureElement* elementFromId(wstring id);
+  CFigureElement* elementFromPoint(TVertex coords);
+  TVertex elementToFigure(TVertex coords);
+  TActionType executeAssociatedActions(const float X, const float Y);
+  TActionType executeAction(TActionType action);
+  void expanded(bool isExpanded);
+  bool expanded(void) { return FExpanded; };
+  CFigure* figure(void) { return FFigure; };
+  TVertex figureToElement(TVertex coords);
+  virtual TFeedbackInfo __cdecl getFeedbackInfo(TVertex coords);
   TFigureElementLayout layout(void) { return FLayout; };
-  void makeDirty(void);
+  virtual void __cdecl makeDirty(TGCChangeReason reason = GC_CHANGE_NONE);
   virtual TGCVariant __cdecl property(const char* name, unsigned int index);
-  virtual void __cdecl property(const char* name, unsigned int index, const TGCVariant& value);
+  virtual void __cdecl property(const char* name, unsigned int index, TGCVariant value);
   void render(void);
+  bool resize(float dX, float dY, TFeedbackInfo info);
   void setCaption(const char* text);
   void style(CGCStyle* NewStyle);
   CGCStyle* style(void);
   CFigureElementTemplate* template_(void) { return FTemplate; };
-  void validate(void);
+  virtual void __cdecl validate(void);
   bool zoomChanged(float ZoomFactor);
 };
 
 //----------------------------------------------------------------------------------------------------------------------
 
+#ifdef __GNUC__
+typedef map<wstring, CFigureElement*> CFigureElementMap;
+#else
 typedef hash_map<wstring, CFigureElement*> CFigureElementMap;
-
-class CFigureElementListener: public CGCListener
-{
-  friend class CFigure;
-protected:
-    CFigure* figure;
-public:
-  virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason);
-  virtual void __cdecl onDestroy(CGCBase* sender);
-  virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* message);
-};
+#endif
 
 /**
  * CFigure is the main element in the model and is created from a figure template. It cannot itself appear in a scene but
  * is represented by one or more figure instances.
  */
-class GENERIC_CANVAS_API CFigure: public CGCBase
+class GENERIC_CANVAS_API CFigure: public CGraphicElement
 {
   friend class CGCModel;
   friend class CFigureInstance;
 private:
+  class CFigureElementListener: private CGCListener
+  {
+    friend class CFigure;
+  protected:
+      CFigure* figure;
+  public:
+    virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TActionType& actionType);
+    virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason);
+    virtual void __cdecl onDestroy(CGCBase* sender);
+    virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* message);
+  };
+
   CGCModel* FModel;          // The model to which this figure belongs.
   CFigureElement* FContent;  // The root element of the figure.
-  bool FDirty;               // True if any of the properties changed that affect the display list.
   float FScaling[3];         // The factors to scale the figure in each direction.
   float FMinimalScale;       // The lesser of x and y scale factors.
   float FTranslation[3];     // The factors to move the figure.
   float FRotation[4];        // The factors to rotate the figure. FRotation[0] is an angle in radians. Index 1-3 form a vector to rotate around.
                              // Note: Order of application of the 3 transformations is scaling first, then rotation, finally translation.
-  TBoundingBox FBounds;      // Bounding box of the figure in figure instance space (i.e. relative to its figure instances).
+  TMatrix FMatrix;           // All three local transformations in one matrix.
+  TMatrix FInverse;          // FMatrix inverted.
   float FLastZoom;           // The most recent minimal overall scale factor (including its own scale factors).
   CFigureTemplate* FTemplate; // The template from which this figure was built.
   CFigureElementListener FListener;
@@ -298,94 +340,103 @@ private:
 protected:
   void applyTransformations(void);
   void buildFromTemplate(CFigureTemplate* Template);
-  void validate(void);
+  void updateMatrix(void);
 public:
   CFigure(CGCModel* Owner, CFigureTemplate* Template);
   virtual ~CFigure(void);
 
   void addMapping(wstring path, CFigureElement* element);
-  virtual TBoundingBox __cdecl bounds(void) { return FBounds; };
   CFigureElement* content(void) { return FContent; };
+  CFigureElement* elementFromId(const wstring& id);
   CFigureElement* elementFromKey(const char* key);
   CFigureElement* elementFromKey(wstring key);
+  virtual CFigureElement* __cdecl elementFromPoint(TVertex coords);
+  virtual TVertex __cdecl figureToView(TVertex coords, CFigureInstance* instance);
   void freeNotification(CFigureElement* object);
-  void makeDirty(void);
   CGCModel* model(void) { return FModel; };
   virtual TGCVariant __cdecl property(const char* name, unsigned int index);
-  virtual void __cdecl property(const char* name, unsigned int index, const TGCVariant& value);
+  virtual void __cdecl property(const char* name, unsigned int index, TGCVariant value);
   void removeMapping(CFigureElement* element);
   virtual void __cdecl render(float currentZoom);
-  virtual void __cdecl rotate(float Angle, const float Axis[3]);
   virtual void __cdecl rotate(float Angle, float Rx, float Ry, float Rz);
-  virtual void __cdecl scale(const float Factor[3], bool Accumulative = false);
+  virtual void __cdecl rotateV(float Angle, const float Axis[3]);
   virtual void __cdecl scale(float Sx, float Sy, float Sz, bool Accumulative = false);
+  virtual void __cdecl scaleV(const float Factor[3], bool Accumulative = false);
+  virtual bool __cdecl setSize(float width, float height);
   CFigureTemplate* template_(void) { return FTemplate; };
-  virtual void __cdecl translate(const float Factor[3], bool Accumulative = false);
   virtual void __cdecl translate(float Tx, float Ty, float Tz, bool Accumulative = false);
+  virtual void __cdecl translateV(const float Factor[3], bool Accumulative = false);
+  virtual void __cdecl validate(void);
+  virtual TVertex __cdecl viewToFigure(TVertex coords, CFigureInstance* instance);
 };
 
 //----------------------------------------------------------------------------------------------------------------------
 
-class CFigureListener: public CGCListener
-{
-  friend class CFigureInstance;
-protected:
-  CFigureInstance* instance;
-public:
-  virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason);
-  virtual void __cdecl onDestroy(CGCBase* sender);
-  virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* Message);
-};
-
 /**
  * The figure instance class is a proxy for a figure on a particular layer. There can be more than one instance pointing 
  * to the same figure.
  */
-class GENERIC_CANVAS_API CFigureInstance: public CGCBase
+class GENERIC_CANVAS_API CFigureInstance: public CGraphicElement
 {
-  friend class CGenericCanvas;
+  friend class CGCView;
   friend class CLayer;
   friend class CFigure;
   friend class CFeedbackLayer;
   friend class CFigureListener;
 private:
+  class CFigureListener: private CGCListener
+  {
+    friend class CFigureInstance;
+  protected:
+    CFigureInstance* instance;
+  public:
+    virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TActionType& action);
+    virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason);
+    virtual void __cdecl onDestroy(CGCBase* sender);
+    virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* Message);
+  };
+
   CLayer* FLayer;            // The layer on which this figure is displayed.
   CFigure* FFigure;          // The figure of which this class is an instance.
-  bool FDirty;               // True if any of the properties changed that affect the display list.
   float FScaling[3];         // The factors to scale the figure in each direction.
   float FMinimalScale;       // The lesser of x and y scale factors.
   float FTranslation[3];     // The factors to move the figure.
   float FRotation[4];        // The factors to rotate the figure. FRotation[0] is an angle in radians. Index 1-3 form a vector to rotate around.
                              // Note: Order of apllication of the 3 transformations is scaling first, then rotation, finally translation.
-  TBoundingBox FBounds;      // Bounding box of the figure instance in layer coordinates.
+  TMatrix FMatrix;           // All three local transformations in one matrix.
+  TMatrix FInverse;          // FMatrix inverted.
   bool FSelected;            // True if this instance is currently selected.
   CFigureListener FListener;
 protected:
   void applyTransformations(void);
-  bool doAction(const float X, const float Y);
-  void makeDirty(void);
-  void onDestroy(CGCBase* Figure);
+  void onDestroy(CGCBase* figure);
+  TActionType executeAssociatedActions(TVertex& coords);
   void replaceFigure(CFigure* figure);
-  void validate(void);
+  void resize(float dX, float dY, TFeedbackInfo info);
+  void updateMatrix(void);
 public:
   CFigureInstance(CLayer* Owner, CFigure* Figure);
   virtual ~CFigureInstance(void);
 
-  virtual TBoundingBox __cdecl bounds(void) { return FBounds; };
-  virtual bool __cdecl containsPoint(const float X, const float Y);
-  virtual bool __cdecl overlaps(TBoundingBox& Box);
+  virtual CFigureElement* __cdecl elementFromPoint(TVertex coords);
+  virtual CFigure* __cdecl figure(void) { return FFigure; };
+  virtual TFeedbackInfo __cdecl getFeedbackInfo(TVertex coords);
+  virtual TVertex __cdecl instanceToView(TVertex coords);
+  virtual CLayer* __cdecl layer(void) { return FLayer; };
+  virtual TBoundingBox __cdecl localBounds(void);
+  virtual void __cdecl makeDirty(TGCChangeReason reason = GC_CHANGE_NONE);
   virtual TGCVariant __cdecl property(const char* name, unsigned int index);
-  virtual void __cdecl property(const char* name, unsigned int index, const TGCVariant& value);
-  virtual void __cdecl render(float CurrentZoom);
-  virtual void __cdecl rotate(float Angle, float Rx, float Ry, float Rz);
-  virtual void __cdecl rotateV(float Angle, const float Axis[3]);
-  virtual void __cdecl scale(float Sx, float Sy, float Sz, bool Accumulative = false);
-  virtual void __cdecl scaleV(const float Factor[3], bool Accumulative = false);
+  virtual void __cdecl property(const char* name, unsigned int index, TGCVariant value);
+  virtual void __cdecl render(float currentZoom);
+  virtual void __cdecl rotate(float angle, float rx, float ry, float rz);
+  virtual void __cdecl rotateV(float angle, const float axis[3]);
+  virtual void __cdecl scale(float sx, float sy, float sz, bool accumulative = false);
+  virtual void __cdecl scaleV(const float factor[3], bool accumulative = false);
   virtual bool __cdecl selected(void);
-  virtual void __cdecl translate(float Tx, float Ty, float Tz, bool Accumulative = false);
-  virtual void __cdecl translateV(const float Factor[3], bool Accumulative = false);
+  virtual void __cdecl translate(float tx, float ty, float tz, bool accumulative = false);
+  virtual void __cdecl translateV(const float factor[3], bool accumulative = false);
+  virtual void __cdecl validate(void);
+  virtual TVertex __cdecl viewToInstance(TVertex coords);
 };
 
-//----------------------------------------------------------------------------------------------------------------------
-
 #endif // __GC_FIGURE_H__
