@@ -15,15 +15,13 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /**
- * @file myx_gc_font_manager.cpp 
- * @brief A class that manages shared display lists used for text output
- * 
+ * @file myx_gc_font_manager.h 
+ * @brief Implements the global font manager class that handles all text output related tasks.
  */
 
 #ifndef __GC_FONT_MANAGER_H__
 #define __GC_FONT_MANAGER_H__
 
-#include "myx_gc.h"
 #include "myx_gc_utilities.h"
 
 #ifdef USE_FONTCONFIG
@@ -31,15 +29,15 @@
 #endif
 
 // FTGL
-#include <FTGL/FTGLOutlineFont.h>
-#include <FTGL/FTGLPolygonFont.h>
-#include <FTGL/FTGLBitmapFont.h>
-#include <FTGL/FTGLTextureFont.h>
-#include <FTGL/FTGLPixmapFont.h>
-#include <FTGL/FTGLExtrdFont.h>
-
+#include <FTGLOutlineFont.h>
+#include <FTGLPolygonFont.h>
+#include <FTGLBitmapFont.h>
+#include <FTGLTextureFont.h>
+#include <FTGLPixmapFont.h>
+#include <FTGLExtrdFont.h>
 
 //----------------------------------------------------------------------------------------------------------------------
+
 #ifdef __GNUC__
 typedef map<string, FTFont*> Fonts;
 #else
@@ -67,6 +65,7 @@ class CFontManager
 private:
   Fonts FFonts;
   bool FUseLocalDisplayLists;
+  string FFontResourcesPath;
 
 #ifndef USE_FONTCONFIG
   FontFiles FFiles;
@@ -80,7 +79,8 @@ public:
 
   void boundingBox(const wstring& Output, string FontFamily, string FontStyle, int Weight, int FontSize, TBoundingBox* Box);
   void clear(void);
-  string createLookupKey(const string& Family, const string& Style, int Weight);
+  string createLookupKey(const string& Family, const string& Style, int Weight, int FontSize);
+  void setFontResourcesPath(const string& path);
   void textOut(const wstring& Output, string FontFamily, string FontStyle, int Weight, int FontSize);
 };
 
