@@ -23,37 +23,35 @@
 #ifndef __GC_GL_HELPER_H__
 #define __GC_GL_HELPER_H__
 
-#include "myx_gc.h"
 #include <libxml/xmlmemory.h>
 
-#include "myx_gc_datatypes.h"
+#include "myx_gc_font_manager.h"
+#include "myx_gc_texture.h"
+
 
 // Reads the attribute with the given name (if it exists) and converts it to a float value.
-bool getFloatAttribute(xmlNodePtr Element, const char* Name, float& Value);
+bool getFloatAttribute(xmlNodePtr Element, const char* name, float& Value);
 
 // Like GetFloatAttribute but with a default value in case the attribute does not exist.
-float getFloatAttributeDef(xmlNodePtr Element, const char* Name, float Default);
+float getFloatAttributeDef(xmlNodePtr Element, const char* name, float Default);
 
 // Reads the attribute with the given name (if it exists) and converts it to an integer value.
-bool getIntAttribute(xmlNodePtr Element, const char* Name, int& Value);
+bool getIntAttribute(xmlNodePtr Element, const char* name, int& Value);
 
 // Like GetIntAttribute but with a default value in case the attribute does not exist.
-int getIntAttributeDef(xmlNodePtr Element, const char* Name, int Default);
+int getIntAttributeDef(xmlNodePtr Element, const char* name, int Default);
 
 // Reads the attribute with the given name (if it exists) and returns it.
-bool getStringAttribute(xmlNodePtr Element, const char* Name, string& Value);
+bool getStringAttribute(xmlNodePtr Element, const char* name, string& Value);
 
 // Like GetStringAttribute but with a default value in case the attribute does not exist.
-string getStringAttributeDef(xmlNodePtr Element, const char* Name, const string Default);
+string getStringAttributeDef(xmlNodePtr Element, const char* name, const string Default);
 
 // Reads the attribute with the given name and treats it as color value.
-int convertColor(xmlNodePtr Element, const char* Name, GLubyte* Color);
+int convertColor(xmlNodePtr Element, const char* name, GLubyte* Color);
 
 // Parses the given XML element for a texture definition.
-void parseTextureEntry(xmlNodePtr XML);
-
-// Parses the given XML element for a texture definition.
-void parseTextureEntry(xmlNodePtr XML);
+void parseTextureEntry(xmlNodePtr XML, CGCModel* model);
 
 #endif // __GC_GL_HELPER_H__
 
