@@ -20,17 +20,8 @@
  * 
  */
 
-#include "myx_gc.h"
-#include "myx_gc_figure.h"
-#include "myx_gc_layer.h"
-#include "myx_gc_model.h"
-#include "myx_gc_gl_helper.h"
-#include "myx_gc_font_manager.h"
-#include "myx_gc_texture.h"
-#include "myx_gc_svgparser.h"
-#include "myx_gc_figure_parser.h"
 #include "myx_gc_canvas.h"
-
+#include "myx_gc_gl_helper.h"
 
 //----------------------------------------------------------------------------------------------------------------------
 
@@ -40,121 +31,31 @@ GENERIC_CANVAS_API CGenericCanvas* CreateGenericCanvas(GCContext Context, char*
 	return new CGenericCanvas(Context, utf8ToUtf16(name));
 }
 
-//----------------- CFigureInstanceEnumerator --------------------------------------------------------------------------
-
-/**
- * Constructor of the enumerator class.
- *
- * @param canvas The canvas which contains the layers which are to be enumerated.
- */
-CFigureInstanceEnumerator::CFigureInstanceEnumerator(CGenericCanvas* canvas)
-{
-  FCanvas = canvas;
-  FLayerIterator = FCanvas->FLayers.end();
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-/**
- * Determines if there is a next figure instance to enumerate.
- *
- * @return True if there is still a figure instance otherwise false.
- */
-bool CFigureInstanceEnumerator::hasNext(void)
-{
-  return FLayerIterator != FCanvas->FLayers.end();
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-/**
- * Returns the next figure instance in the sequence.
- *
- * @return The next figure instance.
- */
-CFigureInstance* CFigureInstanceEnumerator::next(void)
-{
-  CFigureInstance* result = *FFigureInstanceIterator;
-
-  // Advance to next instance.
-  ++FFigureInstanceIterator;
-  while (true)
-  {
-    CLayer* layer = *FLayerIterator;
-    if (FFigureInstanceIterator != layer->FInstances.end())
-      break;
-
-    // This layer is exhausted. Get the next one.
-    ++FLayerIterator;
-    if (FLayerIterator == FCanvas->FLayers.end())
-    {
-      // No more layers.
-      break;
-    };
-
-    if (!(*FLayerIterator)->FEnabled)
-      FFigureInstanceIterator = (*FLayerIterator)->FInstances.end();
-  };
-
-  return result;
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-/**
- * Frees this enumerator instance. Usually called by non-C++ languages as memory is managed by the C++ runtime.
- */
-void CFigureInstanceEnumerator::release(void)
-{
-  delete this;
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-/**
- * Resets the enumerator to the first figure instance in the canvas.
- */
-void CFigureInstanceEnumerator::reset(void)
-{
-  FLayerIterator = FCanvas->FLayers.begin();
-  while (FLayerIterator != FCanvas->FLayers.end())
-  {
-    if ((*FLayerIterator)->FEnabled)
-    {
-      CLayer* layer = *FLayerIterator;
-      FFigureInstanceIterator = layer->FInstances.begin();
-      if (FFigureInstanceIterator == layer->FInstances.end())
-      {
-        ++FLayerIterator;
-      }
-      else
-        break; // Found the first instance.
-    }
-    else
-      ++FLayerIterator;
-  };
-}
-
 //----------------- CCanvasListener ------------------------------------------------------------------------------------
 
-void CCanvasListener::onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason)
+void CGenericCanvas::CCanvasListener::onAction(CGCBase* sender, CGCBase* origin, TActionType& actionType)
 {
-  if (canvas != NULL)
-    canvas->change(origin, reason);
+  canvas->action(origin, actionType);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
 
-void CCanvasListener::onDestroy(CGCBase* object)
+void CGenericCanvas::CCanvasListener::onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason)
+{
+  canvas->change(origin, reason);
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+void CGenericCanvas::CCanvasListener::onDestroy(CGCBase* object)
 {
 }
 
 //----------------------------------------------------------------------------------------------------------------------
 
-void CCanvasListener::onError(CGCBase* sender, CGCBase* origin, const char* message)
+void CGenericCanvas::CCanvasListener::onError(CGCBase* sender, CGCBase* origin, const char* message)
 {
-  if (canvas != NULL)
-    canvas->error(origin, message);
+  canvas->error(origin, message);
 }
 
 //----------------- CGenericCanvas -------------------------------------------------------------------------------------
@@ -168,13 +69,16 @@ CGenericCanvas::CGenericCanvas(GCContext Context, wstring name): CGCBase(NULL)
   FCurrentView = NULL;
   FStates = 0;
 
+#ifdef _WINDOWS
+  InitializeCriticalSection(&FLock);
+#else
   g_static_rec_mutex_init(&FLock);
+#endif
 
   FListener.canvas = this;
   FModel = new CGCModel(this);
   FModel->addListener(&FListener);
-  FFeedbackLayer = new CFeedbackLayer("Feedback", this);
-  FFeedbackLayer->addListener(&FListener);
+  determineExtensions();
 
   // Set a new lock on the font manager (and create it by the way if not yet done).
   lockFontManager();
@@ -185,24 +89,26 @@ CGenericCanvas::CGenericCanvas(GCContext Context, wstring name): CGCBase(NULL)
 CGenericCanvas::~CGenericCanvas(void)
 {
   beginUpdate();
+  disableEvents();
 
-  // Free any dynamically allocated memory. Explicitely increase update count. We don't want any recursive
-  // notifcations anymore.
-  FFeedbackLayer->beginUpdate();
-  for (CLayers::iterator iterator = FLayers.begin(); iterator != FLayers.end(); ++iterator)
-    delete *iterator;
+  // Release all managed data. Start with the visual parts to avoid frequent change events between model and other elements.
   for (CViews::iterator iterator = FViews.begin(); iterator != FViews.end(); ++iterator)
     delete *iterator;
+  for (CLayers::iterator iterator = FLayers.begin(); iterator != FLayers.end(); ++iterator)
+    delete *iterator;
 
   delete FModel;
-  delete FFeedbackLayer;
 
-  // release the lock we placed in the constructor.
+  // Release the lock we placed in the constructor.
   // If this is the last GC instance then also the font manager can be released.
   unlockFontManager();
   endUpdate();
 
+#ifdef _WINDOWS
+  DeleteCriticalSection(&FLock);
+#else
   g_static_rec_mutex_free(&FLock);
+#endif
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -214,19 +120,53 @@ void CGenericCanvas::clearBuffers(void)
 
 //----------------------------------------------------------------------------------------------------------------------
 
+/**
+ * Determines whether certain OpenGL extension are supported.
+ */
+void CGenericCanvas::determineExtensions(void)
+{
+  FExtensions.push_back(false); // GC_OE_MULTISAMPLING
+  FExtensions.push_back(false); // GC_OE_CONVOLUTION
+  FExtensions.push_back(false); // GC_OE_HISTOGRAM
+  FExtensions.push_back(false); // GC_OE_TEXTURE3D
+
+  const GLubyte* extString = glGetString(GL_EXTENSIONS);
+  if (extString != NULL)
+  {
+    if (strstr((char*) extString, "GL_EXT_convolution") != NULL)
+      FExtensions[GC_OE_CONVOLUTION] = true;
+    if (strstr((char*) extString, "GL_EXT_histogram") != NULL)
+      FExtensions[GC_OE_HISTOGRAM] = true;
+    if (strstr((char*) extString, "GL_EXT_texture3D") != NULL)
+      FExtensions[GC_OE_TEXTURE3D] = true;
+    if (strstr((char*) extString, "GL_ARB_multisample") != NULL)
+      FExtensions[GC_OE_MULTISAMPLING] = true;
+  };
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 /**
  * Aquires the canvas lock used to synchronize threads.
  */
 void CGenericCanvas::lock(void)
 {
+#ifdef _WINDOWS
+  EnterCriticalSection(&FLock);
+#else
   g_static_rec_mutex_lock(&FLock);
+#endif
 }
 
 //----------------------------------------------------------------------------------------------------------------------
 
 void CGenericCanvas::unlock(void)
 {
+#ifdef _WINDOWS
+  LeaveCriticalSection(&FLock);
+#else
   g_static_rec_mutex_unlock(&FLock);
+#endif
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -247,54 +187,57 @@ void CGenericCanvas::addLayer(CLayer* layer)
  * Reads the layout info stored in the given (XML) file and creates figure templates.
  * Existing templates remain in where they are but are replaced if a new definition with an existing name is found.
  *
- * @param Filename The name of the file to load.
+ * @param filename The name of the file to load.
  * @return Returns GC_NO_ERROR if everything was ok, otherwise an error code.
  */
-TGCError CGenericCanvas::addLayoutsFromFile(const char* Filename)
+TGCError CGenericCanvas::addLayoutsFromFile(const char* filename)
 {
   TGCError result = GC_NO_ERROR;
-  xmlNodePtr Root, current;
-  CFigureParser Parser;
+  xmlNodePtr root, current;
+  CFigureParser parser(this);
+  parser.addListener(&FListener);
 
   string currentDir = getCurrentDir();
-  xmlDocPtr Document = xmlParseFile(utf8ToANSI(string(Filename)).c_str());
+  xmlDocPtr document = xmlParseFile(utf8ToANSI(string(filename)).c_str());
 
-  if (Document == NULL)
+  if (document == NULL)
     return GC_XML_PARSE_ERROR;
 
-  Root = xmlDocGetRootElement(Document);
+  root = xmlDocGetRootElement(document);
 
-  if (Root == NULL)
+  if (root == NULL)
   {
-    xmlFreeDoc(Document);
+    xmlFreeDoc(document);
     error(this, "XML Error: Template file is empty.");
     return GC_XML_EMPTY_DOCUMENT;
   }
   
-  if (!XML_IS(Root, "gc-layouts"))
+  if (!XML_IS(root, "gc-layouts"))
   {
-    xmlFreeDoc(Document);
+    xmlFreeDoc(document);
     error(this, "XML Error: Template file invalid.");
     return GC_XML_INVALID_DOCUMENT;
   }
 
   // Switch to the directory of the given file. This is necessary to make relative file names working.
-  string path = extractFilePath(Filename);
+  string path = extractFilePath(filename);
   setCurrentDir(path);
 
   // Parse description elements.
   glMatrixMode(GL_MODELVIEW);
-  current = Root->children;
+  current = root->children;
   while (current != NULL)
   {
     // Be flexible, ignore any unknown layout entries.
     if (XML_IS(current, "layout-definition"))
-      Parser.parseLayoutDefinition(current, FModel);
+      parser.parseLayoutDefinition(current, FModel);
     current = current->next;
   }
 
   setCurrentDir(currentDir);
-  xmlFreeDoc(Document);
+  xmlFreeDoc(document);
+
+  parser.removeListener(&FListener);
 
   return result;
 }
@@ -305,77 +248,172 @@ TGCError CGenericCanvas::addLayoutsFromFile(const char* Filename)
  * Reads the style template info stored in the given (XML) file and creates style templates (OpenGL display lists).
  * Existing templates remain where they are but are replaced if a new definition with an existing name is found.
  *
- * @param Filename The name of the file to load.
+ * @param filename The name of the file to load.
+ * @param variables A string containing line break separated name=value entries (note: no white spaces around the equal
+ *                  sign or they will become part of the value and name strings, respectively.
  * @return Returns GC_NO_ERROR if everything was ok, otherwise an error code.
  */
-TGCError CGenericCanvas::addStylesFromFile(const char* Filename)
+TGCError CGenericCanvas::addStylesFromFile(const char* filename, const char* variables)
+{
+  map<string, string> variablesMap;
+
+  if (variables != NULL)
+  {
+    StringTokenizer tokenizer(variables, "\r\n");
+    while (tokenizer.hasMoreTokens())
+    {
+      string token = tokenizer.nextToken();
+      size_t position = token.find("=");
+      if (position != token.npos)
+      {
+        string name = token.substr(0, position);
+        string value = token.substr(position + 1);
+        variablesMap[name] = value;
+      };
+    };
+  };
+
+  return addStylesFromFile(filename, variablesMap);
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+/** 
+ * Reads the style template info stored in the given (XML) file and creates style templates (OpenGL display lists).
+ * Existing templates remain where they are but are replaced if a new definition with an existing name is found.
+ *
+ * @param filename The name of the file to load.
+ * @param variables A set of name/value pairs to be replaced in the xml text.
+ * @return Returns GC_NO_ERROR if everything was ok, otherwise an error code.
+ */
+TGCError CGenericCanvas::addStylesFromFile(const char* filename, map<string, string>& variables)
 {
   TGCError result = GC_NO_ERROR;
-  xmlNodePtr Root, current;
-  CSVGParser Parser;
-
+  xmlNodePtr root, current;
+  CSVGParser parser;
+  xmlDocPtr document = NULL;
   string currentDir = getCurrentDir();
-  xmlDocPtr Document = xmlParseFile(utf8ToANSI(string(Filename)).c_str());
-
-  if (Document == NULL)
-    return GC_XML_PARSE_ERROR;
-
-  Root = xmlDocGetRootElement(Document);
-
-  if (Root == NULL)
+  
+  if (!variables.empty())
   {
-    xmlFreeDoc(Document);
+    char *data;
+    size_t length;
+
+    data= preprocessFile(filename, variables, length);
+    if (data)
+    {
+      document = xmlParseMemory(data, (int) length);
+      free(data);
+    }
+  }
+  else
+    document = xmlParseFile(utf8ToANSI(filename).c_str());
+
+  if (document == NULL)
+    return GC_XML_PARSE_ERROR;                       
+
+  root = xmlDocGetRootElement(document);
+
+  if (root == NULL)
+  {
+    xmlFreeDoc(document);
     error(this, "XML Error: Template file is empty.");
     return GC_XML_EMPTY_DOCUMENT;
   }
   
-  if (!XML_IS(Root, "gc-styles"))
+  if (!XML_IS(root, "gc-styles"))
   {
-    xmlFreeDoc(Document);
+    xmlFreeDoc(document);
     error(this, "XML Error: Template file invalid.");
     return GC_XML_INVALID_DOCUMENT;
   }
 
   // Switch to the directory of the given file. This is necessary to make relative file names working.
-  string path = extractFilePath(Filename);
+  string path = extractFilePath(filename);
   setCurrentDir(path);
 
   // Parse description elements.
   glMatrixMode(GL_MODELVIEW);
-  current = Root->children;
+  current = root->children;
   while (current != NULL)
   {
     // Be flexible, ignore any unknown layout entries.
     if (XML_IS(current, "style-definition"))
     {
-      Parser.parseDefinition(current, FModel);
+      parser.parseDefinition(current, FModel);
     }
     else
       if (XML_IS(current, "texture"))
       {
         // Adds a new texture to the texture list.
-        parseTextureEntry(current);
+        parseTextureEntry(current, FModel);
         checkError();
       };
     current = current->next;
   }
 
   setCurrentDir(currentDir);
-  xmlFreeDoc(Document);
+  xmlFreeDoc(document);
 
   return result;
 }
 
 //----------------------------------------------------------------------------------------------------------------------
 
-/**
- * Adds the given figure instance to the current selection. This call is simply forwared to the selection layer.
- * 
- * @param Instance The instance to be added to the selection. It is taken care that an instance is only added once.
+/** 
+ * Takes the given definition and creates a generic canvas style from it. The format in the definition is the same
+ * format as in the XML file that can be used to load styles from disk. Only a single <style-definition></style-definition>
+ * or <texture></texture> entry must be in the definition. The caller is responsible to ensure the currently active folder 
+ * is set so that eventually used relative paths in the definition work as expected.
+ * If a style with the ID used in the definition exists already then it will be changed to the new definition.
+ *
+ * @param definition An UTF-8 encoded <style-definition></style-definition>.
+ * @return Returns GC_NO_ERROR if everything was ok, otherwise an error code.
  */
-void CGenericCanvas::addToSelection(CFigureInstance* Instance)
+TGCError CGenericCanvas::addStyleFromDefinition(const char* definition)
 {
-  FFeedbackLayer->addToSelection(Instance);
+  TGCError result = GC_NO_ERROR;
+  xmlNodePtr root;
+  CSVGParser parser;
+
+  string documentText = string("<?xml version=\"1.0\" encoding=\"UTF-8\"?>") + definition;
+  xmlDocPtr document = xmlParseMemory(documentText.c_str(), (int) documentText.size());
+
+  if (document == NULL)
+    return GC_XML_PARSE_ERROR;                       
+
+  root = xmlDocGetRootElement(document);
+
+  if (root == NULL)
+  {
+    xmlFreeDoc(document);
+    error(this, "XML Error: Definition is empty.");
+    return GC_XML_EMPTY_DOCUMENT;
+  }
+  
+  if (!XML_IS(root, "style-definition"))
+  {
+    xmlFreeDoc(document);
+    error(this, "XML Error: Definition is not invalid.");
+    return GC_XML_INVALID_DOCUMENT;
+  }
+
+  glMatrixMode(GL_MODELVIEW);
+  if (XML_IS(root, "style-definition"))
+  {
+    parser.parseDefinition(root, FModel);
+  }
+  else
+    if (XML_IS(root, "texture"))
+    {
+      // Adds a new texture to the texture list.
+      parseTextureEntry(root, FModel);
+      checkError();
+    };
+
+  xmlFreeDoc(document);
+
+  return result;
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -389,12 +427,14 @@ void CGenericCanvas::checkError(void)
   if (OGLError != GL_NO_ERROR)
   {
     char buffer[1000];
+    const GLubyte* message = gluErrorString(OGLError);
+
     #ifdef _WINDOWS
       // Microsoft has confirmed that there is no guaranteed NULL termination (see MSDN for details).
       // Hence we better add one.
-      _snprintf(buffer, sizeof(buffer), "OpenGL error encountered (0x%x).\0", (int) OGLError);
+      _snprintf(buffer, sizeof(buffer), "OpenGL error encountered: %s (0x%x).\0", message, (int) OGLError);
     #else
-      snprintf(buffer, sizeof(buffer), "OpenGL error encountered (0x%x).", (int) OGLError);
+      snprintf(buffer, sizeof(buffer), "OpenGL error encountered: %s  (0x%x).", message, (int) OGLError);
     #endif // #ifdef _WINDOWS
     error(this, buffer);
   };
@@ -407,9 +447,12 @@ void CGenericCanvas::checkError(void)
  */
 void CGenericCanvas::clearContent(void)
 {
+  beginUpdate();
   FModel->clearFigures();
+  endUpdate();
+
   change(this, GC_CHANGE_CANVAS_CLEAR_CONTENT);
-  change(this, GC_CHANGE_CANVAS_REFRESH);
+  refresh();
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -419,22 +462,12 @@ void CGenericCanvas::clearContent(void)
  */
 void CGenericCanvas::clearLayouts(void)
 {
+  beginUpdate();
   FModel->clearLayouts();
+  endUpdate();
+
   change(this, GC_CHANGE_CANVAS_CLEAR_LAYOUTS);
-  change(this, GC_CHANGE_CANVAS_REFRESH);
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-/**
- * Removes all currently selected figure instances from the selection set. This call is simply forwareded to the
- * selection layer.
- */
-void CGenericCanvas::clearSelection(void)
-{
-  FFeedbackLayer->clearSelection();
-  change(this, GC_CHANGE_SELECTION_CLEAR);
-  change(this, GC_CHANGE_CANVAS_REFRESH);
+  refresh();
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -444,16 +477,32 @@ void CGenericCanvas::clearSelection(void)
  */
 void CGenericCanvas::clearStyles(void)
 {
+  beginUpdate();
   FModel->clearStyles();
+  endUpdate();
+
   change(this, GC_CHANGE_CANVAS_CLEAR_STYLES);
-  change(this, GC_CHANGE_CANVAS_REFRESH);
+  refresh();
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+/**
+ * Interface function for the model's createConnection function. Read there for details.
+ */
+CConnection* CGenericCanvas::createConnection(CFigure* endPoint1, CFigure* endPoint2)
+{
+  return FModel->createConnection(endPoint1, endPoint2);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
 
 CFigure* CGenericCanvas::createFigure(const char* type, const char* class_)
 {
-  return FModel->createFigure(utf8ToUtf16(type), utf8ToUtf16(class_));
+  if (class_ == NULL)
+    return FModel->createFigure(utf8ToUtf16(type), L"");
+  else
+    return FModel->createFigure(utf8ToUtf16(type), utf8ToUtf16(class_));
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -492,6 +541,7 @@ CGCView* CGenericCanvas::createView(const char* name)
   if (FCurrentView == NULL)
     currentView(View);
 
+  change(View, GC_CHANGE_CANVAS_ADD_VIEW);
   return View;
 }
 
@@ -529,78 +579,6 @@ CGCView* CGenericCanvas::currentView(void)
 
 //----------------------------------------------------------------------------------------------------------------------
 
-/**
- * Tells the given figure instance to execute its default action. This depends on which of the underlying figure
- * elements is hit by the given coordinates.
- *
- * @param Instance The figure instance whose figure is to be called.
- * @param X The horizontal window coordinate.
- * @param Y The vertical window coordinate.
- * @return True if the an action was executed, otherwise false.
- */
-bool CGenericCanvas::doAction(CFigureInstance* Instance, const float X, const float Y)
-{
-  bool result = false;
-
-  if (Instance != NULL && FCurrentView != NULL)
-  {
-    FCurrentView->applyTransformations(true);
-    result = Instance->doAction(X, Y);
-  };
-
-  return result;
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-/**
- * Creates and returns a new figure instance enumerator instance.
- *
- * @return The new enumerator.
- */
-CFigureInstanceEnumerator* CGenericCanvas::getFigureInstanceEnumerator(void)
-{
-  return new CFigureInstanceEnumerator(this);
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-/**
- * Determines whether the given position corresponds to any of the parts (body, handles) of a selection decoration.
- * This test is quite fast and can be used for cursor feedback and such. This method is simply forwarded to
- * CFeedbackLayer::getSelectionInfo after modelview and projection matrix are constructed with the current settings.
- *
- * @param X The horizontal mouse position in window coordinates.
- * @param Y The vertical mouse position in window coordinate.
- * @return One of the direction flags.
- */
-TGCDirection CGenericCanvas::getSelectionInfo(const float X, const float Y)
-{
-  float Zoom = 1;
-
-  if (FCurrentView != NULL)
-  {
-    FCurrentView->applyTransformations(true);
-    Zoom = FCurrentView->feedbackZoom();
-  };
-
-  return FFeedbackLayer->getSelectionInfo(X, Y, Zoom);
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-/**
- * Tells the selection layer to recompute the bounds of the selection decoration for the given figure instance.
- *
- * @param Instance The figure instance for which to recompute the selection decoration.
- */
-void CGenericCanvas::invalidateSelectionBounds(CFigureInstance* Instance)
-{
-  FFeedbackLayer->invalidateBounds(Instance);
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
 /**
  * Returns the first layer with the given name from the layers collection.
  *
@@ -637,6 +615,8 @@ CLayer* CGenericCanvas::layerByName(const char* name)
  */
 TGCVariant CGenericCanvas::property(const char* name, unsigned int index)
 {
+  // TODO: check if all properties are properly handled.
+
   TGCVariant result;
 
   switch (getContainerID(name))
@@ -647,15 +627,13 @@ TGCVariant CGenericCanvas::property(const char* name, unsigned int index)
         {
           case GC_PROPERTY_NAME:
             {
-              result.type = GC_VAR_STRING;
-              result.s = utf16ToUtf8(FName);
+              result = utf16ToUtf8(FName);
               
               break;
             };
           case GC_PROPERTY_DESCRIPTION:
             {
-              result.type = GC_VAR_STRING;
-              result.s = "A generic 2D canvas based on OpenGL.";
+              result = "A generic 2D canvas based on OpenGL.";
               
               break;
             };
@@ -664,8 +642,8 @@ TGCVariant CGenericCanvas::property(const char* name, unsigned int index)
               // There is no owner for the canvas in the canvas library itself.
               // However we recognize the request for it by returning NULL
               // instead of "unknown property".
-              result.type = GC_VAR_OBJECT;
-              
+              result = 0;
+
               break;
             };
         };
@@ -673,25 +651,19 @@ TGCVariant CGenericCanvas::property(const char* name, unsigned int index)
       };
     case GC_CONTAINER_LAYERS:
       {
-        result.type = GC_VAR_LIST;
         if (index < FLayers.size())
-          result.reference = (CGCBase*) FLayers[index];
-        
+          result = FLayers[index];
         break;
       };
     case GC_CONTAINER_VIEWS:
       {
-        result.type = GC_VAR_LIST;
         if (index < FViews.size())
-          result.reference = (CGCBase*) FViews[index];
-        
+          result = FViews[index];
         break;
       };
     case GC_CONTAINER_MODEL:
       {
-        result.type = GC_VAR_OBJECT;
-        result.reference = (CGCBase*) FModel;
-
+        result = FModel;
         break;
       };
   };
@@ -708,15 +680,13 @@ TGCVariant CGenericCanvas::property(const char* name, unsigned int index)
  * @param index If the property is a list then this is the index into that list.
  * @param value The new value of the property. Automatic conversion is performed where possible.
  */
-void CGenericCanvas::property(const char* name, unsigned int index, const TGCVariant& value)
+void CGenericCanvas::property(const char* name, unsigned int index, TGCVariant value)
 {
   switch (getPropertyID(name))
   {
-    // There are no simple properties in the canvas that can be changed.
-    // GC_PROPERTY_NAME is only a placeholder to easy future enhancements.
     case GC_PROPERTY_NAME:
       {
-        FName = utf8ToUtf16(variantToString(value));
+        FName = utf8ToUtf16(value);
         break;
       };
   };
@@ -734,18 +704,6 @@ void CGenericCanvas::refresh(void)
 
 //----------------------------------------------------------------------------------------------------------------------
 
-/**
- * Removes the given figure instance from the current selection. This call is simply forwared to the selection layer.
- *
- * @param Instance The figure instance to be removed from the selection. Nothing happens if it isn't selected.
- */
-void CGenericCanvas::removeFromSelection(CFigureInstance* Instance)
-{
-  FFeedbackLayer->removeFromSelection(Instance);
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
 /**
  * Removes the given layer from the internal layer list. The layer itself will not be destroyed, just removed.
  *
@@ -769,6 +727,20 @@ void CGenericCanvas::removeLayer(CLayer* layer)
 
 //----------------------------------------------------------------------------------------------------------------------
 
+/**
+ * Removes the style with the given name (UTF-8 encoded) from the canvas. Figure elements which used this style
+ * will be left without visual appearence then.
+ *
+ * @param name The name of a style (encoded in UTF-8).
+ */
+void CGenericCanvas::removeStyle(const char* name)
+{
+  wstring styleName = utf8ToUtf16(name);
+  FModel->removeStyle(styleName);
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 /**
  * Removes the given view from the internal list.
  *
@@ -785,6 +757,7 @@ void CGenericCanvas::removeView(CGCView* View)
     {
       (*iterator)->removeListener(&FListener);
       FViews.erase(iterator);
+      change(View, GC_CHANGE_CANVAS_REMOVE_VIEW);
       break;
     };
 
@@ -816,9 +789,6 @@ void CGenericCanvas::render(void)
       if (FCurrentView != NULL)
         FCurrentView->render();
 
-      // The feedback layer uses the transformation of the current view!
-      TBoundingBox dummy;
-      FFeedbackLayer->render(1, dummy);
       endUpdate();
     }
     catch(...)
@@ -834,84 +804,27 @@ void CGenericCanvas::render(void)
 //----------------------------------------------------------------------------------------------------------------------
 
 /**
- * Preparation function for CFeedbackLayer::resizeFiguresStart.
+ * Sets the path to be used by the texture manager for loading textures.
+ *
+ * @param path The UTF-8 encoded texture path.
  */
-void CGenericCanvas::resizeFiguresStart(int X, int Y, TGCDirection Direction)
+void CGenericCanvas::setTexturePath(const char* path)
 {
-  // Initialize our transformation queue for convertion of the given screen coordinates into layer
-  // coordinates (which happens in the feedback layer).
-  if (FCurrentView != NULL)
-    FCurrentView->applyTransformations(true);
-
-  FFeedbackLayer->resizeFiguresStart(X, Y, Direction);
+  // The texture manager is a singleton.
+  FModel->setTexturePath(path);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
 
 /**
- * Preparation function for CFeedbackLayer::resizeFiguresStop.
+ * Returns a flag telling the calling whether a certain OpenGL extension is supported.
+ *
+ * @param extension The extension to check.
+ * @return true or false, depending on the support for this extension.
  */
-void CGenericCanvas::resizeFiguresStop(void)
+bool CGenericCanvas::supportsExtension(TOglExtension extension)
 {
-  FFeedbackLayer->resizeFiguresStop();
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-/**
- * Preparation function for CFeedbackLayer::resizeFiguresTo.
- */
-void CGenericCanvas::resizeFiguresTo(int X, int Y)
-{
-  if (FCurrentView != NULL)
-    FCurrentView->applyTransformations(false);
-
-  FFeedbackLayer->resizeFiguresTo(X, Y);
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-/**
- * Preparation function for CFeedbackLayer::rubberbandResize.
- */
-void CGenericCanvas::rubberbandResize(int X, int Y, TRBSelectionAction Action)
-{
-  if (FCurrentView != NULL)
-    FCurrentView->applyTransformations(true);
-
-  FFeedbackLayer->rubberbandResize(X, Y, Action);
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-/**
- * Preparation function for CFeedbackLayer::rubberbandStart.
- */
-void CGenericCanvas::rubberbandStart(TRubberbandStyle Style, int X, int Y, bool clearSelection)
-{
-  // Initialize our transformation queue for conversion of the given screen coordinates into layer
-  // coordinates (which happens in the feedback layer).
-  if (FCurrentView != NULL)
-    FCurrentView->applyTransformations(true);
-
-  FFeedbackLayer->rubberbandStart(Style, X, Y, clearSelection);
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-/**
- * Preparation function for CFeedbackLayer::rubberbandStop.
- */
-void CGenericCanvas::rubberbandStop(void)
-{
-  FFeedbackLayer->rubberbandStop();
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-void CGenericCanvas::showSelection(bool visible)
-{
-  FFeedbackLayer->visible(visible);
+  return FExtensions[extension];
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -925,13 +838,13 @@ void CGenericCanvas::showSelection(bool visible)
 CGCView* CGenericCanvas::viewByName(const char* name)
 {
   CGCView* result = NULL;
-
+  wstring viewName = utf8ToUtf16(name);
   for (CViews::const_iterator iterator = FViews.begin(); iterator != FViews.end(); ++iterator)
   {
-    CGCView* View = *iterator;
-    if (View->FName == name)
+    CGCView* view = *iterator;
+    if (view->FName == viewName)
     {
-      result = View;
+      result = view;
       break;
     };
   };
