@@ -15,56 +15,65 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /**
- * @file myx_gc_model.cpp 
- * @brief Implementation of the model that manages the visual representation in the generic canvas.
- * 
+ * @file myx_gc_font_manager.cpp 
+ * @brief Implements the global font manager class that handles all text output related tasks.
  */
 
-#include "myx_gc.h"
+#ifdef __APPLE__
+#include <CoreServices/CoreServices.h>
+#include <Carbon/Carbon.h>
+#endif
 
 #include "myx_gc_font_manager.h"
-#include "myx_gc_utilities.h"
+
+//#define FONTCLASS FTGLPixmapFont
+#define FONTCLASS FTGLBitmapFont
 
 //----------------------------------------------------------------------------------------------------------------------
 
-// Converts the given string into a font weight value.
-// Allowed values are: normal | bold | bolder | lighter | 100 | 200 | 300| 400 | 500 | 600 | 700 | 800 | 900 | inherit
-int convertFontWeight(const string Weight)
+/**
+ * Converts the given string into a font weight value.
+ * Allowed values are: normal | bold | bolder | lighter | 100 | 200 | 300| 400 | 500 | 600 | 700 | 800 | 900 | inherit
+ *
+ * @param weight A string containing a font weight specification.
+ * @return A font weight value corresponding to the given description.
+ */
+int convertFontWeight(const string weight)
 {
-  int Result = 0;
+  int result = 0;
 #ifdef USE_FONTCONFIG
-  if (Weight == "normal")
-    Result = FC_WEIGHT_NORMAL;
-  else if (Weight == "bold")
-    Result = FC_WEIGHT_BOLD;
-  else if (Weight == "bolder")
-    Result = FC_WEIGHT_EXTRABOLD;
-  else if (Weight == "lighter")
-    Result = FC_WEIGHT_LIGHT;
-  else if (Weight == "inherit")
-    Result = FC_WEIGHT_NORMAL;
+  if (weight == "normal")
+    result = FC_WEIGHT_NORMAL;
+  else if (weight == "bold")
+    result = FC_WEIGHT_BOLD;
+  else if (weight == "bolder")
+    result = FC_WEIGHT_EXTRABOLD;
+  else if (weight == "lighter")
+    result = FC_WEIGHT_LIGHT;
+  else if (weight == "inherit")
+    result = FC_WEIGHT_NORMAL;
   else
-    Result = atoi(Weight.c_str());
+    result = atoi(weight.c_str());
 #else // !USE_FONTCONFIG
-  if (Weight == "normal")
-    Result = 400;
+  if (weight == "normal")
+    result = 400;
   else
-    if (Weight == "bold")
-      Result = 700;
+    if (weight == "bold")
+      result = 700;
     else
-      if (Weight == "bolder")
-        Result = 800;
+      if (weight == "bolder")
+        result = 800;
       else
-        if (Weight == "lighter")
-          Result = 300;
+        if (weight == "lighter")
+          result = 300;
         else
-          if (Weight == "inherit")
-            Result = 400;
+          if (weight == "inherit")
+            result = 400;
           else
-            Result = atoi(Weight.c_str());
+            result = atoi(weight.c_str());
 #endif
 
-  return Result;
+  return result;
 }
 
 //----------------- CFontManager ---------------------------------------------------------------------------------------
@@ -117,7 +126,7 @@ void unlockFontManager(void)
  */
 CFontManager::CFontManager(void)
 {
-  FUseLocalDisplayLists = true;     
+  FUseLocalDisplayLists = true; // Set only to true if the text is as a whole not rendered in a display list.    
 
 #ifndef USE_FONTCONFIG
   // Fill our font file list with some predefined font files.
@@ -166,12 +175,12 @@ CFontManager::CFontManager(void)
   entry->useCount = 1;
   FFiles["Tahoma"] = entry;
 
-  // Tahoma-Bold, not an own font but Adobe Illustrator denotes it so.
+  // Tahoma-bold, not an own font but Adobe Illustrator denotes it so.
   entry = new FontFileEntry;
   entry->entries[false][false] = "Tahomabd.ttf"; entry->entries[false][true] = "Tahomabd.ttf"; 
   entry->entries[true][false] = "Tahomabdi.ttf";  entry->entries[true][true] = "Tahomabdi.ttf";
   entry->useCount = 1;
-  FFiles["Tahoma-Bold"] = entry;
+  FFiles["Tahoma-bold"] = entry;
 
   // Times New Roman
   entry = new FontFileEntry;
@@ -198,6 +207,7 @@ CFontManager::CFontManager(void)
 #endif // !USE_FONTCONFIG
 }
 
+
 //----------------------------------------------------------------------------------------------------------------------
 
 /**
@@ -213,9 +223,9 @@ CFontManager::~CFontManager(void)
 /**
  * Determines platform dependantly the full path of a font file depending on some characteristics.
  *
- * @param Family The font family (e.g. Arial, Tahoma, Verdana).
- * @param Style The font style (normal, italic).
- * @param Weight The "boldness" of the font in the range of [100..900]. Currently values <= 400 are
+ * @param family The font family (e.g. Arial, Tahoma, Verdana).
+ * @param style The font style (normal, italic).
+ * @param weight The "boldness" of the font in the range of [100..900]. Currently values <= 400 are
  *               considered als normal font, everything else as bold.
  *
  * @return The full path and file name of a font that represents the given characteristics.
@@ -225,30 +235,30 @@ CFontManager::~CFontManager(void)
  *          default. If this files is also not present on the system then the FTGL lib will throw an exception.
  * @note The returned file name is ANSI encoded as FTGL expects it so.
  */
-string CFontManager::getFontFile(string Family, string Style, int Weight)
+string CFontManager::getFontFile(string family, string style, int weight)
 {
-  string Filename;
+  string filename;
 
 #ifdef USE_FONTCONFIG
   FcPattern *pattern= FcPatternCreate();
   FcResult result;
   int slant;
   
-  if (Style == "italic")
+  if (style == "italic")
     slant= FC_SLANT_ITALIC;
-  else if (Style == "oblique")
+  else if (style == "oblique")
     slant= FC_SLANT_OBLIQUE;
   else
     slant= FC_SLANT_ROMAN;
   
-  FcPatternAddString(pattern, FC_FAMILY, (FcChar8*)Family.c_str());
+  FcPatternAddString(pattern, FC_FAMILY, (FcChar8*)family.c_str());
   FcPatternAddInteger(pattern, FC_SLANT, slant);
-  FcPatternAddInteger(pattern, FC_WEIGHT, Weight);
+  FcPatternAddInteger(pattern, FC_WEIGHT, weight);
 
   if (!FcConfigSubstitute(FcConfigGetCurrent(), pattern, FcMatchPattern))
   {
     g_warning("Error calling FcConfigSubstitute() on font '%s %s'",
-              Family.c_str(), Style.c_str());
+              family.c_str(), style.c_str());
     FcPatternDestroy(pattern);
   }
   else
@@ -260,7 +270,7 @@ string CFontManager::getFontFile(string Family, string Style, int Weight)
     fpat= FcFontMatch(FcConfigGetCurrent(), pattern, &result);
     if (result != FcResultMatch)
       g_warning("Error looking up for font '%s %s'",
-                Family.c_str(), Style.c_str());
+                family.c_str(), style.c_str());
     else
     {
       FcValue value;
@@ -269,19 +279,103 @@ string CFontManager::getFontFile(string Family, string Style, int Weight)
       {
         g_message("FILE --> %s", value.u.s);
       
-        Filename= (char*)value.u.s;
+        filename= (char*)value.u.s;
       }
       FcPatternDestroy(fpat);
     }
     FcPatternDestroy(pattern);
   }
+#elif defined(__APPLE__)
+  FSSpec pathSpec;
+  FSRef fsref;
+  FT_Long index;
+  FMFontFamilyIterator famIter;
+  OSStatus status;
+  FMFont match= 0;
+  FMFontFamily ffamily= 0;
+  bool wants_bold= weight > 400;
+  bool wants_italic= style == "italic" || style == "Oblique";
+
+  status= FMCreateFontFamilyIterator(NULL, NULL, 
+                                     kFMUseGlobalScopeOption,
+                                     &famIter);
+    
+  index = 0;
+  while (status == 0 && !match)
+  {
+    status = FMGetNextFontFamily(&famIter, &ffamily);
+    if (status == 0)
+    {
+      int stat2;
+      FMFontFamilyInstanceIterator instIter;
+      Str255 famNameStr;
+      char famName[256];
+
+      /* get the family name */
+      FMGetFontFamilyName(ffamily, famNameStr);
+      CopyPascalStringToC(famNameStr, famName);
+      
+      if (strcasecmp(famName, family.c_str()) == 0)
+      {      
+        /* iterate through the styles */
+        FMCreateFontFamilyInstanceIterator(ffamily, &instIter);
+      
+        index= 0;
+        stat2= 0;
+        while (stat2 == 0 && !match)
+        {
+          FMFontStyle style;
+          FMFontSize size;
+          FMFont font;
+          
+          stat2= FMGetNextFontFamilyInstance(&instIter, &font,
+                                             &style, &size );
+          if (stat2 == 0 && size == 0)
+          {
+            bool f_bold= (style & bold)!=0;
+            bool f_italic= (style & italic)!=0;
+
+            if (f_bold == wants_bold && f_italic == wants_italic)
+            {
+              match= font;
+            }
+            else
+              index++;
+          }
+        }
+      }
+      FMDisposeFontFamilyInstanceIterator(&instIter);
+    }
+  }
+  
+  FMDisposeFontFamilyIterator(&famIter);
+  
+  if (match)
+  {
+    FMGetFontContainer(match, &pathSpec);
+    if (FSpMakeFSRef(&pathSpec, &fsref) == 0)
+    {  
+      UInt8 thePath[1024];
+      if (FSRefMakePath(&fsref, thePath, sizeof(thePath)) == 0)
+      {
+        filename= (char*)thePath;
+      }
+    }    
+  }
+  
+//  if (filename.empty() && family != "Arial")
+//    filename= getFontFile("Arial", "", 0);
+  if (FFontResourcesPath.empty())
+    filename="/Library/Fonts/Vera.ttf";
+  else
+    filename=FFontResourcesPath+"/Vera.ttf";
 #else // !USE_FONTCONFIG
   FontFileEntry* entry;
 
-  // Adobe illustrator (and probably others too) put the family name in quotes.
-  if (Family[0] == '"' || Family[0] == '\'')
-    Family = Family.substr(1, Family.size() - 2);
-  FontFiles::iterator iterator = FFiles.find(Family);
+  // Adobe Illustrator (and probably others too) surround the family name with quotes.
+  if (family[0] == '"' || family[0] == '\'')
+    family = family.substr(1, family.size() - 2);
+  FontFiles::iterator iterator = FFiles.find(family);
   if (iterator == FFiles.end())
   {
     // Family is unknown, use Arial as default.
@@ -292,73 +386,72 @@ string CFontManager::getFontFile(string Family, string Style, int Weight)
     entry = iterator->second;
   };
 
-  bool Italic = (Style == "italic") || (Style == "oblique");
-  bool Bold = Weight > 400;
+  bool italic = (style == "italic") || (style == "oblique");
+  bool bold = weight > 400;
 
   LPITEMIDLIST pidl;
   wchar_t Path[MAX_PATH];
   if (SHGetSpecialFolderLocation(0, CSIDL_FONTS, &pidl) == NOERROR)
   {
     if (SHGetPathFromIDList(pidl, Path))
-      Filename = utf16ToANSI(Path);
+      filename = utf16ToANSI(Path);
     
     // We are required to free the returned pidl via the shell's IMalloc interface.
     LPMALLOC pMalloc;
     SHGetMalloc(&pMalloc);
     pMalloc->Free(pidl);
   }
-  string FontFile = entry->entries[Italic][Bold];
-  string Testfile = Filename + '/' + FontFile;
+  string fontFile = entry->entries[italic][bold];
+  string testFile = filename + '/' + fontFile;
 
-  FILE* File = fopen(Testfile.c_str(), "rb");
-  if (File != NULL)
+  FILE* file = fopen(testFile.c_str(), "rb");
+  if (file != NULL)
   {
-    fclose(File);
-    Filename = Testfile;
+    fclose(file);
+    filename = testFile;
   }
   else
   {
     // There is no file for the required style. Try the base entry.
-    FontFile = entry->entries[false][false];
-    Testfile = Filename + '/' + FontFile;
-    File = fopen(Testfile.c_str(), "rb");
-    if (File != NULL)
+    fontFile = entry->entries[false][false];
+    testFile = filename + '/' + fontFile;
+    file = fopen(testFile.c_str(), "rb");
+    if (file != NULL)
     {
-      fclose(File);
-      Filename = Testfile;
+      fclose(file);
+      filename = testFile;
     }
+    else
     {
       // That's bad luck. No standard file either. Use the default font.
       entry = FFiles["Arial"];
-      Filename += '/' + entry->entries[false][false];
+      filename += '/' + entry->entries[false][false];
     };
   };
 #endif // !USE_FONTCONFIG
-  return Filename;
+  return filename;
 }
 
 //----------------------------------------------------------------------------------------------------------------------
 
-void CFontManager::boundingBox(const wstring& Output, string FontFamily, string FontStyle, int Weight, int FontSize, 
+void CFontManager::boundingBox(const wstring& Output, string FontFamily, string FontStyle, int weight, int FontSize, 
                                TBoundingBox* Box)
 {
   FTFont* Font;
-  string Key = createLookupKey(FontFamily, FontStyle, Weight);
+  string Key = createLookupKey(FontFamily, FontStyle, weight, FontSize);
   Fonts::iterator iterator = FFonts.find(Key);
 
   if (iterator == FFonts.end())
   {
-    string FontFile = getFontFile(FontFamily, FontStyle, Weight);
-    Font = new FTGLPixmapFont(FontFile.c_str());
+    string fontFile = getFontFile(FontFamily, FontStyle, weight);
+    Font = new FONTCLASS(fontFile.c_str());
+    Font->FaceSize(FontSize);
     Font->UseDisplayList(FUseLocalDisplayLists);
     FFonts[Key] = Font;
   }
   else
     Font = iterator->second;
 
-  if ((int)Font->FaceSize() != FontSize)
-    Font->FaceSize(FontSize);
-
   Font->BBox(Output.c_str(), Box->upper.x, Box->upper.y, Box->upper.z, Box->lower.x, Box->lower.y, Box->lower.z);
 }
 
@@ -391,25 +484,26 @@ void CFontManager::clear(void)
 
 /**
  * Creates a string that can be used for lookup in the font list.
- * Either parameter can be NULL (or 0 for Weight) causing the manager to use the default values for each missing 
+ * Either parameter can be NULL (or 0 for weight) causing the manager to use the default values for each missing 
  * parameter. See header file for the list of default values.
  *
- * @param Family The font family (Arial, Courier etc.)
- * @param Style The font style.
- * @param Weight The boldness of the font (400 for normal).
+ * @param family The font family (Arial, Courier etc.)
+ * @param style The font style.
+ * @param weight The boldness of the font (400 for normal).
+ * @param FontSize The font size.
  */
-string CFontManager::createLookupKey(const string& Family, const string& Style, int Weight)
+string CFontManager::createLookupKey(const string& family, const string& style, int weight, int FontSize)
 {
   char WeightString[30];
 
   // Construct a lookup string out of the font properties.
-  string Key = Family + Style;
+  string Key = family + style;
   #ifdef _WINDOWS
     // Microsoft has confirmed that there is no guaranteed NULL termination (see MSDN for details).
     // Hence we better add one.
-    _snprintf(WeightString, sizeof(WeightString), "%i\0", Weight);
+    _snprintf(WeightString, sizeof(WeightString), "%i:%i\0", weight, FontSize);
   #else
-    snprintf(WeightString, sizeof(WeightString), "%i", Weight);
+    snprintf(WeightString, sizeof(WeightString), "%i:%i", weight, FontSize);
   #endif // #ifdef _WINDOWS
   Key += WeightString;
 
@@ -418,16 +512,24 @@ string CFontManager::createLookupKey(const string& Family, const string& Style,
 
 //----------------------------------------------------------------------------------------------------------------------
 
-void CFontManager::textOut(const wstring& Output, string FontFamily, string FontStyle, int Weight, int FontSize)
+void CFontManager::setFontResourcesPath(const string& path)
+{
+  FFontResourcesPath= path;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+void CFontManager::textOut(const wstring& Output, string FontFamily, string FontStyle, int weight, int FontSize)
 {
   FTFont* Font;
-  string Key = createLookupKey(FontFamily, FontStyle, Weight);
-  Fonts::iterator iterator = FFonts.find(Key);
+  string Key = createLookupKey(FontFamily, FontStyle, weight, FontSize);
+  Fonts::iterator iterator = FFonts.find(Key); 
 
   if (iterator == FFonts.end())
   {
-    string FontFile = getFontFile(FontFamily, FontStyle, Weight);
-    Font = new FTGLPixmapFont(FontFile.c_str());
+    string fontFile = getFontFile(FontFamily, FontStyle, weight);
+    Font = new FONTCLASS(fontFile.c_str());
+    Font->FaceSize(FontSize);
     Font->UseDisplayList(FUseLocalDisplayLists);
     
     FFonts[Key] = Font;
@@ -435,8 +537,6 @@ void CFontManager::textOut(const wstring& Output, string FontFamily, string Font
   else
     Font = iterator->second;
 
-  if ((int) Font->FaceSize() != FontSize)
-    Font->FaceSize(FontSize);
   Font->Render(Output.c_str());
 }
 
