@@ -20,16 +20,19 @@
  * 
  */
 
-#include "myx_gc.h"
-
-#include "myx_gc_figure.h"
 #include "myx_gc_model.h"
-#include "myx_gc_canvas.h"
-#include "myx_gc_utilities.h"
+#include "myx_gc_texture.h"
 
-//----------------------------------------------------------------------------------------------------------------------
+// Default values for texturing.
+extern const string defaultTextureWrapMode("clamp");     
+extern const string defaultTextureMinFilter("nearest"); 
+extern const string defaultTextureMagFilter("nearest");      
+extern const int defaultTextureDimensions = 2;
+extern const string defaultTextureMode("decal");
 
-void CModelFigureListener::onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason)
+//----------------- CFigureListener ------------------------------------------------------------------------------------
+
+void CGCModel::CFigureListener::onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason)
 {
   if (model != NULL)
     model->change(origin, reason);
@@ -37,19 +40,25 @@ void CModelFigureListener::onChange(CGCBase* sender, CGCBase* origin, TGCChangeR
 
 //----------------------------------------------------------------------------------------------------------------------
 
-void CModelFigureListener::onDestroy(CGCBase* sender)
+void CGCModel::CFigureListener::onDestroy(CGCBase* sender)
 {
   if (model != NULL)
   {
-    CFigure* figure = (CFigure*) sender;
-    if (figure != NULL)
-      model->removeFigure(figure);
+    if (strcmp(sender->className(), "CFigure") == 0)
+    {
+      CFigure* figure = static_cast<CFigure*>(sender);
+      if (figure != NULL)
+        model->removeFigure(figure);
+    }
+    else
+      if (strcmp(sender->className(), "CConnection") == 0)
+        model->removeConnection((CConnection*) sender);
   };
 }
   
 //----------------------------------------------------------------------------------------------------------------------
 
-void CModelFigureListener::onError(CGCBase* sender, CGCBase* origin, const char* message)
+void CGCModel::CFigureListener::onError(CGCBase* sender, CGCBase* origin, const char* message)
 {
   if (model != NULL)
     model->error(origin, message);
@@ -57,9 +66,8 @@ void CModelFigureListener::onError(CGCBase* sender, CGCBase* origin, const char*
 
 //----------------- CGCModel -------------------------------------------------------------------------------------------
 
-CGCModel::CGCModel(CGenericCanvas* Controller): CGCBase(Controller)
+CGCModel::CGCModel(CGenericCanvas* canvas): CGCBase(canvas)
 {
-  FCanvas = Controller;
   _className = "CGCModel";
   FListener.model = this;
 }
@@ -70,9 +78,11 @@ CGCModel::~CGCModel(void)
 {
   setDestroying();
 
+  clearConnections();
   clearFigures();
   clearLayouts();
   clearStyles();
+  clearTextures();
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -88,16 +98,6 @@ void CGCModel::addFigure(CFigure* figure)
 
 //----------------------------------------------------------------------------------------------------------------------
 
-/**
- * Checks if there is any pending OpenGL error (e.g. from style creation). Call is forwared to the canvas and handeled there.
- */
-void CGCModel::checkError(void)
-{
-  FCanvas->checkError();
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
 /**
  * Removes the given figure from the internal figures list.
  *
@@ -121,6 +121,20 @@ void CGCModel::removeFigure(CFigure* figure)
 
 //----------------------------------------------------------------------------------------------------------------------
 
+/**
+ * Removes all connections.
+ */
+void CGCModel::clearConnections(void)
+{
+  beginUpdate();
+  for (CConnectionList::iterator iterator = FConnections.begin(); iterator != FConnections.end(); ++iterator)
+    delete *iterator;
+  FConnections.clear();
+  endUpdate();
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 /**
  * Clears the model, that is, the figures defined in this model.
  */
@@ -167,6 +181,41 @@ void CGCModel::clearStyles(void)
 
 //----------------------------------------------------------------------------------------------------------------------
 
+/**
+ * Clears all defined textures.
+ *
+ * @note: Deleting all textures means to leave all styles without their assigned textures.
+ */
+void CGCModel::clearTextures(void)
+{
+  beginUpdate();
+  for (CTextureList::iterator iterator = FTextures.begin(); iterator != FTextures.end(); ++iterator)
+    delete iterator->second;
+  FTextures.clear();
+  endUpdate();
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+/**
+ * Creates a connection object for a logical connection between both end points. This connection, like a figure,
+ * cannot be displayed on its own. It needs a connection instance, which is placed on a connection layer in a view.
+ * Connections aren't directed so there is no start point. They just have two end points.
+ *
+ * @param endPoint1 One endpoint of the connection.
+ * @param endPoint1 Another endpoint of the connection
+ */
+CConnection* CGCModel::createConnection(CFigure* endPoint1, CFigure* endPoint2)
+{
+  CConnection* result = new CConnection(canvas(), endPoint1, endPoint2);
+  result->addListener(&FListener);
+  FConnections.push_back(result);
+
+  return result;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 /**
  * Creates a new figure and puts in into the internal list.
  *
@@ -194,7 +243,7 @@ CFigure* CGCModel::createFigure(wstring type, wstring layoutClass)
  */
 CFigureTemplate* CGCModel::createLayout(wstring type, wstring layoutClass)
 {
-  CFigureTemplate* result = getLayout(type, layoutClass);
+  CFigureTemplate* result = layout(type, layoutClass);
 
   if (result == NULL)
   {
@@ -207,12 +256,103 @@ CFigureTemplate* CGCModel::createLayout(wstring type, wstring layoutClass)
 
 //----------------------------------------------------------------------------------------------------------------------
 
+/**
+ * Creates a new texture entry and adds the entry to the texture list. No image data is loaded yet as this will happen 
+ * when the texture is used the first time.
+ *
+ * @param lodData A list of level identifiers and names of files, which contain the image data for that level.
+ * @param id The identifier (name) of the texture.
+ * @param wrapH Horizontal wrap mode as string.
+ * @param wrapV Vertical wrap mode as string.
+ * @param MinificationFilter The filter mode for minification as string.
+ * @param MagnificationFilter The filter mode for magnification as string.
+ * @param dimensions The number of dimensions of the image data (must be 1 or 2).
+ * @param mode The mode how the texture must be applied as string.
+ * return The newly created texture object.
+ */
+CGCTexture* CGCModel::createTexture(const TLODList& lodData, const wstring& id, const string& wrapH, const string& wrapV, 
+                                    const string& minificationFilterStr, const string& magnificationFilterStr,
+                                    int dimensions, const string& mode)
+{
+  static map<string, GLenum> mapper;
+  map<string, GLenum>::const_iterator iterator;
+
+  // Fill our lookup table first if not yet done.
+  if (mapper.size() == 0)
+  {
+    mapper["clamp"] = GL_CLAMP;
+    mapper["clamp-to-border"] = GL_CLAMP_TO_BORDER;
+    mapper["clamp-to-edge"] = GL_CLAMP_TO_EDGE;
+    mapper["repeat"] = GL_REPEAT;
+    mapper["nearest"] = GL_NEAREST;
+    mapper["linear"] = GL_LINEAR;
+    mapper["nearest-mipmap-nearest"] = GL_NEAREST_MIPMAP_NEAREST;
+    mapper["linear-mipmap-nearest"] = GL_LINEAR_MIPMAP_NEAREST;
+    mapper["nearest-mipmap-linear"] = GL_NEAREST_MIPMAP_LINEAR;
+    mapper["linear-mipmap-linear"] = GL_LINEAR_MIPMAP_LINEAR;
+    mapper["decal"] = GL_DECAL;
+    mapper["modulate"] = GL_MODULATE;
+    mapper["blend"] = GL_BLEND;
+    mapper["replace"] = GL_REPLACE;
+  };
+
+  GLenum wrapModeS = GL_CLAMP;
+  iterator = mapper.find(wrapH);
+  if (iterator != mapper.end())
+    wrapModeS = iterator->second;
+
+  GLenum wrapModeT = GL_CLAMP;
+  iterator = mapper.find(wrapV);
+  if (iterator != mapper.end())
+    wrapModeT = iterator->second;
+
+  GLenum minFilter = GL_NEAREST;
+  iterator = mapper.find(minificationFilterStr);
+  if (iterator != mapper.end())
+    minFilter = iterator->second;
+
+  GLenum magFilter = GL_NEAREST;
+  iterator = mapper.find(magnificationFilterStr);
+  if (iterator != mapper.end())
+    magFilter = iterator->second;
+
+  GLenum textureMode = GL_DECAL;
+  iterator = mapper.find(mode);
+  if (iterator != mapper.end())
+    textureMode = iterator->second;
+
+  TLODList prefixedLodData;
+  for (TLODList::const_iterator path= lodData.begin(); path != lodData.end(); ++path)
+  {
+    wstring finalPath;
+    if (path->substr(0, 2) == L"./")
+      finalPath = FTexturePath + path->substr(2);
+    else 
+      // If an absolute path is given then do not apply the texture image path.
+      if ((*path)[0] == '/' || (path->size() > 2 && (*path)[1] == ':'))
+        finalPath = *path;
+      else
+        finalPath = FTexturePath + *path;
+
+    prefixedLodData.push_back(finalPath);
+  }
+  CGCTexture* result = new CGCTexture(prefixedLodData, id, wrapModeS, wrapModeT, minFilter, magFilter, dimensions, textureMode);
+  FTextures[id] = result;
+
+  return result;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 /**
  * Same as the getLayout(wstring, wstring) but for UTF-8 encoded strings.
  */
-CFigureTemplate* CGCModel::getLayout(const char* type, const char* layoutClass)
+CFigureTemplate* CGCModel::layout(const char* type, const char* layoutClass)
 {
-  return getLayout(utf8ToUtf16(type), utf8ToUtf16(layoutClass));
+  if (layoutClass == NULL)
+    return layout(utf8ToUtf16(type), L"");
+  else
+    return layout(utf8ToUtf16(type), utf8ToUtf16(layoutClass));
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -224,7 +364,7 @@ CFigureTemplate* CGCModel::getLayout(const char* type, const char* layoutClass)
  * @param layoutClass An additional criterion specifying a certain layout arrangement. This value is app. specific.
  * @return The corresponding layout entry if it exists or NULL if not.
  */
-CFigureTemplate* CGCModel::getLayout(wstring type, wstring layoutClass)
+CFigureTemplate* CGCModel::layout(wstring type, wstring layoutClass)
 {
   CFigureTemplate* result = NULL;
   pair<CLayoutList::iterator, CLayoutList::iterator> range;
@@ -232,15 +372,20 @@ CFigureTemplate* CGCModel::getLayout(wstring type, wstring layoutClass)
   range = FLayouts.equal_range(type);
   CLayoutList::iterator start = range.first;
   CLayoutList::iterator stop = range.second;
-  while (start != stop)
-  {
-    if (start->second->layoutClass() == layoutClass)
+
+  // If no layout class is given then use the first one that was defined for the given type.
+  if (start!= stop && layoutClass.size() == 0)
+    result = start->second;
+  else
+    while (start != stop)
     {
-      result = start->second;
-      break;
+      if (start->second->layoutClass() == layoutClass)
+      {
+        result = start->second;
+        break;
+      };
+      ++start;
     };
-    ++start;
-  };
 
   return result;
 }
@@ -268,22 +413,17 @@ TGCVariant CGCModel::property(const char* name, unsigned int index)
         {
           case GC_PROPERTY_NAME:
             {
-              result.type = GC_VAR_STRING;
-              result.s = "canvas model";
-              
+              result = "canvas model";
               break;
             };
           case GC_PROPERTY_DESCRIPTION:
             {
-              result.type = GC_VAR_STRING;
-              result.s = "The central model of the canvas containing all styles, layouts and figures.";
-              
+              result = "The central model of the canvas containing all styles, layouts and figures.";
               break;
             };
           case GC_PROPERTY_OWNER:
             {
-              result.type = GC_VAR_OBJECT;
-              result.reference = FCanvas;
+              result = canvas();
               break;
             };
         };
@@ -292,15 +432,13 @@ TGCVariant CGCModel::property(const char* name, unsigned int index)
       };
     case GC_CONTAINER_FIGURES:
       {
-        result.type = GC_VAR_LIST;
         if (index < FFigures.size())
-          result.reference = (CGCBase*) FFigures[index];
+          result = FFigures[index];
 
         break;
       };
     case GC_CONTAINER_STYLES:
       {
-        result.type = GC_VAR_LIST;
         if (index < FStyles.size())
         {
           CStyleList::iterator iterator = FStyles.begin();
@@ -309,14 +447,13 @@ TGCVariant CGCModel::property(const char* name, unsigned int index)
             ++iterator;
             --index;
           };
-          result.reference = (CGCBase*) iterator->second;
+          result = iterator->second;
         };
 
         break;
       };
     case GC_CONTAINER_LAYOUTS:
       {
-        result.type = GC_VAR_LIST;
         if (index < FLayouts.size())
         {
           CLayoutList::iterator iterator = FLayouts.begin();
@@ -325,7 +462,7 @@ TGCVariant CGCModel::property(const char* name, unsigned int index)
             ++iterator;
             --index;
           };
-          result.reference = (CGCBase*) iterator->second;
+          result = iterator->second;
         };
 
         break;
@@ -344,36 +481,128 @@ TGCVariant CGCModel::property(const char* name, unsigned int index)
  * @param index If the property is a list then this is the index into that list.
  * @param Value The new value of the property. Automatic conversion is performed where possible.
  */
-void CGCModel::property(const char* name, unsigned int index, const TGCVariant& value)
+void CGCModel::property(const char* name, unsigned int index, TGCVariant value)
 {
   // No properties to set at this point.
 }
 
 //----------------------------------------------------------------------------------------------------------------------
 
+/**
+ * Removes the given connection from the model. The connection itself is not destroyed.
+ *
+ * @param connection The connection to be removed from this model.
+ */
+void CGCModel::removeConnection(CConnection* connection)
+{
+  if (!destroying())
+  {
+    for (CConnectionList::iterator iterator = FConnections.begin(); iterator != FConnections.end(); ++iterator)
+      if (*iterator == connection)
+      {
+        connection->removeListener(&FListener);
+        FConnections.erase(iterator);
+        break;
+      };
+  };
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+void CGCModel::removeStyle(wstring ID)
+{
+  CStyleList::iterator iterator = FStyles.find(ID);
+  if (iterator != FStyles.end())
+  {
+    delete iterator->second;
+    FStyles.erase(iterator);
+  };
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+/**
+ * Called by the canvas to set a new path from which textures are loaded.
+ *
+ * @param path The new texture path (UTF-8 encoded).
+ */
+void CGCModel::setTexturePath(const string& path)
+{
+  if (path.size() == 0)
+    FTexturePath.clear();
+  else
+  {
+    // Make sure the path ends with a slash.
+    FTexturePath = utf8ToUtf16(path);
+    if (FTexturePath[FTexturePath.size() - 1] != '/')
+      FTexturePath += '/';
+  };
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 /**
  * Returns the style entry for the given identifier. If there is no style with that name one is created.
  *
  * @param ID The identification of the style.
  * @return The corresponding style entry, can be NULL if ID is empty.
  */
-CGCStyle* CGCModel::style(wstring ID)
+CGCStyle* CGCModel::style(const wstring &ID)
 {
-  CGCStyle* Result = NULL;
+  CGCStyle* result = NULL;
 
   if (ID.size() > 0)
   {
     CStyleList::iterator iterator = FStyles.find(ID);
     if (iterator == FStyles.end())
     {
-      Result = new CGCStyle(this, ID);
-      FStyles[ID] = Result;
+      result = new CGCStyle(this, ID);
+      FStyles[ID] = result;
     }
     else
-      Result = iterator->second;
+      result = iterator->second;
   };
 
-  return Result;
+  return result;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+/**
+ * Returns 1 if a style with the give name exists.
+ *
+ * @param ID The identification of the style.
+ * @return 1 if the style exists, 0 if it does not exist
+ */
+int CGCModel::styleExists(const wstring &ID)
+{
+  int result = 0;
+
+  if (ID.size() > 0)
+  {
+    CStyleList::iterator iterator = FStyles.find(ID);
+    if (iterator != FStyles.end())
+      result = 1;
+  }
+
+  return result;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+/**
+ * Looks through the textures and tries to find one with the given name.
+ *
+ * @param name The name of the texture to find.
+ * @return The texture if there is one with the given name or NULL.
+ */
+CGCTexture* CGCModel::texture(const wstring& name)
+{
+  CTextureList::iterator iterator = FTextures.find(name);
+  if (iterator == FTextures.end())
+    return NULL;
+  else
+    return iterator->second;
 }
 
 //----------------------------------------------------------------------------------------------------------------------
