@@ -51,7 +51,7 @@ class GRTAgent
     
     void handle_client(MNSocket *client);
     
-    static int store_messages(MYX_GRT_MSGS *msgs, void *user_data);
+    static void store_messages(MYX_GRT_MSGS *msgs, void *user_data);
 
     static gpointer loop(gpointer data);
     static gpointer handle_requests(gpointer data);
