@@ -60,6 +60,9 @@ struct MYX_GRT
   MYX_GRT_INPUT_CALLBACK process_input;
   void *process_input_data;
 
+  MYX_GRT_STATUS_QUERY_CALLBACK process_status_query;
+  void *process_status_query_data;
+
 
   GHashTable *struct_icon_cache;
 
@@ -100,6 +103,9 @@ MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_add_module(MYX_GRT *grt, MYX_GRT_MODULE *m
 MYX_PUBLIC_FUNC MYX_GRT_MODULE* myx_grt_find_module(MYX_GRT *grt, char* name);
 MYX_PUBLIC_FUNC void myx_grt_remove_module(MYX_GRT *grt, MYX_GRT_MODULE *module);
 
+void myx_grt_shell_show_help(MYX_GRT *grt, const char *command);
+void myx_grt_shell_show_command_help(MYX_GRT *grt, const char *command);
+
 MYX_GRT_MODULE_LOADER *myx_builtin_init_loader(MYX_GRT *grt);
 
 void myx_grt_parse_function_spec(const char *spec, MYX_GRT_FUNCTION *func);
