@@ -17,7 +17,7 @@
 #ifndef __MYX_GRT_H__
 #define __MYX_GRT_H__
 
-#define GRT_VERSION "2.0.8 beta"
+#define GRT_VERSION "2.0.11 beta"
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
 # define __LCC__
@@ -31,8 +31,17 @@
 #include <lua.h>
 #endif
 #ifdef ENABLE_JAVA_MODULES
-#include <jni.h>
-#include <com_mysql_grt_GrtCallbackNative.h>
+# ifdef __APPLE__
+#  include <JavaVM/jni.h>
+#  ifdef FROM_XCODE
+#   include <MySQLGRT/com_mysql_grt_GrtCallbackNative.h>
+#  else
+#   include <com_mysql_grt_GrtCallbackNative.h>
+#  endif
+# else
+#  include <jni.h>
+#  include <com_mysql_grt_GrtCallbackNative.h>
+# endif
 #endif
 
 #ifdef __cplusplus
@@ -138,7 +147,10 @@ typedef enum
 
 typedef enum
 {
-  MYX_GVCR_DELETE
+  MYX_GVCR_DELETE,
+  MYX_GVCR_DICT_ITEM_CHANGE,
+  MYX_GVCR_LIST_CHANGE,
+  MYX_GVCR_LIST_ITEM_CHANGE
 } MYX_GRT_VALUE_CALLBACK_REASON;
 
 /*
@@ -185,6 +197,7 @@ typedef struct MYX_GRT_MODULE
   MYX_GRT_FUNCTION *functions;
   char *extends;
   struct MYX_GRT_MODULE_PRIVATE *priv; // private data for each loader
+  char dont_bridge_loaded_values;
 } MYX_GRT_MODULE;
 
 
@@ -281,7 +294,6 @@ typedef struct MYX_GRT_VALUE
   char *bridge_dict_key; // if this is a dict item, store the key name so the bridge can identify the property associated with the value
   unsigned int bridge_list_index; // if this is a list item, store the index so the bridge can identify the item associated with the value
 
-  MYX_GRT *listeners_grt;
   unsigned int listeners_num;
   struct MYX_GRT_VALUE_LISTENER **listeners;
 
@@ -289,6 +301,7 @@ typedef struct MYX_GRT_VALUE
 
 typedef struct MYX_GRT_VALUE_LISTENER
 {
+  MYX_GRT *listeners_grt;
   int (*function)(MYX_GRT *, MYX_GRT_VALUE *, MYX_GRT_VALUE_CALLBACK_REASON, void *);
   void *user_data;
 } MYX_GRT_VALUE_LISTENER;
@@ -331,7 +344,8 @@ typedef enum
     MYX_LUA_MODULE_TYPE,
     MYX_PYTHON_MODULE_TYPE,
     MYX_PHP_MODULE_TYPE,
-    MYX_DELPHI_MODULE_TYPE
+    MYX_DELPHI_MODULE_TYPE,
+    MYX_OBJC_MODULE_TYPE
 //    MYX_PERL_MODULE_TYPE,
 } MYX_GRT_MODULE_TYPE;
 
@@ -349,8 +363,9 @@ typedef struct MYX_GRT_AGENT_SESSION MYX_GRT_AGENT_SESSION;
  */
 
 typedef void (*MYX_GRT_PRINT_CALLBACK)(const char *, void*);
-typedef int (*MYX_GRT_MESSAGE_CALLBACK)(MYX_GRT_MSGS *msgs, void *user_data);
+typedef void (*MYX_GRT_MESSAGE_CALLBACK)(MYX_GRT_MSGS *msgs, void *user_data);
 typedef int (*MYX_GRT_INPUT_CALLBACK)(const char *caption, MYX_GRT_INPUT_OPTIONS options, const char **text, void *user_data);
+typedef int (*MYX_GRT_STATUS_QUERY_CALLBACK)(void *user_data);
 typedef void (*MYX_GRT_LOG_CALLBACK)(MYX_GRT *, int, const char *, const char*);
 typedef int (*MYX_GRT_VALUE_LISTENER_CALLBACK)(MYX_GRT *grt, MYX_GRT_VALUE *value, MYX_GRT_VALUE_CALLBACK_REASON reason, void *user_data);
 
@@ -359,6 +374,8 @@ MYX_PUBLIC_FUNC int myx_free_stringlist(MYX_STRINGLIST *stringlist);
 MYX_PUBLIC_FUNC MYX_GRT * myx_grt_initialize(int options);
 MYX_PUBLIC_FUNC void myx_grt_finalize(MYX_GRT *grt);
 
+MYX_PUBLIC_FUNC void myx_grt_catch_glib_messages(MYX_GRT *grt, int do_catch);
+
 #ifdef ENABLE_JAVA_MODULES
 JNIEXPORT jstring JNICALL Java_Grt_callGrtFunction
   (JNIEnv *, jobject, jlong, jstring, jstring, jstring);
@@ -372,9 +389,11 @@ const char *myx_grt_error_string(MYX_GRT_ERROR error);
 MYX_PUBLIC_FUNC void myx_grt_set_output_callback(MYX_GRT *grt, void *user_data,
                                  void (*process_output_func)(const char *text, void *user_data));
 MYX_PUBLIC_FUNC void myx_grt_set_message_callback(MYX_GRT *grt, void *user_data,
-                                 int (*process_message_func)(MYX_GRT_MSGS *msgs, void *user_data));
+                                 void (*process_message_func)(MYX_GRT_MSGS *msgs, void *user_data));
 MYX_PUBLIC_FUNC void myx_grt_set_input_callback(MYX_GRT *grt, void *user_data,
                                  int (*process_input_func)(const char *caption, MYX_GRT_INPUT_OPTIONS options, const char **text, void *user_data));
+MYX_PUBLIC_FUNC void myx_grt_set_status_query_callback(MYX_GRT *grt, void *user_data,
+                                 int (*process_status_query_func)(void *user_data));
 
 
 // ----------------------------------------------------------------------------------------------------------------------
@@ -382,6 +401,8 @@ MYX_PUBLIC_FUNC void myx_grt_set_input_callback(MYX_GRT *grt, void *user_data,
 
 MYX_PUBLIC_FUNC const char * myx_grt_version();
 
+MYX_PUBLIC_FUNC void myx_grt_init_threads();
+
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_get_root(MYX_GRT *grt);
 MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_set_root(MYX_GRT *grt, MYX_GRT_VALUE *new_root);
 
@@ -434,6 +455,7 @@ MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_grt_module_loader_create(MYX_GRT *gr
                   MYX_GRT_ERROR (*init_module)(MYX_GRT_MODULE_LOADER *loader, const char *file_name, MYX_GRT_MODULE **retmodule),
                   MYX_GRT_ERROR (*call_function)(MYX_GRT_FUNCTION *func, MYX_GRT_VALUE *argument, MYX_GRT_VALUE **retval),
                   void *priv);
+MYX_PUBLIC_FUNC void * myx_grt_module_get_private_data(MYX_GRT_MODULE *module);
 MYX_PUBLIC_FUNC void * myx_grt_module_loader_get_private_data(MYX_GRT_MODULE_LOADER *loader);
 MYX_PUBLIC_FUNC MYX_GRT_MODULE * myx_grt_module_create(MYX_GRT_MODULE_LOADER *loader, const char *name, 
                                                        const char *path, const char *extends, void *priv);
@@ -522,8 +544,10 @@ MYX_PUBLIC_FUNC const char * myx_grt_struct_get_desc(MYX_GRT *grt, MYX_GRT_STRUC
 MYX_PUBLIC_FUNC const char * myx_grt_struct_get_bridge_name(MYX_GRT_STRUCT *gstruct);
 MYX_PUBLIC_FUNC MYX_GRT_MODULE * myx_grt_struct_get_bridge(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct);
 
-MYX_PUBLIC_FUNC int myx_grt_struct_check_inherits_from(MYX_GRT *grt, const char *struct_name, const char *parent_name);
-MYX_PUBLIC_FUNC int myx_grt_struct_check_is_or_inherits_from(MYX_GRT *grt, const char *struct_name, const char *parent_name);
+MYX_PUBLIC_FUNC int myx_grt_struct_inherits_from(MYX_GRT *grt, const char *struct_name, const char *parent_name);
+MYX_PUBLIC_FUNC int myx_grt_struct_is_or_inherits_from(MYX_GRT *grt, const char *struct_name, const char *parent_name);
+
+MYX_PUBLIC_FUNC char * myx_grt_struct_get_icon_path(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct, MYX_ICON_TYPE icon_type);
 
 MYX_PUBLIC_FUNC const char * myx_grt_struct_get_icon(MYX_GRT *grt, const char *source_path, 
                  MYX_GRT_STRUCT *gstruct, MYX_ICON_TYPE icon_type, unsigned int *length);
@@ -557,6 +581,7 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_struct_member_get_content_type(MYX_GR
 MYX_PUBLIC_FUNC const char * myx_grt_struct_member_get_struct_name(MYX_GRT_STRUCT_MEMBER *member);
 MYX_PUBLIC_FUNC const char * myx_grt_struct_member_get_content_struct_name(MYX_GRT_STRUCT_MEMBER *member);
 MYX_PUBLIC_FUNC unsigned int myx_grt_struct_member_get_is_ref(MYX_GRT_STRUCT_MEMBER *member);
+MYX_PUBLIC_FUNC const char * myx_grt_struct_member_get_content_struct_name_overridden(MYX_GRT_STRUCT_MEMBER *member);
 
 // Internal
 MYX_PUBLIC_FUNC int myx_grt_scan_for_structs(MYX_GRT *grt, const char *directory, MYX_GRT_ERROR *error);
@@ -571,14 +596,21 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_dup(MYX_GRT_VALUE *value);
   
 MYX_PUBLIC_FUNC void myx_grt_value_listener_add(MYX_GRT *grt, MYX_GRT_VALUE *value, void *user_data, 
                                                 int (*listener_callback)(MYX_GRT *grt, MYX_GRT_VALUE *value, MYX_GRT_VALUE_CALLBACK_REASON reason, void *user_data));
-MYX_PUBLIC_FUNC void myx_grt_value_listener_remove(MYX_GRT_VALUE *value, 
+MYX_PUBLIC_FUNC void myx_grt_value_listener_remove(MYX_GRT_VALUE *value, void *user_data, 
                                                 int (*listener_callback_remove)(MYX_GRT *grt, MYX_GRT_VALUE *value, MYX_GRT_VALUE_CALLBACK_REASON reason, void *user_data));
+MYX_PUBLIC_FUNC int myx_grt_value_listener_get(MYX_GRT *grt, MYX_GRT_VALUE *value, void *user_data, 
+                                                int (*listener_callback_get)(MYX_GRT *grt, MYX_GRT_VALUE *value, MYX_GRT_VALUE_CALLBACK_REASON reason, void *user_data));
+MYX_PUBLIC_FUNC int myx_grt_value_listener_call(MYX_GRT_VALUE *value, MYX_GRT_VALUE_CALLBACK_REASON reason);
 
-MYX_PUBLIC_FUNC char * myx_grt_value_to_xml(MYX_GRT_VALUE *value);
+
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_diff_make(MYX_GRT *grt, MYX_GRT_VALUE *source, MYX_GRT_VALUE *target);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_diff_apply(MYX_GRT *grt, MYX_GRT_VALUE *value, MYX_GRT_VALUE *diff);
+
+MYX_PUBLIC_FUNC char * myx_grt_value_to_xml(MYX_GRT *grt, MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_xml(MYX_GRT *grt, const char *str, size_t size);
 
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_retrieve_from_file(MYX_GRT *grt, const char *filename);
-MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_store_to_file(MYX_GRT_VALUE *value, const char *filename);
+MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_store_to_file(MYX_GRT *grt, MYX_GRT_VALUE *value, const char *filename);
 
 MYX_PUBLIC_FUNC char * myx_grt_value_to_xml_global_object(const char *objectPath, MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_xml_global_object(MYX_GRT *grt, const char *str, size_t size);
@@ -623,6 +655,7 @@ MYX_PUBLIC_FUNC int myx_grt_list_item_add(MYX_GRT_VALUE *list, MYX_GRT_VALUE *va
 MYX_PUBLIC_FUNC int myx_grt_list_item_add_as_string(MYX_GRT_VALUE *list, const char *s);
 MYX_PUBLIC_FUNC int myx_grt_list_item_del(MYX_GRT_VALUE *list, int index);
 MYX_PUBLIC_FUNC int myx_grt_list_item_del_as_string(MYX_GRT_VALUE *list, const char *s);
+MYX_PUBLIC_FUNC int myx_grt_list_item_del_value(MYX_GRT_VALUE *list, MYX_GRT_VALUE *value);
 
 // Items, index based access
 MYX_PUBLIC_FUNC unsigned int myx_grt_list_item_count(MYX_GRT_VALUE *list);
@@ -630,6 +663,9 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_item_get(MYX_GRT_VALUE *list, unsig
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_item_get_reference_value(MYX_GRT *grt, MYX_GRT_VALUE *list, unsigned int index);
 MYX_PUBLIC_FUNC const char * myx_grt_list_item_get_as_string(MYX_GRT_VALUE *list, unsigned int index);
 MYX_PUBLIC_FUNC int myx_grt_list_item_set(MYX_GRT_VALUE *list, unsigned int index, MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_item_get_by_object_name(MYX_GRT_VALUE *list, const char *name);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_item_get_reference_value_by_object_name(MYX_GRT *grt, MYX_GRT_VALUE *list, const char *name);
+MYX_PUBLIC_FUNC int myx_grt_list_del_by_object_name(MYX_GRT_VALUE *list, const char *name);
 
 // Struct/content handling
 MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_list_content_get_type(MYX_GRT_VALUE *list);
@@ -676,7 +712,7 @@ MYX_PUBLIC_FUNC int myx_grt_dict_item_by_index(MYX_GRT_VALUE *dict, unsigned int
 
 // Items, by path
 MYX_PUBLIC_FUNC char * myx_get_parent_path(const char *path);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_item_get_by_path(MYX_GRT_VALUE *dict, const char *path);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_item_get_by_path(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *path);
 MYX_PUBLIC_FUNC int myx_grt_dict_item_set_by_path(MYX_GRT_VALUE *dict, const char *path, MYX_GRT_VALUE *new_value);
 
 
@@ -698,6 +734,7 @@ MYX_PUBLIC_FUNC int myx_grt_dict_struct_set_name_no_cache_register(MYX_GRT *grt,
 MYX_PUBLIC_FUNC int myx_grt_dict_struct_validate(MYX_GRT *grt, MYX_GRT_VALUE *value, const char *struct_name, int strict);
 
 MYX_PUBLIC_FUNC int myx_grt_dict_struct_inherits_from(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *parent_struct_name);
+MYX_PUBLIC_FUNC int myx_grt_dict_struct_is_or_inherits_from(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *parent_struct_name);
 
 MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_dict_content_get_type(MYX_GRT_VALUE *dict);
 MYX_PUBLIC_FUNC const char * myx_grt_dict_content_get_struct_name(MYX_GRT_VALUE *dict);
@@ -734,6 +771,13 @@ MYX_PUBLIC_FUNC int myx_grt_bridge_dict_item_set_value_from_string(MYX_GRT_VALUE
 MYX_PUBLIC_FUNC int myx_grt_bridge_dict_item_set_value_from_int(MYX_GRT_VALUE *dict, const char *key, int i, int do_bridge_callback);
 MYX_PUBLIC_FUNC int myx_grt_bridge_dict_item_set_value_from_real(MYX_GRT_VALUE *dict, const char *key, double d, int do_bridge_callback);
 
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_bridge_dict_item_get_value(MYX_GRT_VALUE *dict, const char *key, int do_bridge_callback);
+
+MYX_PUBLIC_FUNC unsigned int myx_grt_bridge_dict_item_count(MYX_GRT_VALUE *dict, int do_bridge_callback);
+MYX_PUBLIC_FUNC const char * myx_grt_bridge_dict_item_key_by_index(MYX_GRT_VALUE *dict, unsigned int index, int do_bridge_callback);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_bridge_dict_item_value_by_index(MYX_GRT_VALUE *dict, unsigned int index, int do_bridge_callback);
+MYX_PUBLIC_FUNC int myx_grt_bridge_dict_item_del(MYX_GRT_VALUE *dict, const char *key, int do_bridge_callback);
+
 MYX_PUBLIC_FUNC int myx_grt_bridge_value_as_int(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC double myx_grt_bridge_value_as_real(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC const char * myx_grt_bridge_value_as_string(MYX_GRT_VALUE *value);
@@ -751,7 +795,7 @@ MYX_PUBLIC_FUNC int myx_grt_bridge_dict_struct_set_name(MYX_GRT *grt, MYX_GRT_VA
 // Java module
 
 #ifdef ENABLE_JAVA_MODULES
-MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_java_init_loader(MYX_GRT *grt, const char *class_path, MYX_GRT_ERROR *error, char *jvm_library);
+MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_java_init_loader(MYX_GRT *grt, const char *class_path, MYX_GRT_ERROR *error, const char *jvm_library, const char *grt_class_path_prefix);
 #endif
 
 // ----------------------------------------------------------------------------------------------------------------------
@@ -765,7 +809,7 @@ MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_php_init_loader(MYX_GRT *grt, MYX_GR
 // Lua module
 
 #ifdef ENABLE_LUA_MODULES
-MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_lua_init_loader(MYX_GRT *grt, MYX_GRT_ERROR *error);
+MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_lua_init_loader(MYX_GRT *grt, MYX_GRT_ERROR *error, const char *lua_module_path);
 
 // Lua Shell
 MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_init_lua_shell(MYX_GRT *grt);
