@@ -31,6 +31,8 @@
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
 #include <objbase.h>
+#elif defined(__APPLE__)
+#include <CoreFoundation/CoreFoundation.h>
 #else
 // Linux specific
 #include <uuid/uuid.h>
@@ -38,7 +40,8 @@
 
 
 static MYX_GRT_VALUE *unserialize_from_xml(MYX_GRT *grt, xmlNodePtr node, GHashTable *objects_by_id);
-static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value, GHashTable *saved_ids);
+static MYX_GRT_VALUE *unserialize_from_xml_global_object(MYX_GRT *grt, xmlNodePtr node, GHashTable *objects_by_id);
+static xmlNodePtr serialize_to_xml(MYX_GRT *grt, xmlNodePtr parent, MYX_GRT_VALUE *value, GHashTable *saved_ids);
 
 
 static void default_print(const char *msg, void *data)
@@ -52,6 +55,11 @@ static void default_log(MYX_GRT *grt, int code, const char *msg, const char *det
   printf("%i: %s: %s\n", code, msg, detail?detail:"");
 }
 
+void myx_grt_init_threads()
+{
+  g_thread_init(NULL);
+}
+
 /**
  **************************************************************************** 
  * @brief Returns the version of the GRT environment
@@ -126,6 +134,31 @@ void myx_grt_finalize(MYX_GRT *grt)
 }
 
 
+void glib_message_handler(const gchar *log_domain, GLogLevelFlags log_level, 
+                          const gchar *message, gpointer user_data)
+{
+  myx_grt_messages_stack_add_message((MYX_GRT *) user_data, message, NULL, 0);
+}
+
+/**
+ ****************************************************************************
+ * @brief Activates or deactivates the catching of glib messages
+ * 
+ *  Sets the function that is used when GRT needs to print messages.
+ *
+ * @param grt      the GRT environment
+ * @param user_data a pointer to be passed to the callback function. 
+ * @param process_output_func the callback function which should take 
+ *        the text and the userdata pointer as arguments
+ ****************************************************************************/
+void myx_grt_catch_glib_messages(MYX_GRT *grt, int do_catch)
+{
+  if (do_catch)
+    g_log_set_handler(NULL, G_LOG_LEVEL_MASK, glib_message_handler, grt);
+  else
+    g_log_default_handler(NULL, G_LOG_LEVEL_MASK, "", NULL);
+}
+
 /**
  ****************************************************************************
  * @brief Sets the function to be used to print values
@@ -191,6 +224,17 @@ void myx_grt_set_input_callback(MYX_GRT *grt, void *user_data,
   GRT_LEAVE(grt);
 }
 
+void myx_grt_set_status_query_callback(MYX_GRT *grt, void *user_data,
+                                 int (*process_status_query_func)(void *user_data))
+{
+  GRT_ENTER(grt);
+
+  grt->process_status_query= process_status_query_func;
+  grt->process_status_query_data= user_data;
+  
+  GRT_LEAVE(grt);
+}
+
 
 /**
  ****************************************************************************
@@ -628,7 +672,7 @@ MYX_GRT_VALUE *myx_grt_retrieve_from_file(MYX_GRT *grt, const char *filename)
  *
  * @return  MYX_GRT_NO_ERROR if there were no errors.
  ****************************************************************************/
-MYX_GRT_ERROR myx_grt_store_to_file(MYX_GRT_VALUE *value, const char *filename)
+MYX_GRT_ERROR myx_grt_store_to_file(MYX_GRT *grt, MYX_GRT_VALUE *value, const char *filename)
 {
   xmlDocPtr doc;
   int res;
@@ -639,7 +683,7 @@ MYX_GRT_ERROR myx_grt_store_to_file(MYX_GRT_VALUE *value, const char *filename)
   
   saved_ids= g_hash_table_new(g_str_hash, g_str_equal);
   
-  serialize_to_xml(doc->children, value, saved_ids);
+  serialize_to_xml(grt, doc->children, value, saved_ids);
 
   g_hash_table_destroy(saved_ids);
   
@@ -693,9 +737,10 @@ int myx_grt_printf(MYX_GRT *grt, const char *fmt, ...)
 
 
 
-static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value,
+static xmlNodePtr serialize_to_xml(MYX_GRT *grt, xmlNodePtr parent, MYX_GRT_VALUE *value,
                                    GHashTable *saved_ids)
 {
+  MYX_GRT_STRUCT *gstruct;
   unsigned int i;
   char buffer[100];
   const char *id;
@@ -732,15 +777,19 @@ static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value,
     {
       g_return_val_if_fail((value->value.l->items[i]->type == value->value.l->content_type) ||
          (value->value.l->content_type == MYX_ANY_VALUE), NULL);
-      serialize_to_xml(node, value->value.l->items[i], saved_ids);
+      serialize_to_xml(grt, node, value->value.l->items[i], saved_ids);
     }
     break;
   case MYX_DICT_VALUE:
+    gstruct= NULL;
     node= xmlNewTextChild(parent, NULL, "value", NULL);
     xmlNewProp(node, "type", "dict");
 
     if ((value->value.d->struct_name) && (value->value.d->struct_name[0]))
+    {
       xmlNewProp(node, "struct-name", value->value.d->struct_name);
+      gstruct= myx_grt_struct_get(grt, value->value.d->struct_name);
+    }
 
     if (value->value.d->content_type != MYX_ANY_VALUE)
       xmlNewProp(node, "content-type", myx_get_value_type_as_string(value->value.d->content_type));
@@ -768,8 +817,16 @@ static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value,
       
       if (value->value.d->items[i].value)
       {
-        child= serialize_to_xml(node, value->value.d->items[i].value, saved_ids);
+        child= serialize_to_xml(grt, node, value->value.d->items[i].value, saved_ids);
         xmlNewProp(child, "key", value->value.d->items[i].key);
+        
+        if (gstruct)
+        {
+          MYX_GRT_STRUCT_MEMBER *member= myx_grt_struct_get_member_by_name(grt, gstruct, value->value.d->items[i].key, 1);
+
+          if (member && member->is_ref)
+            xmlNewProp(child, "option", "ref");
+        }
       }
     }
     break;
@@ -812,9 +869,17 @@ static MYX_GRT_VALUE *unserialize_from_xml(MYX_GRT *grt, xmlNodePtr node, GHashT
   }
   else if (strcmp(node_type, "dict")==0)
   {
+    xmlChar *global_object_path= xmlGetProp(node, "global-object-path");
     xmlChar *link_id= xmlGetProp(node, "link");
     xmlNodePtr child;
 
+    if (global_object_path)
+    {
+      xmlFree(link_id);
+      return unserialize_from_xml_global_object(grt, node, objects_by_id);
+    }
+    xmlFree(global_object_path);
+
     if (link_id)
     {
       MYX_GRT_VALUE *linked_object;
@@ -854,7 +919,12 @@ static MYX_GRT_VALUE *unserialize_from_xml(MYX_GRT *grt, xmlNodePtr node, GHashT
       {
         xmlChar *struct_name= xmlGetProp(node, "struct-name");
 
-        value= myx_grt_dict_new(grt, struct_name);
+        // this does the work of myx_grt_dict_new()
+        value= g_new0(MYX_GRT_VALUE, 1);
+        value->type= MYX_DICT_VALUE;
+        value->value.d= g_new0(MYX_GRT_DICT, 1);
+        value->refcount= 1;
+        value->value.d->struct_name= g_strdup((char*)struct_name);
 
         xmlFree(struct_name);
       }
@@ -912,9 +982,13 @@ static MYX_GRT_VALUE *unserialize_from_xml(MYX_GRT *grt, xmlNodePtr node, GHashT
 
     if (error == MYX_GRT_NO_ERROR)
     {
-      value= myx_grt_list_new(content_type, NULL);
-
-      value->value.l->content_struct_name= g_strdup(cstruct_name);
+      // this does the work of myx_grt_list_new()
+      value= g_new0(MYX_GRT_VALUE, 1);
+      value->type= MYX_LIST_VALUE;
+      value->value.l= g_new0(MYX_GRT_LIST, 1);
+      value->value.l->content_type= content_type;
+      value->refcount= 1;
+      value->value.l->content_struct_name= g_strdup((char*)cstruct_name);
 
       child= node->children;
       while (child)
@@ -1042,7 +1116,7 @@ static xmlNodePtr serialize_to_xml_global_object(const char *objectPath, xmlNode
 
 static MYX_GRT_VALUE *unserialize_from_xml_global_object(MYX_GRT *grt, xmlNodePtr node, GHashTable *objects_by_id)
 {
-  xmlChar *node_type= xmlGetProp(node, "type");
+  char *node_type= (char*)xmlGetProp(node, (xmlChar*)"type");
   MYX_GRT_VALUE *value= NULL;
 
   if (!node_type)
@@ -1060,7 +1134,7 @@ static MYX_GRT_VALUE *unserialize_from_xml_global_object(MYX_GRT *grt, xmlNodePt
 
     if (global_object_path)
     {
-      value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), global_object_path);
+      value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), global_object_path);
       xmlFree(global_object_path);
       GRT_RETURN(grt, value, MYX_GRT_VALUE*);
     }
@@ -1081,7 +1155,7 @@ static MYX_GRT_VALUE *unserialize_from_xml_global_object(MYX_GRT *grt, xmlNodePt
  * 
  * @see myx_grt_value_from_xml
  *****************************************************************************/
-char *myx_grt_value_to_xml(MYX_GRT_VALUE *value)
+char *myx_grt_value_to_xml(MYX_GRT *grt, MYX_GRT_VALUE *value)
 {
   xmlDocPtr doc;
   xmlNodePtr root;
@@ -1096,7 +1170,7 @@ char *myx_grt_value_to_xml(MYX_GRT_VALUE *value)
     
     saved_ids= g_hash_table_new(g_str_hash, g_str_equal);
     
-    serialize_to_xml(root, value, saved_ids);
+    serialize_to_xml(grt, root, value, saved_ids);
     
     g_hash_table_destroy(saved_ids);
 
@@ -1133,7 +1207,17 @@ MYX_GRT_VALUE *myx_grt_value_from_xml(MYX_GRT *grt, const char *str, size_t size
 
   if (!doc)
   {
-    g_warning("Could not parse XML data");
+    xmlErrorPtr error= xmlGetLastError();
+
+    if (error) 
+    {
+      myx_grt_messages_stack_add_error(grt, "Could not parse XML data. Line %d, %s", NULL, 0, 
+        error->line, error->message);
+    }
+    else
+    {
+      g_warning("Could not parse XML data");
+    }
     return NULL;
   }
 
@@ -1470,6 +1554,42 @@ void myx_grt_messages_stack_flush(MYX_GRT *grt, unsigned int count)
 }
 
 
+/**
+ ****************************************************************************
+ * @brief Called by the GRT when certain actions happen to a value, e.g. it is being deleted.
+ *
+ * @param grt The generic runtime that is acting.
+ * @param value The value on which the runtime is acting.
+ * @param reason The reason why this callback was called.
+ * @param data User data given when this callback was registered.
+ * @return 0
+ ****************************************************************************
+ */
+int refcache_callback(MYX_GRT* grt, MYX_GRT_VALUE* value, MYX_GRT_VALUE_CALLBACK_REASON reason, void* data)
+{
+  if (reason == MYX_GVCR_DELETE)
+  {
+    const char *_id= myx_grt_dict_item_get_as_string(value, "_id");
+    g_hash_table_remove(grt->reference_cache, _id);
+  };
+
+  return 0;
+}
+
+/**
+ ****************************************************************************
+ * @brief Called by the GRT when the cache is cleared.
+ *
+ * @param key The key of the entry that is currently removed.
+ * @param value The value part of the entry.
+ * @param data User data (NULL).
+ ****************************************************************************
+ */
+void refcache_callback2(gpointer key, gpointer value, gpointer data)
+{
+  MYX_GRT_VALUE* grtValue = value;
+  myx_grt_value_listener_remove(value, data, refcache_callback);
+}
 
 /** 
  ****************************************************************************
@@ -1481,6 +1601,9 @@ void myx_grt_reference_cache_clear(MYX_GRT *grt)
 {
   GRT_ENTER(grt);
   
+  // Remove all value listener registrations.
+  g_hash_table_foreach(grt->reference_cache, refcache_callback2, NULL); 
+
   // free current cache
   g_hash_table_destroy(grt->reference_cache);
 
@@ -1503,7 +1626,7 @@ static void grt_rescan_dict(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *objec
   
   // if this dict inherits from GrtObject, cache its _id if it not empty
   // and set the object path
-  if (struct_name && myx_grt_struct_check_inherits_from(grt, struct_name, "GrtObject"))
+  if (struct_name && myx_grt_struct_inherits_from(grt, struct_name, "GrtObject"))
   {
     const char *_id= myx_grt_dict_item_get_as_string(dict, "_id");
 
@@ -1512,6 +1635,7 @@ static void grt_rescan_dict(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *objec
       dict->value.d->grt= grt;
       dict->value.d->object_path= g_strdup(object_path);
       g_hash_table_insert(grt->reference_cache, g_strdup(_id), dict);
+      myx_grt_value_listener_add(grt, dict, NULL, refcache_callback);
     }
   }
 
@@ -1617,6 +1741,23 @@ char * myx_grt_get_guid()
     (LPSTR) guid_str, 200, NULL, NULL);
 
   return g_strdup(guid_str);
+#elif defined(__APPLE__)
+  CFUUIDRef uid;
+  CFStringRef str;
+  CFRange range;
+  char *data;
+  
+  uid= CFUUIDCreate(NULL);
+  str= CFUUIDCreateString(NULL, uid);
+  
+  data= g_malloc(sizeof(gchar)*(CFStringGetLength(str)+1));
+  
+  CFStringGetCString(str, data, CFStringGetLength(str)+1, kCFStringEncodingUTF8);
+  
+  CFRelease(uid);
+  CFRelease(str);
+  
+  return data;
 #else
   {
     uuid_t gid;
