@@ -1,18 +1,18 @@
-/* Copyright (C) 2004 MySQL AB
+/* Copyright (C) 2005 MySQL AB
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2.1 of the License, or (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
 
 
 #ifdef ENABLE_JAVA_MODULES
@@ -28,7 +28,13 @@
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
 #include <Basetsd.h>
+#define DLOAD_JVM
 #else
+
+#ifdef DLOAD_JVM
+#include <dlfcn.h>
+#endif
+
 #define LongToPtr(l) ((void*)(long)(l))
 #define PtrToLong(p) ((long)(p))
 #endif
@@ -39,34 +45,37 @@
 static MYX_GRT_ERROR java_call_function(MYX_GRT_FUNCTION *function, MYX_GRT_VALUE *value, MYX_GRT_VALUE **retval);
 static MYX_GRT_ERROR java_init_module(MYX_GRT_MODULE_LOADER *loader, const char *file, MYX_GRT_MODULE **retmodule);
 
-static jobject j_xml_object_from_grt_value(JNIEnv *env, MYX_GRT_VALUE *value);
+static jobject j_xml_object_from_grt_value(MYX_GRT *grt, JNIEnv *env, MYX_GRT_VALUE *value);
 static MYX_GRT_VALUE *j_grt_value_from_java_xml_object(MYX_GRT *grt, JNIEnv *env, jobject object);
 static jobject j_call_method(JNIEnv *env, jclass theclass, jobject object, const char *name, const char *signature, jvalue *args);
 static jobject j_call_static_method(JNIEnv *env, jclass theclass, const char *name, const char *signature, jvalue *args);
 static char *j_get_string(JNIEnv *env, jobject strobj);
-static MYX_GRT_LIST *j_get_string_list(MYX_JAVA_LOADER *jloader, jobject lobj);
+//static MYX_GRT_LIST *j_get_string_list(MYX_JAVA_LOADER *jloader, jobject lobj);
 
 static MYX_GRT_ERROR java_cache_java_objects_and_methods(MYX_GRT *grt, MYX_JAVA_LOADER *priv);
 
-#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
+#ifdef DLOAD_JVM
 typedef jint (JNICALL *CreateJavaVM_t)(JavaVM **pvm, void **env, void *args);
 
 static int get_java_runtimelib_filename(const char *version, char **runtimelib_filename);
 #endif
 
-MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path, MYX_GRT_ERROR *error, char *jvm_library)
+static JavaVM *global_jvm= NULL;
+
+MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path, MYX_GRT_ERROR *error, const char *jvm_library,
+                                            const char *grt_class_path_prefix)
 {
   MYX_GRT_MODULE_LOADER *loader= g_new0(MYX_GRT_MODULE_LOADER, 1);
   MYX_JAVA_LOADER *priv= g_new0(MYX_JAVA_LOADER, 1);
   JavaVMInitArgs vm_init_args;
-  JavaVMOption jvm_options[6];
+  JavaVMOption jvm_options[7];
   jclass grt_class;
   static char *file_extensions[]= {
     ".class"
   };
   int res;
-  const char *grt_class_path= "java";
-  const char *grt_class_path_libraries_dir= "java/lib";
+  char *grt_class_path= g_build_filename(grt_class_path_prefix, "java", NULL);
+  char *grt_class_path_libraries_dir= g_build_filename(grt_class_path_prefix, "java/lib", NULL);
   char *grt_class_path_libraries= NULL;
   GDir *dir;
   const char *entry;
@@ -76,8 +85,12 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
   jobject grt_object;
   jvalue *args= NULL;
   char *runtimelib_filename;
-#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
+#ifdef DLOAD_JVM
+# if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
   HINSTANCE handle;
+# else
+  void *handle;
+# endif
   CreateJavaVM_t create_java_vm;
 #endif
 
@@ -92,38 +105,48 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
   loader->call_function= java_call_function;
   loader->extensions_num= 1;
   loader->extensions= file_extensions;
-
+  
+  memset(&vm_init_args, 0, sizeof(vm_init_args));
   vm_init_args.version = JNI_VERSION_1_4;
+#ifndef DLOAD_JVM
+  JNI_GetDefaultJavaVMInitArgs(&vm_init_args);
+#endif
 
   if (grt->options & MYX_GRT_REMOTE_DEBUG)
-    vm_init_args.nOptions = 5;
+    vm_init_args.nOptions = 6;
   else
-    vm_init_args.nOptions = 1;
+    vm_init_args.nOptions = 2;
 
   if (jvm_library && jvm_library[0]) 
   {
     runtimelib_filename= g_strdup(jvm_library);
   }
-#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
+#ifdef DLOAD_JVM
   else
   {
     //try to locate the JVM .dll
-    *error= get_java_runtimelib_filename("1.4", &runtimelib_filename);
+    *error= get_java_runtimelib_filename("1.5", &runtimelib_filename);
     if (*error == MYX_GRT_JAVA_NOT_FOUND)
-      *error= get_java_runtimelib_filename("1.5", &runtimelib_filename);
+      *error= get_java_runtimelib_filename("1.4", &runtimelib_filename);
 
     if (*error != MYX_GRT_NO_ERROR)
     {
+#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
       myx_grt_messages_stack_add_error(grt, "The JRE 1.4 cannot be found." _br
         "Please make sure the Sun JRE 1.4 is installed correctly." _br
         "Verify the following registry key:" _br
         "HKEY_LOCAL_MACHINE\\SOFTWARE\\JavaSoft\\Java Runtime Environment\\1.4\\RuntimeLib",
                                        NULL, 0, 0);
-
+#else
+      myx_grt_messages_stack_add_error(grt, "The JRE 1.4 cannot be found." _br
+        "Please make sure the Sun JRE 1.4 is installed correctly." _br
+        "Verify the GRT_JVM_PATH is set to the directory libjvm.so is located.",
+                                       NULL, 0, 0);
+#endif
       goto errorfree;
     }
   }
-#endif
+#endif // DLOAD_JVM
 
   //scan for .jars in the lib directory
   dir= g_dir_open(grt_class_path_libraries_dir, 0, NULL);
@@ -165,7 +188,11 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
     jvm_options[3].optionString= "-Xnoagent";
     jvm_options[4].optionString= "-Djava.compiler=NONE";
     jvm_options[5].optionString= "-Xss200m";
+
+    jvm_options[6].optionString= "-Xmx256m";
   }
+  else
+    jvm_options[1].optionString= "-Xmx256m";
 
   vm_init_args.options = jvm_options;
   vm_init_args.ignoreUnrecognized = JNI_FALSE;
@@ -200,10 +227,44 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
 
   // Call JNI_CreateJavaVM to get the Java environment
   res= create_java_vm(&priv->jvm, (void **)&priv->env, &vm_init_args);
-#else
-  res= JNI_CreateJavaVM(&priv->jvm, (void **)&priv->env, &vm_init_args);
-#endif
 
+#elif defined(DLOAD_JVM)
+
+  if (!(handle= dlopen(runtimelib_filename, RTLD_LAZY)))
+  {
+    myx_grt_messages_stack_add_error(grt, "The JRE library cannot be found at %s.", NULL, 0, runtimelib_filename);
+    
+    *error= MYX_GRT_JAVA_JRE_CANNOT_BE_LOADED;
+    GRT_RETURN(grt, NULL, MYX_GRT_MODULE_LOADER*);
+  }
+
+  // Locate the JNI_CreateJavaVM function
+  create_java_vm= dlsym(handle, "JNI_CreateJavaVM");
+  if (create_java_vm == 0)
+  {
+    myx_grt_messages_stack_add_error(grt, "The JRE cannot be loaded.", NULL, 0);
+
+    *error= MYX_GRT_JAVA_JRE_CANNOT_BE_LOADED;
+    GRT_RETURN(grt, NULL, MYX_GRT_MODULE_LOADER*);
+  }    
+    
+  // Call JNI_CreateJavaVM to get the Java environment
+  res= create_java_vm(&priv->jvm, (void **)&priv->env, &vm_init_args);
+
+#else
+  if (!global_jvm)
+  {
+    res= JNI_CreateJavaVM(&priv->jvm, (void **)&priv->env, &vm_init_args);
+    global_jvm= priv->jvm;
+  }
+  else
+  {
+    vm_init_args.nOptions= 0;
+    jvm_options[0].optionString= 0;
+    
+    res= (*global_jvm)->AttachCurrentThread(global_jvm, (void **)&priv->env, &vm_init_args);
+  }
+#endif  
   g_free(jvm_options[0].optionString);
 
   if (res < 0)
@@ -280,9 +341,13 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
   args= g_new0(jvalue, 2);
   args[0].l= (*priv->env)->NewStringUTF(priv->env, "GrtCallbackNative");
   {
+#ifdef FROM_XCODE
+    args[1].l= (*priv->env)->NewStringUTF(priv->env, grt_class_path_prefix);
+#else
     char *path= str_g_append(g_get_current_dir(), "/");
     args[1].l= (*priv->env)->NewStringUTF(priv->env, path);
     g_free(path);
+#endif
   }
 
   (*priv->env)->CallVoidMethodA(priv->env, grt_object, setCallback_methodID, args);
@@ -308,6 +373,8 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
 error:
   *error= MYX_GRT_MODULE_INIT_ERROR;
 errorfree:
+  g_free(grt_class_path_libraries_dir);
+  g_free(grt_class_path);
   g_free(priv);
   g_free(loader);
   g_free(args);
@@ -722,12 +789,12 @@ static MYX_GRT_ERROR java_call_function(MYX_GRT_FUNCTION *function, MYX_GRT_VALU
   args[0].l= jmodule->class_ref;
   args[1].l= (*env)->NewStringUTF(env, function->name);
   args[2].l= (*env)->NewStringUTF(env, jfunc->java_signature);
-  args[3].l= j_xml_object_from_grt_value(env, value);
+  args[3].l= j_xml_object_from_grt_value(function->module->loader->grt, env, value);
   
   res= (*env)->CallStaticObjectMethodA(env, jloader->grt_class, jloader->grt_call_func, args);
   if (res)
   {
-    *retval= j_grt_value_from_java_xml_object(NULL, env, res);
+    *retval= j_grt_value_from_java_xml_object(function->module->loader->grt, env, res);
     (*env)->DeleteLocalRef(env, res);
   }
   else
@@ -809,12 +876,12 @@ static char *j_get_string(JNIEnv *env, jobject strobj)
 }
 
 
-static jobject j_xml_object_from_grt_value(JNIEnv *env, MYX_GRT_VALUE *value)
+static jobject j_xml_object_from_grt_value(MYX_GRT *grt, JNIEnv *env, MYX_GRT_VALUE *value)
 {
   jobject jstr= NULL;
   char *data;
 
-  data= myx_grt_value_to_xml(value);
+  data= myx_grt_value_to_xml(grt, value);
   if (data)
   {
     jstr= (*env)->NewStringUTF(env, data);
@@ -1182,9 +1249,9 @@ static MYX_GRT_VALUE *java_object_to_global_grt_value(MYX_GRT *grt, JNIEnv *env,
       }
       else if (myx_grt_struct_member_get_type(member) == MYX_LIST_VALUE)
       {
-        if (myx_grt_struct_member_get_content_struct_name(member) != NULL)
+        if (myx_grt_struct_member_get_content_struct_name_overridden(member) != NULL)
         {
-          char *class_name= str_g_replace(g_strdup(myx_grt_struct_member_get_content_struct_name(member)), ".", "/");
+          char *class_name= str_g_replace(g_strdup(myx_grt_struct_member_get_content_struct_name_overridden(member)), ".", "/");
           method_sig= str_g_append(method_sig, "L" GRTCLASSPREFIX);
           method_sig= str_g_append_and_free(method_sig, class_name);
           method_sig= str_g_append(method_sig, "List;");
@@ -1196,7 +1263,14 @@ static MYX_GRT_VALUE *java_object_to_global_grt_value(MYX_GRT *grt, JNIEnv *env,
       }
       else if (myx_grt_struct_member_get_type(member) == MYX_DICT_VALUE)
       {
-        if (myx_grt_struct_member_get_struct_name(member) != NULL)
+        if (myx_grt_struct_member_get_content_struct_name_overridden(member) != NULL)
+        {
+          char *class_name= str_g_replace(g_strdup(myx_grt_struct_member_get_content_struct_name_overridden(member)), ".", "/");
+          method_sig= str_g_append(method_sig, "L" GRTCLASSPREFIX);
+          method_sig= str_g_append_and_free(method_sig, class_name);
+          method_sig= str_g_append(method_sig, ";");
+        }
+        else if (myx_grt_struct_member_get_struct_name(member) != NULL)
         {
           char *class_name= str_g_replace(g_strdup(myx_grt_struct_member_get_struct_name(member)), ".", "/");
           method_sig= str_g_append(method_sig, "L" GRTCLASSPREFIX); 
@@ -1208,9 +1282,9 @@ static MYX_GRT_VALUE *java_object_to_global_grt_value(MYX_GRT *grt, JNIEnv *env,
           if (myx_grt_struct_member_get_content_type(member) == MYX_STRING_VALUE)
             method_sig= str_g_append(method_sig, "L" GRTCLASSPREFIX "GrtStringHashMap;");
           else if ( (myx_grt_struct_member_get_content_type(member) == MYX_DICT_VALUE) &&
-            (myx_grt_struct_member_get_content_struct_name(member) != NULL) )
+            (myx_grt_struct_member_get_content_struct_name_overridden(member) != NULL) )
           {
-            char *class_name= str_g_replace(g_strdup(myx_grt_struct_member_get_content_struct_name(member)), ".", "/");
+            char *class_name= str_g_replace(g_strdup(myx_grt_struct_member_get_content_struct_name_overridden(member)), ".", "/");
             method_sig= str_g_append(method_sig, "L" GRTCLASSPREFIX);
             method_sig= str_g_append_and_free(method_sig, class_name);
             method_sig= str_g_append(method_sig, "HashMap;");
@@ -1243,13 +1317,13 @@ static MYX_GRT_VALUE *java_object_to_global_grt_value(MYX_GRT *grt, JNIEnv *env,
           jobject j_member= (*env)->CallObjectMethodA(env, value, j_getter, NULL);
           if (j_member)
           {
-            if ( (myx_grt_struct_member_get_content_struct_name(member) != NULL)
+            if ( (myx_grt_struct_member_get_content_struct_name_overridden(member) != NULL)
                 && (myx_grt_struct_member_get_content_type(member) == MYX_STRING_VALUE) )
             {
               //deal with reference lists
               jint item_count= (*env)->CallIntMethodA(env, j_member, jloader->java_grtlist_size, NULL);
               MYX_GRT_VALUE *ref_list= myx_grt_list_new(MYX_STRING_VALUE, 
-                  myx_grt_struct_member_get_content_struct_name(member));
+                  myx_grt_struct_member_get_content_struct_name_overridden(member));
               int j;
 
               for (j= 0; j < item_count; j++)
@@ -1346,7 +1420,7 @@ JNIEXPORT jstring JNICALL Java_com_mysql_grt_GrtCallbackNative_callGrtFunction
   if (c_str_name)
     (*env)->ReleaseStringUTFChars(env, name, c_str_name);
 
-  GRT_RETURN(grt, j_xml_object_from_grt_value(env, result), jstring);
+  GRT_RETURN(grt, j_xml_object_from_grt_value(grt, env, result), jstring);
 }
 
 // ------------------------------------------------------------------------------------------------------------------
@@ -1389,7 +1463,7 @@ JNIEXPORT jobject JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalAsObj
   
   GRT_ENTER(grt);
   
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   //if the value doesn't exist yet, create it
   if (!value)
@@ -1401,7 +1475,7 @@ JNIEXPORT jobject JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalAsObj
     *ptr= 0;
 
     //fetch parent value
-    parent_value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), parent_path);
+    parent_value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), parent_path);
 
     //if the parent is a dict
     if ((parent_value) && (myx_grt_value_get_type(parent_value) == MYX_DICT_VALUE))
@@ -1447,7 +1521,7 @@ JNIEXPORT jstring JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalAsStr
   
   GRT_ENTER(grt);
   
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   if (c_str_objectPath)
     (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
@@ -1467,7 +1541,7 @@ JNIEXPORT jint JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalAsInt
   
   GRT_ENTER(grt);
   
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   if (c_str_objectPath)
     (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
@@ -1487,7 +1561,7 @@ JNIEXPORT jdouble JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalAsRea
   
   GRT_ENTER(grt);
   
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   if (c_str_objectPath)
     (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
@@ -1594,7 +1668,7 @@ JNIEXPORT jint JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalListSize
   
   GRT_ENTER(grt);
   
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   if (c_str_objectPath)
     (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
@@ -1616,12 +1690,19 @@ JNIEXPORT jobject JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalListI
   
   GRT_ENTER(grt);
   
-  list= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  list= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   if (myx_grt_value_get_type(list) == MYX_LIST_VALUE)
   {
-    MYX_GRT_VALUE *list_item= myx_grt_list_item_get(list, index);
-    char *item_path= g_strdup_printf("%s/%d", c_str_objectPath, index);
+    MYX_GRT_VALUE *list_item;
+    char *item_path= g_strdup_printf("%s/%d", c_str_objectPath, (int)index);
+
+    /*// if this is a reference list, add _id of list_item instead
+    if ((myx_grt_list_content_get_type(list) == MYX_STRING_VALUE) && 
+      (myx_grt_list_content_get_struct_name(list) != NULL))
+      list_item= myx_grt_list_item_get_reference_value(grt, list, index);
+    else*/
+    list_item= myx_grt_list_item_get(list, index);
 
     jobj= java_global_object_from_grt_value(grt, env, item_path, list_item);
     g_free(item_path);
@@ -1643,11 +1724,18 @@ JNIEXPORT void JNICALL Java_com_mysql_grt_GrtCallbackNative_addGrtGlobalListItem
 
   GRT_ENTER(grt);
   
-  list= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  list= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
   list_item= java_object_to_global_grt_value(grt, env, value);
   
   if ( (list) && (list_item) )
-    myx_grt_list_item_add(list, list_item);
+  {
+    // if this is a reference list, add _id of list_item instead
+    if ((myx_grt_list_content_get_type(list) == MYX_STRING_VALUE) && 
+      (myx_grt_list_content_get_struct_name(list) != NULL))
+      myx_grt_list_item_add(list, myx_grt_dict_item_get_value(list_item, "_id"));
+    else
+      myx_grt_list_item_add(list, list_item);
+  }
 
   if (c_str_objectPath)
     (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
@@ -1677,7 +1765,7 @@ JNIEXPORT jobject JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalDictI
   
   GRT_ENTER(grt);
   
-  dict= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  dict= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   if (myx_grt_value_get_type(dict) == MYX_DICT_VALUE)
   {
@@ -1703,11 +1791,11 @@ JNIEXPORT void JNICALL Java_com_mysql_grt_GrtCallbackNative_addGrtGlobalDictItem
   const char *c_str_objectPath= (*env)->GetStringUTFChars(env, objectPath, 0);
   const char *c_str_key= (*env)->GetStringUTFChars(env, key, 0);
   MYX_GRT_VALUE *dict;
-  MYX_GRT_VALUE *result= NULL;
+//  MYX_GRT_VALUE *result= NULL;
 
   GRT_ENTER(grt);
   
-  dict= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  dict= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
   
   if (c_str_objectPath)
     (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
@@ -1736,7 +1824,7 @@ JNIEXPORT jint JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalDictKeyC
 
   GRT_ENTER(grt);
 
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   if (c_str_objectPath)
     (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
@@ -1756,7 +1844,7 @@ JNIEXPORT jstring JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalDictK
   
   GRT_ENTER(grt);
   
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   if (c_str_objectPath)
     (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
@@ -1767,11 +1855,10 @@ JNIEXPORT jstring JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalDictK
     GRT_RETURN(grt, NULL, jstring);
 }
 
-JNIEXPORT jint JNICALL Java_com_mysql_grt_GrtCallbackNative_processMessages
+JNIEXPORT void JNICALL Java_com_mysql_grt_GrtCallbackNative_processMessages
   (JNIEnv *env, jobject obj, jlong grtMyxGrtPointer, jstring msgs)
 {
   MYX_GRT *grt= LongToPtr(grtMyxGrtPointer);
-  jint res;
   
   if (grt->process_messages)
   {
@@ -1781,12 +1868,25 @@ JNIEXPORT jint JNICALL Java_com_mysql_grt_GrtCallbackNative_processMessages
     MYX_GRT_MSGS *msgs= myx_grt_messages_convert(value);
 
     // call the callback function to process the messages
-    res= grt->process_messages(msgs, grt->process_messages_data);
+    grt->process_messages(msgs, grt->process_messages_data);
 
     myx_grt_messages_free(msgs);
 
     myx_grt_value_release(value);
   }
+}
+
+JNIEXPORT jint JNICALL Java_com_mysql_grt_GrtCallbackNative_processStatusQuery
+  (JNIEnv *env, jobject obj, jlong grtMyxGrtPointer)
+{
+  MYX_GRT *grt= LongToPtr(grtMyxGrtPointer);
+  jint res= 0;
+  
+  if (grt->process_status_query)
+  {
+    // call the callback function to query the status, if returned a int <> 0 the execution will be aborted
+    res= grt->process_status_query(grt->process_status_query_data);
+  }
 
   return res;
 }
@@ -1866,7 +1966,13 @@ static int get_java_runtimelib_filename(const char *version, char **runtimelib_f
       {
         if (subkey_length > 3)
         {
-          int better_update= atoi(subkey_name + 6);
+          char subversion[2];
+          int better_update;
+
+          subversion[0]= subkey_name[4];
+          subversion[1]= 0;
+
+          better_update= atoi(subversion) * 100 + atoi(subkey_name + 6);
 
           if (better_update > best_update)
           {
@@ -1905,6 +2011,9 @@ static int get_java_runtimelib_filename(const char *version, char **runtimelib_f
     if (!registry_get_value_as_string(java_key, "RuntimeLib", buffer, sizeof(buffer))) 
 	    return MYX_GRT_JAVA_REGISTRY_CORRUPTED;
 
+    if (!buffer || !buffer[0])
+      return MYX_GRT_JAVA_NOT_FOUND;
+
     *runtimelib_filename= g_strdup(buffer);
 
     RegCloseKey(java_key);
@@ -1917,6 +2026,24 @@ static int get_java_runtimelib_filename(const char *version, char **runtimelib_f
   }
 }
 
-#endif
+#else // !windows
 
-#endif
+#ifdef DLOAD_JVM
+static int get_java_runtimelib_filename(const char *version, char **runtimelib_filename)
+{
+  char *path= getenv("GRT_JVM_PATH");
+  if (!path)
+    return MYX_GRT_JAVA_NOT_FOUND;
+  
+  if (access(path, F_OK)!=0)
+    return MYX_GRT_JAVA_NOT_FOUND;
+  
+  *runtimelib_filename= g_strdup(path);
+
+  return MYX_GRT_NO_ERROR;
+}
+#endif // DLOAD_JVM
+
+#endif // !windows
+
+#endif // ENABLE_JAVA_MODULES
