@@ -278,8 +278,6 @@ MYX_GRT_VALUE * myx_grt_function_call(MYX_GRT *grt, MYX_GRT_FUNCTION *func,
   }
 
   *error= (*func->module->loader->call_function)(func, argument, &retval);
-  if (*error != MYX_GRT_NO_ERROR)
-    GRT_RETURN(grt, NULL, MYX_GRT_VALUE*);
   
   if (retval && func->return_struct_name)
   {
@@ -520,7 +518,7 @@ int myx_grt_modules_get_that_extend(MYX_GRT *grt, const char *module, MYX_GRT_MO
 
   for (i= 0; i < grt->modules_num; i++)
   {
-    if (!module || !*module || strcmp(grt->modules[i]->extends, module)==0)
+    if (!module || !*module || strcmp2(grt->modules[i]->extends, module)==0)
     {
       count++;
       (*retmodules)[count-1]= grt->modules[i];
@@ -679,6 +677,12 @@ void * myx_grt_module_loader_get_private_data(MYX_GRT_MODULE_LOADER *loader)
   return loader->priv;
 }
 
+void * myx_grt_module_get_private_data(MYX_GRT_MODULE *module)
+{
+  return module->priv;
+}
+
+
 MYX_GRT_MODULE * myx_grt_module_create(MYX_GRT_MODULE_LOADER *loader, const char *name, 
                                                        const char *path, const char *extends, void *priv)
 {
