@@ -1,19 +1,18 @@
-/* Copyright (C) 2004 MySQL AB
+/* Copyright (C) 2005 MySQL AB
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2.1 of the License, or (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
 
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
 
 #ifdef ENABLE_PHP_MODULES
 
@@ -489,12 +488,12 @@ PHP_FUNCTION(grtCallGrtFunction)
       // return an error
       php_error_docref(NULL TSRMLS_CC, E_WARNING, "the function returned and error %d", error);
     }
-  }
 
-  // return result as XML string
-	Z_STRVAL_P(return_value) = myx_grt_value_to_xml(result);
-	Z_STRLEN_P(return_value) = (int) strlen(Z_STRVAL_P(return_value));
-	Z_TYPE_P(return_value) = IS_STRING;
+    // return result as XML string
+	  Z_STRVAL_P(return_value) = myx_grt_value_to_xml(grt, result);
+	  Z_STRLEN_P(return_value) = (int) strlen(Z_STRVAL_P(return_value));
+	  Z_TYPE_P(return_value) = IS_STRING;
+  }
 }
 
 /**
@@ -563,7 +562,7 @@ PHP_FUNCTION(getGrtGlobalAsObject)
   if (Z_TYPE_PP(objectPath) == IS_STRING)
     c_str_object_path= Z_STRVAL_PP(objectPath);
 
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_object_path);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_object_path);
 
   //if the value doesn't exist yet, create it
   if (!value)
@@ -575,7 +574,7 @@ PHP_FUNCTION(getGrtGlobalAsObject)
     *ptr= 0;
 
     //fetch parent value
-    parent_value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), parent_path);
+    parent_value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), parent_path);
 
     //if the parent is a dict
     if ((parent_value) && (myx_grt_value_get_type(parent_value) == MYX_DICT_VALUE))
@@ -657,7 +656,7 @@ PHP_FUNCTION(getGrtGlobalListSize)
   if (Z_TYPE_PP(objectPath) == IS_STRING)
     c_str_object_path= Z_STRVAL_PP(objectPath);
 
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_object_path);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_object_path);
 
   if (myx_grt_value_get_type(value) == MYX_LIST_VALUE)
     list_size= myx_grt_list_item_count(value);
@@ -694,7 +693,7 @@ PHP_FUNCTION(getGrtGlobalListItem)
   if (Z_TYPE_PP(index) == IS_LONG)
     c_index= Z_LVAL_PP(index);
 
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_object_path);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_object_path);
 
   if (myx_grt_value_get_type(value) == MYX_LIST_VALUE)
   {
@@ -729,7 +728,7 @@ PHP_FUNCTION(addGrtGlobalListItem)
   if (Z_TYPE_PP(objectPath) == IS_STRING)
     c_str_object_path= Z_STRVAL_PP(objectPath);
 
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_object_path);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_object_path);
   list_item= php_object_to_global_grt_value(grt, "com_mysql_grt_global_object", 0);
 
   if ( (value) && (list_item) )
@@ -772,7 +771,7 @@ PHP_FUNCTION(getGrtGlobalDictItem)
   if (Z_TYPE_PP(key) == IS_STRING)
     c_str_key= Z_STRVAL_PP(key);
 
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_object_path);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_object_path);
 
   if (myx_grt_value_get_type(value) == MYX_DICT_VALUE)
   {
@@ -811,7 +810,7 @@ PHP_FUNCTION(addGrtGlobalDictItem)
   if (Z_TYPE_PP(key) == IS_STRING)
     c_str_key= Z_STRVAL_PP(key);
 
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_object_path);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_object_path);
   dict_item= php_object_to_global_grt_value(grt, "com_mysql_grt_global_object", 0);
 
   if (value)
@@ -853,16 +852,40 @@ PHP_FUNCTION(processMessages)
     MYX_GRT_MSGS *msgs= myx_grt_messages_convert(value);
 
     // call the callback function to process the messages
-    res= grt->process_messages(msgs, grt->process_messages_data);
+    grt->process_messages(msgs, grt->process_messages_data);
 
     myx_grt_messages_free(msgs);
 
     myx_grt_value_release(value);
   }
 
-  RETURN_LONG (res);
+  //RETURN_LONG (res);
 }
 
+PHP_FUNCTION(processStatusQuery)
+{
+  zval **myx_grt_pointer;
+  MYX_GRT *grt;
+  int res= 0;
+
+  // fetch parameters
+  if (ZEND_NUM_ARGS() != 1 || 
+    zend_get_parameters_ex(1, &myx_grt_pointer) == FAILURE) {
+		WRONG_PARAM_COUNT;
+	}
+
+  // get pointer to grt
+  convert_to_long_ex(myx_grt_pointer);
+  grt= LongToPtr(Z_LVAL_PP(myx_grt_pointer));
+
+  if (grt->process_status_query)
+  {
+    // call the callback function to process the messages
+    res= grt->process_status_query(grt->process_status_query_data);
+  }
+
+  RETURN_LONG (res);
+}
 
 function_entry php_grt_functions[] = {
   PHP_FE(grtCallGrtFunction, NULL)
@@ -1309,11 +1332,11 @@ static MYX_GRT_ERROR php_call_function(MYX_GRT_FUNCTION *function, MYX_GRT_VALUE
   MYX_PHP_MODULE *pmodule= function->module->priv;
   MYX_PHP_FUNCTION *pfunc= function->priv;
   char *xml;
-  char *value_as_xml= myx_grt_value_to_xml(value);
+  char *value_as_xml= myx_grt_value_to_xml(grt, value);
 
   //escape "
   if (value_as_xml)
-    value_as_xml= str_g_replace(myx_grt_value_to_xml(value), "\"", "\\\"");
+    value_as_xml= str_g_replace(value_as_xml, "\"", "\\\"");
 
   xml= php_eval_string("Grt::callModuleFunction(\"%s\", \"%s\", \"%s\", \"%s\");", pmodule->classname, function->name,
     pfunc->php_signature, value_as_xml);
