@@ -16,6 +16,10 @@
 
 
 #include "myx_grt_private.h"
+#ifdef ENABLE_LUA_MODULES
+  #include "myx_grt_lua.h"
+  #include "myx_grt_lua.h"
+#endif
 
 //----------------------------------------------------------------------------------------------------------------------
 
@@ -51,6 +55,11 @@ MYX_GRT_ERROR myx_grt_add_module(MYX_GRT *grt, MYX_GRT_MODULE *module)
   grt->modules= g_realloc(grt->modules, sizeof(MYX_GRT_MODULE*)*(grt->modules_num+1));
   grt->modules[grt->modules_num++]= module;
 
+#ifdef ENABLE_LUA_MODULES
+  if (grt->lshell)
+    myx_lua_refresh(grt, grt->lshell->lua);
+#endif
+
   return MYX_GRT_NO_ERROR;
 }
 
