@@ -17,8 +17,8 @@
 
 #ifdef ENABLE_PYTHON_MODULES
 
-#include "myx_grt_python.h"
 #include <Python.h>
+#include "myx_grt_python.h"
 
 
 typedef struct MYX_GRT_MODULE_LOADER_PRIVATE
@@ -216,6 +216,8 @@ PyObject *py_value_to_object(MYX_GRT_VALUE *value)
       }
     }
     break;
+   case MYX_ANY_VALUE:
+    g_assert(0);
   }
 
   return obj;
@@ -286,7 +288,7 @@ MYX_GRT_VALUE *py_object_to_value(PyObject *object)
       MYX_GRT_VALUE *gitem;
 
       gitem= py_object_to_value(item);
-      myx_grt_list_add_item(value, gitem);
+      myx_grt_list_item_add(value, gitem);
       myx_grt_value_release(gitem);
     }
     return value;
@@ -298,7 +300,7 @@ MYX_GRT_VALUE *py_object_to_value(PyObject *object)
     PyObject *pykey, *pyvalue;
 
     i= 0;
-    value= myx_grt_dict_new(NULL);
+    value= myx_grt_dict_new(NULL, NULL);
     while (PyDict_Next(object, &i, &pykey, &pyvalue))
     {
       MYX_GRT_VALUE *tmp;
