@@ -21,9 +21,10 @@
 extern "C" {
 #endif
 
-MYX_GRT_VALUE *test_connection(MYX_GRT_VALUE *param, void *data);
+MYX_GRT_VALUE *get_version(MYX_GRT_VALUE *param, void *data);
 MYX_GRT_VALUE *get_schemata(MYX_GRT_VALUE *param, void *data);
 MYX_GRT_VALUE *reverse_engineer(MYX_GRT_VALUE *param, void *data);
+MYX_GRT_VALUE *get_character_sets(MYX_GRT_VALUE *param, void *data);
 
 #ifdef __cplusplus
 }
