@@ -22,25 +22,14 @@ extern "C" {
 #endif
 
 MYX_GRT_VALUE * generate_sql_create_statements(MYX_GRT_VALUE *param, void *data);
-static void generate_sql_create_statements_assets(MYX_GRT_VALUE *schema, const char *assets_name, void *data);
+MYX_GRT_VALUE * execute_sql_statements(MYX_GRT_VALUE *param, void *data);
 
 MYX_GRT_VALUE *get_sql_create_script(MYX_GRT_VALUE *param, void *data);
-static char *get_sql_create_script_assets(MYX_GRT_VALUE *schema, const char *assets_name);
 
 MYX_GRT_VALUE *get_sql_create(MYX_GRT_VALUE *param, void *data);
 MYX_GRT_VALUE *get_sql_drop(MYX_GRT_VALUE *param, void *data);
 MYX_GRT_VALUE *get_sql_merge(MYX_GRT_VALUE *param, void *data);
 
-static char * get_sql_create_schema(MYX_GRT *grt, MYX_GRT_VALUE *schema);
-static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table);
-static char * get_sql_create_view(MYX_GRT *grt, MYX_GRT_VALUE *view);
-static char * get_sql_create_procedure(MYX_GRT *grt, MYX_GRT_VALUE *procedure);
-
-static char * get_sql_drop_schema(MYX_GRT *grt, MYX_GRT_VALUE *schema);
-static char * get_sql_drop_table(MYX_GRT *grt, MYX_GRT_VALUE *table);
-static char * get_sql_drop_view(MYX_GRT *grt, MYX_GRT_VALUE *view);
-static char * get_sql_drop_procedure(MYX_GRT *grt, MYX_GRT_VALUE *procedure);
-
 #ifdef __cplusplus
 }
 #endif
