@@ -19,9 +19,10 @@
 
 
 static MYX_GRT_BUILTIN_FUNCTION functions_reverse_engineer[]= {
-  {"testConnection::", test_connection },
+  {"getVersion::", get_version },
   {"getSchemata::", get_schemata },
   {"reverseEngineer::", reverse_engineer },
+  {"getCharacterSets::", get_character_sets },
 };
 
 static MYX_GRT_BUILTIN_MODULE grt_module_reverse_engineer_mysql= {
@@ -129,22 +130,26 @@ MYX_GRT_VALUE *make_return_value_mysql_error_and_close(MYSQL *mysql, const char
  *****************************************************************************/
 void get_connection_info(MYX_GRT_VALUE *value, MYX_USER_CONNECTION *conn)
 {
-  MYX_GRT_VALUE *port= myx_grt_dict_item_get_value(value, "port");
-  const char *param= myx_grt_dict_item_get_as_string(value, "username");
+  MYX_GRT_VALUE *port;
+  const char *param;
 
+  // if this is a "db.mgmt.Connection" struct, the parameters are in the parameterValues item
+  if (strcmp2(myx_grt_dict_struct_get_name(value), "db.mgmt.Connection") == 0)
+    value= myx_grt_dict_item_get_value(value, "parameterValues");
+
+  param= myx_grt_dict_item_get_as_string(value, "username");
   if (param)
     conn->username= g_strdup(param);
   else
     conn->username= g_strdup("root");
 
   param= myx_grt_dict_item_get_as_string(value, "password");
-
   if (param)
     conn->password= g_strdup(param);
   else
     conn->password= NULL;
 
-  param= myx_grt_dict_item_get_as_string(value, "hostname");
+  param= myx_grt_dict_item_get_as_string(value, "host");
 
   if (param)
     conn->hostname= g_strdup(param);
@@ -152,6 +157,7 @@ void get_connection_info(MYX_GRT_VALUE *value, MYX_USER_CONNECTION *conn)
     conn->hostname= g_strdup("localhost");
 
 
+  port= myx_grt_dict_item_get_value(value, "port");
   if (port)
   {
     if (myx_grt_value_get_type(port) == MYX_INT_VALUE)
@@ -161,6 +167,8 @@ void get_connection_info(MYX_GRT_VALUE *value, MYX_USER_CONNECTION *conn)
     else
       conn->port= 3306;
   }
+  else
+    conn->port= 3306;
 }
 
 
