@@ -31,28 +31,36 @@ static char *get_sql_create_script_assets(MYX_GRT_VALUE *schema, const char *ass
 static char * get_sql_create_schema(MYX_GRT *grt, MYX_GRT_VALUE *schema);
 static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table);
 static char * get_sql_create_view(MYX_GRT *grt, MYX_GRT_VALUE *view);
-static char * get_sql_create_procedure(MYX_GRT *grt, MYX_GRT_VALUE *procedure);
+static char * get_sql_create_routine(MYX_GRT *grt, MYX_GRT_VALUE *routine);
 
 static char * get_sql_drop_schema(MYX_GRT *grt, MYX_GRT_VALUE *schema);
 static char * get_sql_drop_table(MYX_GRT *grt, MYX_GRT_VALUE *table);
 static char * get_sql_drop_view(MYX_GRT *grt, MYX_GRT_VALUE *view);
-static char * get_sql_drop_procedure(MYX_GRT *grt, MYX_GRT_VALUE *procedure);
+static char * get_sql_drop_routine(MYX_GRT *grt, MYX_GRT_VALUE *routine);
+
+static char * comment_out(char *sql);
 
 
 
 MYX_GRT_VALUE *generate_sql_create_statements(MYX_GRT_VALUE *param, void *data)
 {
 //  MYX_GRT *grt= (MYX_GRT *) data;
+  MYX_GRT_VALUE *catalog= NULL;
 
   if (!param)
     return myx_grt_function_create_error_result("You have to submit one object as parameter.", NULL);
 
-  if ( (myx_grt_value_get_type(param) != MYX_DICT_VALUE) || 
-    (strcmp2(myx_grt_dict_struct_get_name(param), "db.mysql.Catalog") != 0) )
+  if ((myx_grt_value_get_type(param) == MYX_LIST_VALUE) && (myx_grt_list_item_count(param) == 1))
+    catalog= myx_grt_list_item_get(param, 0);
+  else if ( (myx_grt_value_get_type(param) == MYX_DICT_VALUE) &&
+    (strcmp2(myx_grt_dict_struct_get_name(param), "db.mysql.Catalog") == 0) )
+    catalog= param;
+
+  if (!catalog)
     return myx_grt_function_create_error_result("The submitted parameter has to be a dict value of the struct db.mysql.catalog.", NULL);
   else
   {
-    MYX_GRT_VALUE *schemata= myx_grt_dict_item_get_value(param, "schemata");
+    MYX_GRT_VALUE *schemata= myx_grt_dict_item_get_value(catalog, "schemata");
     unsigned int i;
 
     for (i= 0; i < myx_grt_list_item_count(schemata); i++)
@@ -67,7 +75,7 @@ MYX_GRT_VALUE *generate_sql_create_statements(MYX_GRT_VALUE *param, void *data)
 
       generate_sql_create_statements_assets(schema, "tables", data);
       generate_sql_create_statements_assets(schema, "views", data);
-      generate_sql_create_statements_assets(schema, "procedures", data);
+      generate_sql_create_statements_assets(schema, "routines", data);
     }
   }
 
@@ -104,17 +112,23 @@ MYX_GRT_VALUE *get_sql_create_script(MYX_GRT_VALUE *param, void *data)
 {
 //  MYX_GRT *grt= (MYX_GRT *) data;
   MYX_GRT_VALUE *result= NULL;
+  MYX_GRT_VALUE *catalog= NULL;
   char *sql;
 
   if (!param)
     return myx_grt_function_create_error_result("You have to submit one object as parameter.", NULL);
 
-  if ( (myx_grt_value_get_type(param) != MYX_DICT_VALUE) || 
-    (strcmp2(myx_grt_dict_struct_get_name(param), "db.mysql.Catalog") != 0) )
+  if ((myx_grt_value_get_type(param) == MYX_LIST_VALUE) && (myx_grt_list_item_count(param) == 1))
+    catalog= myx_grt_list_item_get(param, 0);
+  else if ( (myx_grt_value_get_type(param) == MYX_DICT_VALUE) &&
+    (strcmp2(myx_grt_dict_struct_get_name(param), "db.mysql.Catalog") == 0) )
+    catalog= param;
+
+  if (!catalog)
     return myx_grt_function_create_error_result("The submitted parameter has to be a dict value of the struct db.mysql.catalog.", NULL);
   else
   {
-    MYX_GRT_VALUE *schemata= myx_grt_dict_item_get_value(param, "schemata");
+    MYX_GRT_VALUE *schemata= myx_grt_dict_item_get_value(catalog, "schemata");
     unsigned int i;
 
     sql= g_strdup(
@@ -126,6 +140,7 @@ MYX_GRT_VALUE *get_sql_create_script(MYX_GRT_VALUE *param, void *data)
 
     for (i= 0; i < myx_grt_list_item_count(schemata); i++)
     {
+      char *sql_part;
       MYX_GRT_VALUE *schema= myx_grt_list_item_get(schemata, i);
       MYX_GRT_VALUE *schema_drop_string= get_sql_drop(schema, data);
 
@@ -138,17 +153,45 @@ MYX_GRT_VALUE *get_sql_create_script(MYX_GRT_VALUE *param, void *data)
       sql= str_g_append(sql, myx_grt_dict_item_get_as_string(schema, "sql"));
       sql= str_g_append(sql, _br _br);
 
-      sql= str_g_append_and_free(sql,
-          get_sql_create_script_assets(schema, "tables"));
-      sql= str_g_append(sql, _br _br);
 
-      sql= str_g_append_and_free(sql,
-          get_sql_create_script_assets(schema, "views"));
-      sql= str_g_append(sql, _br _br);
+      // tables
+      sql_part= get_sql_create_script_assets(schema, "tables");
+      if (sql_part && sql_part[0])
+      {
+        sql= str_g_append(sql, "-- -------------------------------------" _br 
+          "-- Tables" _br _br);
 
-      sql= str_g_append_and_free(sql,
-          get_sql_create_script_assets(schema, "procedures"));
-      sql= str_g_append(sql, _br _br);
+        sql= str_g_append_and_free(sql, sql_part);
+        sql= str_g_append(sql, _br _br);
+      }
+      else
+        g_free(sql_part);
+
+      // views
+      sql_part= get_sql_create_script_assets(schema, "views");
+      if (sql_part && sql_part[0])
+      {
+        sql= str_g_append(sql, "-- -------------------------------------" _br 
+          "-- Views" _br _br);
+
+        sql= str_g_append_and_free(sql, sql_part);
+        sql= str_g_append(sql, _br _br);
+      }
+      else
+        g_free(sql_part);
+
+
+      sql_part= get_sql_create_script_assets(schema, "routines");
+      if (sql_part && sql_part[0])
+      {
+        sql= str_g_append(sql, "-- -------------------------------------" _br 
+          "-- Routines" _br _br);
+
+        sql= str_g_append_and_free(sql, sql_part);
+        sql= str_g_append(sql, _br _br);
+      }
+      else
+        g_free(sql_part);
     }
 
     sql= str_g_append(sql, "SET FOREIGN_KEY_CHECKS = 1;" _br _br);
@@ -198,8 +241,8 @@ MYX_GRT_VALUE *get_sql_create(MYX_GRT_VALUE *param, void *data)
     sql= get_sql_create_table(grt, param);
   else if (strcmp2(struct_name, "db.mysql.View") == 0)
     sql= get_sql_create_view(grt, param);
-  else if (strcmp2(struct_name, "db.mysql.Procedure") == 0)
-    sql= get_sql_create_procedure(grt, param);
+  else if (strcmp2(struct_name, "db.mysql.Routine") == 0)
+    sql= get_sql_create_routine(grt, param);
   else
     return myx_grt_function_create_error_result("The struct of the submitted dict is of a supported type.", NULL);
 
@@ -230,8 +273,8 @@ MYX_GRT_VALUE *get_sql_drop(MYX_GRT_VALUE *param, void *data)
     sql= get_sql_drop_table(grt, param);
   else if (strcmp2(struct_name, "db.mysql.View") == 0)
     sql= get_sql_drop_view(grt, param);
-  else if (strcmp2(struct_name, "db.mysql.Procedure") == 0)
-    sql= get_sql_drop_procedure(grt, param);
+  else if (strcmp2(struct_name, "db.mysql.Routine") == 0)
+    sql= get_sql_drop_routine(grt, param);
   else
     return myx_grt_function_create_error_result("The struct of the submitted dict is of a supported type.", NULL);
 
@@ -293,12 +336,29 @@ static char * get_sql_create_schema(MYX_GRT *grt, MYX_GRT_VALUE *schema)
 
   if ((charset) && (charset[0]))
   {
-    sql = str_g_append_and_free(sql, g_strdup_printf("  CHARACTER SET %s", charset));
+    int skip_collation= 0;
 
-    if ((collation) && (collation[0]))
-      sql = str_g_append_and_free(sql, g_strdup_printf(" COLLATE %s" _br, collation));
-    else
-      sql = str_g_append(sql, _br);
+    MYX_GRT_VALUE *catalog= myx_grt_dict_item_get_reference_value(grt, schema, "owner");
+    if (catalog)
+    {
+      MYX_GRT_VALUE *version= myx_grt_dict_item_get_value(catalog, "version");
+      if (version)
+      {
+        if ((myx_grt_dict_item_get_as_int(version, "major") == 4) &&
+          (myx_grt_dict_item_get_as_int(version, "minor") == 0))
+          skip_collation= 1;
+      }
+    }
+
+    if (!skip_collation)
+    {
+      sql = str_g_append_and_free(sql, g_strdup_printf("  CHARACTER SET %s", charset));
+
+      if ((collation) && (collation[0]))
+        sql = str_g_append_and_free(sql, g_strdup_printf(" COLLATE %s" _br, collation));
+      else
+        sql = str_g_append(sql, _br);
+    }
   }
 
   // remove last \n
@@ -311,6 +371,9 @@ static char * get_sql_create_schema(MYX_GRT *grt, MYX_GRT_VALUE *schema)
 
   g_free(quote_char);
 
+  if (myx_grt_dict_item_get_as_int(schema, "commentedOut") == 1)
+    sql= comment_out(sql);
+
   return sql;
 }
 
@@ -376,15 +439,19 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
   for (i= 0; i < myx_grt_list_item_count(columns); i++)
   {
     MYX_GRT_VALUE *column= myx_grt_list_item_get(columns, i);
+    MYX_GRT_VALUE *column_flags= myx_grt_dict_item_get_value(column, "flags");
     const char *is_nullable= (myx_grt_dict_item_get_as_int(column, "isNullable") == 1) ? "NULL" : "NOT NULL";
     const char *datatype= myx_grt_dict_item_get_as_string(column, "datatypeName");
     const char *datatype_explicit_params= myx_grt_dict_item_get_as_string(column, "datatypeExplicitParams");
-    const char *datatype_unsigned= (myx_grt_dict_item_get_as_int(column, "unsigned") == 1) ? " UNSIGNED" : "";
+    const char *column_charset= myx_grt_dict_item_get_as_string(column, "characterSetName");
+    const char *column_collation= myx_grt_dict_item_get_as_string(column, "collationName");
     char *datatype_definition;
+    const char *default_value;
     char *column_comment;
     char *column_length= myx_grt_dict_item_get_formated_as_string(column, "length");
     char *column_precision= myx_grt_dict_item_get_formated_as_string(column, "precision");
     char *column_scale= myx_grt_dict_item_get_formated_as_string(column, "scale");
+    unsigned int j;
 
     // if there is an explicit parameter definition for the datatype, use it
     if (datatype_explicit_params && datatype_explicit_params[0])
@@ -429,14 +496,46 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
         datatype_definition= g_strdup(datatype);
     }
 
-    sql_cols= str_g_append_and_free(sql_cols, g_strdup_printf("  %s%s%s %s%s %s", 
+    // add Character Set and Collation for CHAR | VARCHAR | TEXT columns
+    if (((strcmp2(datatype, "CHAR") == 0) ||
+        (strcmp2(datatype, "VARCHAR") == 0) ||
+        (strcmp2(datatype, "TEXT") == 0)) && 
+        (column_charset))
+    {
+      datatype_definition= str_g_append_and_free(datatype_definition, 
+         g_strdup_printf(" CHARACTER SET %s", column_charset));
+
+      // also add collation if it is defined
+      if (column_collation)
+      {
+        datatype_definition= str_g_append_and_free(datatype_definition, 
+          g_strdup_printf(" COLLATE %s", column_collation));
+      }
+    }
+
+    // add column datatype flags
+    for (j= 0; j < myx_grt_list_item_count(column_flags); j++)
+    {
+      MYX_GRT_VALUE *flag= myx_grt_list_item_get(column_flags, j);
+
+      datatype_definition= str_g_append_and_free(datatype_definition, 
+         g_strdup_printf(" %s", myx_grt_value_as_string(flag)));
+    }
+
+    sql_cols= str_g_append_and_free(sql_cols, g_strdup_printf("  %s%s%s %s %s", 
       quote_char,
       myx_grt_dict_item_get_as_string(column, "name"), 
       quote_char,
       datatype_definition,
-      datatype_unsigned,
       is_nullable));
 
+    // add default value
+    default_value= myx_grt_dict_item_get_as_string(column, "defaultValue");
+    if (default_value && default_value[0])
+    {
+      sql_cols= str_g_append_and_free(sql_cols, g_strdup_printf(" DEFAULT %s", default_value));
+    }
+
     // add column auto increment
     if (myx_grt_dict_item_get_as_int(column, "autoIncrement") == 1)
       sql_cols= str_g_append(sql_cols, " AUTO_INCREMENT");
@@ -467,7 +566,7 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
 
     for (i= 0; i < myx_grt_list_item_count(pk_columns); i++)
     {
-      MYX_GRT_VALUE *column= myx_grt_list_item_get(columns, i);
+      MYX_GRT_VALUE *column= myx_grt_list_item_get_reference_value(grt, pk_columns, i);
 
       if (sql_pk_cols)
         sql_pk_cols= str_g_append(sql_pk_cols, ", ");
@@ -505,6 +604,7 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
         int column_length= myx_grt_dict_item_get_as_int(index_col, "columnLength");
         MYX_GRT_VALUE *column_ref= myx_grt_dict_item_get_reference_value(grt, index_col, "referedColumn");
         const char *column_name= myx_grt_dict_item_get_as_string(column_ref, "name");
+        //MYX_GRT_VALUE *simple_datatype= myx_grt_dict_item_get_reference_value(grt, column_ref, "simpleType");
 
         if (sql_index_cols)
           sql_index_cols= str_g_append(sql_index_cols, ", ");
@@ -512,9 +612,33 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
         sql_index_cols= str_g_append_and_free(sql_index_cols, g_strdup_printf("%s%s%s", 
             quote_char, column_name, quote_char));
 
+        /*if (simple_datatype)
+        {
+          MYX_GRT_VALUE *group= myx_grt_dict_item_get_reference_value(grt, simple_datatype, "group");
+          if (group)
+          {
+            const char *group_name= myx_grt_dict_name_item_as_string(group);
+
+            // if the group_name is not string, text or blob, there must be no column_length
+            if ((strcmp2(group_name, "string") != 0) && (strcmp2(group_name, "text") != 0) && 
+              (strcmp2(group_name, "blob") != 0) && (column_length > 0))
+              column_length= 0;
+
+            // if the group_name is not text or blob, there has to be a column_length
+            if (((strcmp2(group_name, "text") == 0) || 
+              (strcmp2(group_name, "blob") == 0)) && (column_length == 0))
+              column_length= 10;
+          }
+        }*/
+
         if (column_length > 0)
+        {
+          /*if (column_length > 1024)
+            column_length= 1024;*/
+
           sql_index_cols= str_g_append_and_free(sql_index_cols, g_strdup_printf("(%d)", 
             column_length));
+        }
       }
 
       sql_indices= str_g_append_and_free(sql_indices, 
@@ -535,11 +659,15 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
     {
       MYX_GRT_VALUE *fk= myx_grt_list_item_get(fks, i);
       const char *fk_name= myx_grt_dict_item_get_as_string(fk, "name");
+      MYX_GRT_VALUE *fk_columns= myx_grt_dict_item_get_value(fk, "columns");
+
+      MYX_GRT_VALUE *fk_ref_table= myx_grt_dict_item_get_reference_value(grt, fk, "referedTable");
       const char *fk_ref_schema_name= myx_grt_dict_item_get_as_string(fk, "referedTableSchemaName");
       const char *fk_ref_table_name= myx_grt_dict_item_get_as_string(fk, "referedTableName");
 
-      MYX_GRT_VALUE *fk_columns= myx_grt_dict_item_get_value(fk, "columns");
-      MYX_GRT_VALUE *fk_ref_columns= myx_grt_dict_item_get_value(fk, "referedColumnNames");
+      MYX_GRT_VALUE *fk_ref_columns= myx_grt_dict_item_get_value(fk, "referedColumns");
+      MYX_GRT_VALUE *fk_ref_column_names= myx_grt_dict_item_get_value(fk, "referedColumnNames");
+
       const char *delete_rule= myx_grt_dict_item_get_as_string(fk, "deleteRule");
       const char *update_rule= myx_grt_dict_item_get_as_string(fk, "updateRule");
       char *sql_fk_cols= NULL;
@@ -560,31 +688,68 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
             quote_char));
       }
 
-      for (j= 0; j < myx_grt_list_item_count(fk_ref_columns); j++)
+      // if possible, use real reference to table. If the table is in a different schema, use text names
+      if (fk_ref_table)
       {
-        if (sql_fk_ref_cols)
-          sql_fk_ref_cols= str_g_append(sql_fk_ref_cols, ", ");
+        MYX_GRT_VALUE *fk_ref_schema= myx_grt_dict_item_get_reference_value(grt, fk_ref_table, "owner");
 
-        sql_fk_ref_cols= str_g_append_and_free(sql_fk_ref_cols, g_strdup_printf("%s%s%s", 
-            quote_char,
-            myx_grt_list_item_get_as_string(fk_ref_columns, j),
-            quote_char));
+        fk_ref_schema_name= myx_grt_dict_item_get_as_string(fk_ref_schema, "name");
+        fk_ref_table_name= myx_grt_dict_item_get_as_string(fk_ref_table, "name");
+      }
+      else
+      {
+        fk_ref_schema_name= myx_grt_dict_item_get_as_string(fk, "referedTableSchemaName");
+        fk_ref_table_name= myx_grt_dict_item_get_as_string(fk, "referedTableName");
       }
 
-      sql_fk= g_strdup_printf("  CONSTRAINT %s%s%s FOREIGN KEY %s%s%s (%s)" _br
-        "    REFERENCES %s%s%s.%s%s%s (%s)" _br, 
-        quote_char, fk_name, quote_char,
-        quote_char, fk_name, quote_char,
-        sql_fk_cols,
-        quote_char, fk_ref_schema_name, quote_char,
-        quote_char, fk_ref_table_name, quote_char,
-        sql_fk_ref_cols);
+      // if possible, use real reference to columns. If not, use the columns names stored as text
+      if (fk_ref_columns && (myx_grt_list_item_count(fk_ref_columns) > 0))
+      {
+        for (j= 0; j < myx_grt_list_item_count(fk_ref_columns); j++)
+        {
+          MYX_GRT_VALUE *fk_ref_col= myx_grt_list_item_get_reference_value(grt, fk_ref_columns, j);
+
+          if (fk_ref_col)
+          {
+            if (sql_fk_ref_cols)
+              sql_fk_ref_cols= str_g_append(sql_fk_ref_cols, ", ");
+
+            sql_fk_ref_cols= str_g_append_and_free(sql_fk_ref_cols, g_strdup_printf("%s%s%s", 
+                quote_char,
+                myx_grt_dict_item_get_as_string(fk_ref_col, "name"),
+                quote_char));
+          }
+        }
+      }
+      else
+      {
+        for (j= 0; j < myx_grt_list_item_count(fk_ref_column_names); j++)
+        {
+          if (sql_fk_ref_cols)
+            sql_fk_ref_cols= str_g_append(sql_fk_ref_cols, ", ");
+
+          sql_fk_ref_cols= str_g_append_and_free(sql_fk_ref_cols, g_strdup_printf("%s%s%s", 
+              quote_char,
+              myx_grt_list_item_get_as_string(fk_ref_column_names, j),
+              quote_char));
+        }
+      }
+
+      sql_fk= str_g_append_and_free(sql_fk, 
+        g_strdup_printf("  CONSTRAINT %s%s%s FOREIGN KEY %s%s%s (%s)" _br
+          "    REFERENCES %s%s%s.%s%s%s (%s)", 
+          quote_char, fk_name, quote_char,
+          quote_char, fk_name, quote_char,
+          sql_fk_cols,
+          quote_char, fk_ref_schema_name, quote_char,
+          quote_char, fk_ref_table_name, quote_char,
+          sql_fk_ref_cols));
 
       if (delete_rule)
-        sql_fk= str_g_append_and_free(sql_fk, g_strdup_printf("    ON DELETE %s" _br, delete_rule));
+        sql_fk= str_g_append_and_free(sql_fk, g_strdup_printf(_br "    ON DELETE %s", delete_rule));
 
       if (update_rule)
-        sql_fk= str_g_append_and_free(sql_fk, g_strdup_printf("    ON UPDATE %s" _br, update_rule));
+        sql_fk= str_g_append_and_free(sql_fk, g_strdup_printf(_br "    ON UPDATE %s", update_rule));
 
       sql_fk= str_g_append(sql_fk, "," _br);
 
@@ -684,12 +849,33 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
 
   if ((opt_charset) && (opt_charset[0]))
   {
-    sql= str_g_append_and_free(sql, g_strdup_printf("CHARACTER SET %s", opt_charset));
+    int skip_collation= 0;
 
-    if ((opt_collation) && (opt_collation[0]))
-      sql= str_g_append_and_free(sql, g_strdup_printf(" COLLATE %s" _br, opt_collation));
-    else
-      sql= str_g_append(sql, _br);
+    MYX_GRT_VALUE *schema= myx_grt_dict_item_get_reference_value(grt, table, "owner");
+    if (schema)
+    {
+      MYX_GRT_VALUE *catalog= myx_grt_dict_item_get_reference_value(grt, schema, "owner");
+      if (catalog)
+      {
+        MYX_GRT_VALUE *version= myx_grt_dict_item_get_value(catalog, "version");
+        if (version)
+        {
+          if ((myx_grt_dict_item_get_as_int(version, "major") == 4) &&
+            (myx_grt_dict_item_get_as_int(version, "minor") == 0))
+            skip_collation= 1;
+        }
+      }
+    }
+
+    if (!skip_collation)
+    {
+      sql= str_g_append_and_free(sql, g_strdup_printf("CHARACTER SET %s", opt_charset));
+
+      if ((opt_collation) && (opt_collation[0]))
+        sql= str_g_append_and_free(sql, g_strdup_printf(" COLLATE %s" _br, opt_collation));
+      else
+        sql= str_g_append(sql, _br);
+    }
   }
 
   // remove last \n
@@ -701,6 +887,9 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
 
   sql= str_g_append(sql, ";");
 
+  if (myx_grt_dict_item_get_as_int(table, "commentedOut") == 1)
+    sql= comment_out(sql);
+
   g_free(quote_char);
   g_free(quoted_schema_name);
 
@@ -714,7 +903,6 @@ static char * get_sql_create_view(MYX_GRT *grt, MYX_GRT_VALUE *view)
   char *quoted_schema_name= get_owner_name(grt, view, quote_char);
   const char *queryExpression= myx_grt_dict_item_get_as_string(view, "queryExpression");
   int with_check_condition= myx_grt_dict_item_get_as_int(view, "withCheckCondition");
-  int commented_out= myx_grt_dict_item_get_as_int(view, "commentedOut");
   char *sql;
 
   sql= g_strdup_printf("CREATE OR REPLACE VIEW %s%s%s%s AS" _br
@@ -735,13 +923,8 @@ static char * get_sql_create_view(MYX_GRT *grt, MYX_GRT_VALUE *view)
 
   sql= str_g_append(sql, ";");
 
-
-  if (commented_out == 1)
-  {
-    char *sql_commented_out= g_strdup_printf("/*%s*/", sql);
-    g_free(sql);
-    sql= sql_commented_out;
-  }
+  if (myx_grt_dict_item_get_as_int(view, "commentedOut") == 1)
+    sql= comment_out(sql);
     
   g_free(quote_char);
   g_free(quoted_schema_name);
@@ -749,11 +932,14 @@ static char * get_sql_create_view(MYX_GRT *grt, MYX_GRT_VALUE *view)
   return sql;
 }
 
-static char * get_sql_create_procedure(MYX_GRT *grt, MYX_GRT_VALUE *procedure)
+static char * get_sql_create_routine(MYX_GRT *grt, MYX_GRT_VALUE *routine)
 {
-  const char *procedure_code= myx_grt_dict_item_get_as_string(procedure, "procedureCode");
+  char *sql= g_strdup(myx_grt_dict_item_get_as_string(routine, "routineCode"));
 
-  return g_strdup(procedure_code);
+  if (myx_grt_dict_item_get_as_int(routine, "commentedOut") == 1)
+    sql= comment_out(sql);
+
+  return sql;
 }
 
 static char * get_sql_drop_schema(MYX_GRT *grt, MYX_GRT_VALUE *schema)
@@ -766,6 +952,9 @@ static char * get_sql_drop_schema(MYX_GRT *grt, MYX_GRT_VALUE *schema)
 
   g_free(quote_char);
 
+  if (myx_grt_dict_item_get_as_int(schema, "commentedOut") == 1)
+    sql= comment_out(sql);
+
   return sql;
 }
 
@@ -784,6 +973,9 @@ static char * get_sql_drop_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
   g_free(quote_char);
   g_free(quoted_schema_name);
 
+  if (myx_grt_dict_item_get_as_int(table, "commentedOut") == 1)
+    sql= comment_out(sql);
+
   return sql;
 }
 
@@ -802,23 +994,39 @@ static char * get_sql_drop_view(MYX_GRT *grt, MYX_GRT_VALUE *view)
   g_free(quote_char);
   g_free(quoted_schema_name);
 
+  if (myx_grt_dict_item_get_as_int(view, "commentedOut") == 1)
+    sql= comment_out(sql);
+
   return sql;
 }
 
-static char * get_sql_drop_procedure(MYX_GRT *grt, MYX_GRT_VALUE *procedure)
+static char * get_sql_drop_routine(MYX_GRT *grt, MYX_GRT_VALUE *routine)
 {
   char *quote_char= g_strdup("`");
-  const char *procedure_name= myx_grt_dict_item_get_as_string(procedure, "name");
-  char *quoted_schema_name= get_owner_name(grt, procedure, quote_char);
-  const char *procedure_type= myx_grt_dict_item_get_as_string(procedure, "procedureType");
+  const char *routine_name= myx_grt_dict_item_get_as_string(routine, "name");
+  char *quoted_schema_name= get_owner_name(grt, routine, quote_char);
+  const char *routine_type= myx_grt_dict_item_get_as_string(routine, "routineType");
 
   char *sql= g_strdup_printf("DROP %s IF EXISTS %s%s%s%s;",
-    procedure_type,
+    routine_type,
     quoted_schema_name,
-    quote_char, procedure_name, quote_char);
+    quote_char, routine_name, quote_char);
     
   g_free(quote_char);
   g_free(quoted_schema_name);
+
+  if (myx_grt_dict_item_get_as_int(routine, "commentedOut") == 1)
+    sql= comment_out(sql);
     
   return sql;
 }
+
+static char * comment_out(char *sql)
+{
+  char *sql_commented_out= g_strdup_printf("-- %s", sql);
+  g_free(sql);
+
+  sql_commented_out= str_g_replace(sql_commented_out, "\n", "\n-- ");
+  
+  return sql_commented_out;
+}
