@@ -123,6 +123,7 @@ static int generate_java_classes(const char *structs_file, const char *output_pa
     char *output_path= g_strdup(output_path_param);
     char *package_name= g_strdup(filename+8);
     char path_sep[]= {MYX_PATH_SEPARATOR, 0};
+    int error_no;
 
     g_free(filename);
 
@@ -131,7 +132,7 @@ static int generate_java_classes(const char *structs_file, const char *output_pa
     if (output_path[strlen(output_path)-1] != MYX_PATH_SEPARATOR)
       output_path= str_g_append(output_path, path_sep);
     
-    myx_mkdir(output_path, 0755);
+    myx_mkdir(output_path, 0755, &error_no);
     
     error= myx_grt_struct_export_java_classes(gstructs, package_name, output_path);
     g_free(package_name);
@@ -173,13 +174,14 @@ static int generate_php_classes(const char *structs_file, const char *output_pat
   {
     char *output_path= g_strdup(output_path_param);
     char path_sep[]= {MYX_PATH_SEPARATOR, 0};
+    int error_no;
 
     g_free(filename);
 
     if (output_path[strlen(output_path)-1] != MYX_PATH_SEPARATOR)
       output_path= str_g_append(output_path, path_sep);
     
-    myx_mkdir(output_path, 0755);
+    myx_mkdir(output_path, 0755, &error_no);
     
     error= myx_grt_struct_export_php_classes(gstructs, output_path);
     
@@ -234,8 +236,13 @@ static int process_messages(MYX_GRT_MSGS *msgs, void *user_data)
   return 0;
 }
 
+void process_output(const char *text, void *user_data)
+{
+  printf("%s", text);
+}
 
-MYX_GRT *initialize_grt(const char *classpath, int interactive, int use_forms, int grt_options,
+
+MYX_GRT *initialize_grt(const char *classpath, const char *jvm_path, int interactive, int use_forms, int grt_options,
                         int listen_port)
 {
   MYX_GRT *grt= myx_grt_initialize(grt_options);
@@ -244,69 +251,79 @@ MYX_GRT *initialize_grt(const char *classpath, int interactive, int use_forms, i
   unsigned int c;
 
   myx_grt_set_message_callback(grt, NULL, &process_messages);
+  myx_grt_set_output_callback(grt, NULL, &process_output);
 
   if (interactive)
-    g_message("Scanning for struct definitions in ./xml ...");
+    myx_grt_shell_print_welcome(grt);
+
+
+  if (interactive)
+    printf("\nScanning for struct definitions in ./xml ...\n");
   c= myx_grt_scan_for_structs(grt, "./xml", &error);
   if (error != MYX_GRT_NO_ERROR)
   {
     g_warning("Error while scanning for struct definitions (%i).", error);
   }
   else if (interactive)
-    g_message("Registered %i struct definition files.", c);
+    printf("Registered %i struct definition files.\n", c);
 
   if (interactive)
-    g_message("Initializing Builtin modules...");
+    printf("Initializing Builtin modules...\n");
   myx_register_builtin_grt_module_base(grt);
   myx_register_builtin_grt_module_reverse_engineer_mysql(grt);
   myx_register_builtin_grt_module_transformation_mysql(grt);
 
+#ifdef ENABLE_FORMS
 #if !defined(__WIN__) && !defined(_WIN32) && !defined(_WIN64)
   // register forms module
   if (use_forms)
     myx_register_builtin_grt_module_forms(grt);
+#endif
 #endif
 
   // initialize the loaders
 
 #ifdef ENABLE_JAVA_MODULES
   // initialize Java loader
-  if (interactive)
-    g_message("Initializing Java loader...");
-  loader= myx_java_init_loader(grt, classpath, &error, NULL);
-  if (!loader)
+  if (g_file_test("./java", G_FILE_TEST_IS_DIR))
   {
-    g_warning("Error initializing Java module loader (%i).", error);
-  }
-  else
-  {
-    if (myx_grt_register_module_loader(grt, loader) < 0)
+    if (interactive)
+      printf("Initializing Java loader...\n");
+    loader= myx_java_init_loader(grt, classpath, &error, jvm_path, "./");
+    if (!loader)
     {
-      g_warning("Could not register Java module loader.");
+      g_warning("Error initializing Java module loader (%i).", error);
     }
     else
     {
-      // Scan for Java modules
-      if (interactive)
-        g_message("Scanning for Java modules in ./java/com/mysql/grt/modules ...");
-
-      c= myx_grt_scan_for_modules(grt, "./java/com/mysql/grt/modules", &error);
-      if (error!=MYX_GRT_NO_ERROR)
+      if (myx_grt_register_module_loader(grt, loader) < 0)
       {
-        g_warning("Error while scanning for Java modules (%i).", error);
+        g_warning("Could not register Java module loader.");
+      }
+      else
+      {
+        // Scan for Java modules
+        if (interactive)
+          printf("Scanning for Java modules in ./java/com/mysql/grt/modules ...\n");
+
+        c= myx_grt_scan_for_modules(grt, "./java/com/mysql/grt/modules", &error);
+        if (error!=MYX_GRT_NO_ERROR)
+        {
+          g_warning("Error while scanning for Java modules (%i).", error);
+        }
+        else if (interactive)
+          printf("Registered %i modules.\n", c);
       }
-      else if (interactive)
-        g_message("Registered %i modules.", c);
     }
   }
 #endif
 
 #ifdef ENABLE_PHP_MODULES
   // initialize PHP loader
-  if (g_file_test("./path", G_FILE_TEST_IS_DIR))
+  if (g_file_test("./php", G_FILE_TEST_IS_DIR))
   {
     if (interactive)
-      g_message("Initializing PHP loader...");
+      printf("Initializing PHP loader...\n");
     loader= myx_php_init_loader(grt, &error);
     if (!loader)
     {
@@ -322,7 +339,7 @@ MYX_GRT *initialize_grt(const char *classpath, int interactive, int use_forms, i
       {
         // Scan for PHP modules
         if (interactive)
-          g_message("Scanning for php modules in ./php/modules ...");
+          printf("Scanning for php modules in ./php/modules ...\n");
 
         c= myx_grt_scan_for_modules(grt, "./php/modules", &error);
         if (error!=MYX_GRT_NO_ERROR)
@@ -330,7 +347,7 @@ MYX_GRT *initialize_grt(const char *classpath, int interactive, int use_forms, i
           g_warning("Error while scanning for PHP modules (%i).", error);
         }
         else if (interactive)
-          g_message("Registered %i modules.", c);
+          printf("Registered %i modules.\n", c);
       }
     }
   }
@@ -338,8 +355,8 @@ MYX_GRT *initialize_grt(const char *classpath, int interactive, int use_forms, i
 
   // initialize Lua loader
   if (interactive)
-    g_message("Initializing Lua loader...");
-  loader= myx_lua_init_loader(grt, &error);
+    printf("Initializing Lua loader...\n");
+  loader= myx_lua_init_loader(grt, &error, "./lua");
   if (!loader)
   {
     g_warning("Error initializing Lua module loader (%i).", error);
@@ -359,7 +376,7 @@ MYX_GRT *initialize_grt(const char *classpath, int interactive, int use_forms, i
     {
       // Scan for Lua modules
       if (interactive)
-        g_message("Scanning for Lua plugins in ./lua ...");
+        printf("Scanning for Lua plugins in ./lua ...\n");
       
       c= myx_grt_scan_for_modules(grt, "./lua", &error);
       if (error != MYX_GRT_NO_ERROR)
@@ -367,10 +384,13 @@ MYX_GRT *initialize_grt(const char *classpath, int interactive, int use_forms, i
         g_warning("Error while scanning for Lua modules (%i).", error);
         }
       else if (interactive)
-        g_message("Registered %i modules.", c);
+        printf("Registered %i modules.\n", c);
     }
   }
 
+  if (interactive)
+    printf("\n");
+
   myx_grt_init_lua_shell(grt);
 
   return grt;
@@ -380,15 +400,21 @@ MYX_GRT *initialize_grt(const char *classpath, int interactive, int use_forms, i
 static void show_help(const char *argv0)
 {
     fprintf(stdout, 
-      "Usage: %s [-classpath path] [-d path] [-listen port] [-x] [luafile] \n"
-      "       %s -j structsfile outputdir\n\n"    
+      "Usage: %s [-classpath path] [-jvm library] [-d path] [-listen port] [-verbose] [-x] [luafile] \n"
+      "       %s -j structsfile outputdir\n"    
+      "       %s -p structsfile outputdir\n"
+      "\n"
       "  -classpath ... Sets the java classpath to the given value.\n"
+      "  -jvm ......... The java virtual machine library to use (with absolut path).\n"
       "  -d path ...... Modules directory\n"
       "  -x ........... Exits the shell after running the specified file\n"
       "  luafile ...... File that is run at startup.\n\n"
-      "  -j ........... Generates java classes from the given structs file.\n"
       "  -listen port . Runs in 'remote agent' mode on the given port number.\n"
-      "\n", argv0, argv0);
+      "  -verbose ..... Prints detailed startup information.\n"
+      "  -j ........... Generates Java classes from the given structs file.\n"
+      "  -p ........... Generates PHP classes from the given structs file.\n"
+      "  -D var=value . Sets a global shell variable to the given value.\n"
+      "\n", argv0, argv0, argv0);
 }
 
 int main(int argc, char **argv)
@@ -397,6 +423,7 @@ int main(int argc, char **argv)
   MYX_GRT *grt;
   int grt_options= 0;
   char *classpath= NULL;
+  char *jvm_path= NULL;
   char *linebuf= NULL;
   char *load_file= NULL;
   char *module_dir= NULL;
@@ -467,6 +494,8 @@ int main(int argc, char **argv)
       grt_options+= 2;
     else if (strcmp(argv[i], "-listen") == 0)
     {
+      interactive= 0;
+
       if (argc <= i+1)
       {
         show_help(argv[0]);
@@ -495,17 +524,36 @@ int main(int argc, char **argv)
       else
         classpath= argv[++i];
     }
+    else if (strcmp(argv[i], "-jvm")==0)
+    {
+      if (argc <= i+1)
+      {
+        show_help(argv[0]);
+        exit(1);
+      }
+      else
+        jvm_path= argv[++i];
+    }
+    else if (strcmp(argv[i], "-D")==0)
+    {
+      if (argc <= i+1)
+      {
+        show_help(argv[0]);
+        exit(1);
+      }
+      else
+        i++;
+    }
     else
       load_file= argv[i];
   }
-
   
   if (listen_port > 0)
     g_thread_init(NULL);
 
   // ---------------------------------------------------------------------------------------------------------------
   //Initialize grt
-  grt= initialize_grt(classpath, interactive, use_forms, grt_options, listen_port);
+  grt= initialize_grt(classpath, jvm_path, interactive, use_forms, grt_options, listen_port);
   if (!grt)
   {
     g_message("ERROR: The GRT environment cannot be initialized.");
@@ -514,6 +562,36 @@ int main(int argc, char **argv)
 
   g_assert(lua_gettop(myx_grt_lua_shell_get_lua(grt))==0);
 
+  // ---------------------------------------------------------------------------------------------------------------
+  //Set global values
+  for (i= 1; i < argc; i++)
+  {
+    if (strcmp(argv[i], "-D") == 0)
+    {
+      if (argc <= i+1)
+      {
+        show_help(argv[0]);
+        exit(1);
+      }
+      else
+      {
+        char *param= argv[++i];
+        char *var_name= g_strdup(param);
+        char *var_val= g_strdup(param);
+          
+        name_of_str(var_name, var_name);
+        value_of_str(var_val, var_val);
+
+        if (var_val[0])
+        {
+          MYX_GRT_VALUE *val= myx_grt_value_from_string(var_val);
+
+          myx_grt_lua_shell_set_global_var(grt, var_name, val);
+        }
+      }
+    }
+  }
+
   // ---------------------------------------------------------------------------------------------------------------
   // Scan for plugins
   if (module_dir)
@@ -559,6 +637,8 @@ int main(int argc, char **argv)
     // Main interactive loop
     for (;;)
     {
+      //MYX_GRT_VALUE *error;
+
       linebuf= read_command(grt);
       if (!linebuf)
       {
@@ -567,6 +647,15 @@ int main(int argc, char **argv)
       
       if(myx_grt_lua_shell_execute(grt, linebuf) == MYX_GRT_SHELL_COMMAND_EXIT)
         break;
+
+      // check for error
+      /*error= myx_grt_lua_shell_get_global_var(grt, "grtError");
+      if (error)
+      {
+        printf("ERROR: %s\n", myx_grt_dict_item_get_as_string(error, "error"));
+        if (myx_grt_dict_item_get_as_string(error, "detail"))
+          printf("%s\n", myx_grt_dict_item_get_as_string(error, "detail"));
+      }*/
       
       myx_grt_messages_stack_flush(grt, 0);
       
