@@ -1,3 +1,22 @@
+/*
+ Generic Runtime Library (GRT)
+ Copyright (C) 2005 MySQL AB
+ 
+ This library is free software; you can redistribute it and/or
+ modify it under the terms of the GNU Lesser General Public
+ License as published by the Free Software Foundation; either
+ version 2.1 of the License, or (at your option) any later version.
+
+ This library is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ Lesser General Public License for more details.
+
+ You should have received a copy of the GNU Lesser General Public
+ License along with this library; if not, write to the Free Software
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA 
+ */
+
 package com.mysql.grt;
 
 import java.util.Vector;
@@ -6,6 +25,7 @@ import java.lang.Class;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
+import java.io.FileWriter;
 
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
@@ -13,6 +33,7 @@ import javax.xml.parsers.ParserConfigurationException;
 
 import org.xml.sax.SAXException;
 
+import java.io.File;
 import java.io.IOException;
 import java.io.ByteArrayInputStream;
 import java.io.UnsupportedEncodingException;
@@ -38,10 +59,14 @@ public class Grt {
 
 	private String applicationPath;
 
+	private String applicationDataPath = "";
+
 	private HashMap objects = new HashMap();
 
 	private Vector msgs = new Vector();
 
+	private int doJava2GrtXmlLogging = 0;
+
 	public static final int MYX_INT_VALUE = 1;
 
 	public static final int MYX_REAL_VALUE = 2;
@@ -62,6 +87,8 @@ public class Grt {
 	 * Private class constructor.
 	 */
 	private Grt() {
+		// dummy call to avoid Eclipse warning
+		setMyxGrtPointer(0);
 	}
 
 	/**
@@ -78,6 +105,74 @@ public class Grt {
 		return instance;
 	}
 
+	/**
+	 * String replace function because String.replaceAll is broken
+	 * 
+	 * @param orig
+	 *            the original string
+	 * @param searchFor
+	 *            the substring to search for
+	 * @param replacement
+	 *            the string the substring should be replaced with
+	 * 
+	 * @return returns the string with all searchFor substrings replaced
+	 */
+	public static String replace(String orig, String searchFor,
+			String replacement) {
+		if (orig == null || searchFor == null || searchFor.equals("")) {
+			String msg = "Can not perform replace." //
+					+ " orig: \"" + orig + "\"" //
+					+ " searchFor: \"" + searchFor + "\"" //
+					+ " replacement: \"" + replacement;
+			throw new IllegalArgumentException(msg);
+		}
+
+		int sizeWildGuess = orig.length() + replacement.length() + 20;
+		StringBuffer buf = new StringBuffer(sizeWildGuess);
+		replaceInner(buf, orig, searchFor, replacement);
+		return buf.toString();
+	}
+
+	private static void replaceInner(StringBuffer buf, String orig,
+			String searchFor, String replacement) {
+
+		int index = orig.indexOf(searchFor);
+		if (index < 0) {
+			buf.append(orig);
+			return;
+		}
+
+		String left = orig.substring(0, index);
+		buf.append(left);
+		buf.append(replacement);
+
+		String right = orig.substring(index + searchFor.length());
+		replaceInner(buf, right, searchFor, replacement);
+	}
+
+	/**
+	 * Deletes a directory and all its files and sub-directories
+	 * 
+	 * @param dir
+	 *            the directory to delete
+	 * 
+	 * @return true, if the directory could be deleted
+	 */
+	public static boolean deleteDir(File dir) {
+		if (dir.isDirectory()) {
+			String[] children = dir.list();
+			for (int i = 0; i < children.length; i++) {
+				boolean success = deleteDir(new File(dir, children[i]));
+				if (!success) {
+					return false;
+				}
+			}
+		}
+
+		// The directory is now empty so delete it
+		return dir.delete();
+	}
+
 	/**
 	 * This function sets the callback object that is used to call back to the
 	 * JNI.
@@ -120,6 +215,31 @@ public class Grt {
 		return callback;
 	}
 
+	/**
+	 * Enables or disables the logging of xml passed between Java and the GRT
+	 * for objects. The xml will be written to a Java2Grt.xml in the application
+	 * dir.
+	 * 
+	 * @param enable
+	 *            if set to 1, the logging is enabled
+	 */
+	public void java2GrtXmlLogging(int enable) {
+		if (enable == 1)
+			this.doJava2GrtXmlLogging = 1;
+		else
+			this.doJava2GrtXmlLogging = 0;
+	}
+
+	/**
+	 * Returns the current state of the logging of xml passed between Java and
+	 * the GRT
+	 * 
+	 * @return returns 1 if the logging is enabled
+	 */
+	public int java2GrtXmlLogging() {
+		return this.doJava2GrtXmlLogging;
+	}
+
 	/**
 	 * returns the applicationPath string. Has to be set before with
 	 * setApplicationPath
@@ -130,6 +250,20 @@ public class Grt {
 		return applicationPath;
 	}
 
+	/**
+	 * returns the applicationPath string. Has to be set before with
+	 * setApplicationPath
+	 * 
+	 * @return returns the applicationPath string
+	 */
+	public String getApplicationDataPath() {
+		if (applicationDataPath == "")
+			applicationDataPath = (String) Grt.getInstance().callGrtFunction(
+					"Base", "getAppDataDir", null);
+
+		return applicationDataPath;
+	}
+
 	/**
 	 * returns the myxGrtPointer used for callGrtFunction calls casted as a long
 	 * 
@@ -173,7 +307,7 @@ public class Grt {
 
 	/**
 	 * Clears the object cache
-	 *  
+	 * 
 	 */
 	protected void clearObjectCache() {
 		Grt.getInstance().objects.clear();
@@ -317,7 +451,7 @@ public class Grt {
 
 	/**
 	 * Forces the output of the GRT messages
-	 *  
+	 * 
 	 */
 	public int flushMessages() {
 		int res = 0;
@@ -331,8 +465,12 @@ public class Grt {
 
 			msgs.clear();
 
-			res = getCallback().processMessages(getMyxGrtPointer(),
-					msgList.getGrtXml());
+			// print messages
+			getCallback().processMessages(getMyxGrtPointer(),
+					msgList.getGrtXml().toString());
+
+			// get status
+			res = getCallback().processStatusQuery(getMyxGrtPointer());
 		}
 
 		return res;
@@ -366,8 +504,15 @@ public class Grt {
 	 *            a XML string without a XML header and root node
 	 * @return a complete XML document
 	 */
-	public static String prepareGrtXml(String xml) {
-		return "<?xml version=\"1.0\"?>\n<data>\n" + xml + "</data>\n";
+	public static String prepareGrtXml(StringBuffer xml) {
+		StringBuffer grtXml = new StringBuffer(
+				"<?xml version=\"1.0\"?>\n<data>\n");
+
+		grtXml.append(xml);
+
+		grtXml.append("</data>\n");
+
+		return grtXml.toString();
 	}
 
 	/**
@@ -395,6 +540,35 @@ public class Grt {
 		return className1.equals(className2);
 	}
 
+	private static boolean assignGrtListToGrtObject(GrtObject obj,
+			Method setter, GrtList sourceList, Class objListClass) {
+
+		try {
+			Constructor con = objListClass
+					.getConstructor(new Class[] { GrtObject.class });
+			GrtList paramList = (GrtList) con.newInstance(new Object[] { obj });
+
+			Class paramListContentClass = Class.forName(GrtPackagePrefix
+					+ paramList.getContentStructName());
+
+			Class sourceListContentClass = Class.forName(GrtPackagePrefix
+					+ sourceList.getContentStructName());
+
+			if (paramListContentClass.isAssignableFrom(sourceListContentClass)) {
+				for (int k = 0; k < sourceList.size(); k++) {
+					paramList.addObject(sourceList.getObject(k));
+				}
+
+				setter.invoke(obj, new Object[] { paramList });
+
+				return true;
+			}
+		} catch (Exception e) {
+		}
+
+		return false;
+	}
+
 	/**
 	 * Parses a given XML Node of the value type DICT Calls other functions
 	 * recursive to get the complete information
@@ -452,6 +626,9 @@ public class Grt {
 							String option = ((Element) childValueNode)
 									.getAttribute("option");
 
+							String itemType = ((Element) childValueNode)
+									.getAttribute("type");
+
 							// build setter name
 							if (setter.length() == 0) {
 								continue;
@@ -463,7 +640,8 @@ public class Grt {
 										+ setter.substring(1);
 							}
 							setter = "set" + setter;
-							if ((option != null) && (option.equals("ref")))
+							if ((option != null) && (option.equals("ref"))
+									&& !itemType.equalsIgnoreCase("list"))
 								setter = setter + "ById";
 
 							// loop over all publicMethods and see if there
@@ -472,29 +650,33 @@ public class Grt {
 							Method[] methods = c.getMethods();
 							for (int j = 0; j < methods.length; j++) {
 
-								/*
-								 * if ((methods[j].getName().equals(setter)) &&
-								 * (methods[j].getParameterTypes().length > 0)) {
-								 * System.out.println(methods[j].getName() + ": " +
-								 * methods[j].getParameterTypes()[0] .getName() + " - " +
-								 * setter + ": " + item.getClass().getName()); }
-								 */
-
 								if ((methods[j].getName().equals(setter))
-										&& (methods[j].getParameterTypes().length > 0)
-										&& (checkSameClass(methods[j]
-												.getParameterTypes()[0], item
-												.getClass()))) {
+										&& (methods[j].getParameterTypes().length > 0)) {
 
-									methods[j].invoke(obj,
-											new Object[] { item });
+									// check if the classes are both inherited
+									// from GrtList
+									if (GrtList.class.isAssignableFrom(item
+											.getClass())
+											&& GrtList.class
+													.isAssignableFrom(methods[j]
+															.getParameterTypes()[0])) {
+										// if so, try to assign the lists
+										if (assignGrtListToGrtObject(
+												obj,
+												methods[j],
+												(GrtList) item,
+												methods[j].getParameterTypes()[0]))
+											break;
 
-									/*
-									 * System.out.println(methods[j].getName() +
-									 * "(" + item.toString() + ")");
-									 */
+									} else if ((checkSameClass(methods[j]
+											.getParameterTypes()[0], item
+											.getClass()))) {
 
-									break;
+										methods[j].invoke(obj,
+												new Object[] { item });
+
+										break;
+									}
 								}
 							}
 						}
@@ -555,6 +737,7 @@ public class Grt {
 		String globalObjectPath = ((Element) valueNode)
 				.getAttribute("global-object-path");
 		int valueListContentType = getValueTypeFromString(contentType);
+		boolean objectRefList = false;
 
 		// TODO: the following is a hack, it should be implemented using a
 		// ObjectRefList instead
@@ -565,6 +748,11 @@ public class Grt {
 				&& (valueListContentType == Grt.MYX_STRING_VALUE)
 				&& (contentStructName != null) && (contentStructName != "")) {
 			valueListContentType = Grt.MYX_DICT_VALUE;
+		} else if (((globalObjectPath == null) || globalObjectPath.equals(""))
+				&& (valueListContentType == Grt.MYX_STRING_VALUE)
+				&& (contentStructName != null) && (contentStructName != "")) {
+			valueListContentType = Grt.MYX_DICT_VALUE;
+			objectRefList = true;
 		}
 
 		// if a content-struct-name was assigned, create list of given type
@@ -610,6 +798,11 @@ public class Grt {
 				// check if returned object is not null
 				Object item = getObjectsFromGrtXmlNode(childValueNode, list);
 
+				// if this is a objectRefList, take real object instead
+				if (objectRefList) {
+					item = Grt.getInstance().getObjectByRefId((String) item);
+				}
+
 				if (item != null)
 					list.addObject(item);
 			}
@@ -692,8 +885,7 @@ public class Grt {
 	public static Object getObjectsFromGrtXml(String xml) {
 		Document document;
 		ByteArrayInputStream stream;
-		int rootNodeType;
-		int contentType;
+
 		Object obj = null;
 
 		if (xml == null)
@@ -706,8 +898,8 @@ public class Grt {
 		}
 
 		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
-		//factory.setValidating(true);
-		//factory.setNamespaceAware(true);
+		// factory.setValidating(true);
+		// factory.setNamespaceAware(true);
 		try {
 			DocumentBuilder builder = factory.newDocumentBuilder();
 			document = builder.parse(stream);
@@ -752,7 +944,7 @@ public class Grt {
 	 *            the object to serialize
 	 * @return the XML string
 	 */
-	public static String getObjectAsXml(Object obj) {
+	public static StringBuffer getObjectAsXml(Object obj) {
 		return getObjectAsXml(obj, "");
 	}
 
@@ -766,24 +958,26 @@ public class Grt {
 	 *            a DICT
 	 * @return the XML string
 	 */
-	public static String getObjectAsXml(Object obj, String keyTag) {
-		String xml = new String();
+	public static StringBuffer getObjectAsXml(Object obj, String keyTag) {
+		StringBuffer xml = new StringBuffer();
 
 		if (obj != null) {
 			// get XML from GrtObjects, GrtLists and GrtHashMaps
 			if (obj instanceof GrtXml) {
-				xml = xml + ((GrtXml) obj).getGrtXml(keyTag);
+				xml.append(((GrtXml) obj).getGrtXml(keyTag));
 			} else {
 				// get XML for primitive types
 				if (obj.getClass() == String.class) {
-					xml = xml + "<value type=\"string\"" + keyTag + ">"
-							+ escapeStringForXml(obj.toString()) + "</value>\n";
+					xml
+							.append("<value type=\"string\"" + keyTag + ">"
+									+ escapeStringForXml(obj.toString())
+									+ "</value>\n");
 				} else if (obj.getClass() == Integer.class) {
-					xml = xml + "<value type=\"int\"" + keyTag + ">"
-							+ obj.toString() + "</value>\n";
+					xml.append("<value type=\"int\"" + keyTag + ">"
+							+ obj.toString() + "</value>\n");
 				} else if (obj.getClass() == Double.class) {
-					xml = xml + "<value type=\"real\"" + keyTag + ">"
-							+ obj.toString() + "</value>\n";
+					xml.append("<value type=\"real\"" + keyTag + ">"
+							+ obj.toString() + "</value>\n");
 				}
 			}
 		}
@@ -807,44 +1001,45 @@ public class Grt {
 	 * @return returns a GRT XML string containing the infos about this class
 	 */
 	public static String getModuleInfoXml(Class module, String extendsModule) {
-		String xml = "<value type=\"dict\">\n"
-				+ "  <value key=\"name\" type=\"string\">"
-				+ module.getName()
-				+ "</value>"
-				+ "  <value key=\"functions\" type=\"list\" content-type=\"string\">\n";
+		StringBuffer xml = new StringBuffer(
+				"<value type=\"dict\">\n"
+						+ "  <value key=\"name\" type=\"string\">"
+						+ module.getName()
+						+ "</value>"
+						+ "  <value key=\"functions\" type=\"list\" content-type=\"string\">\n");
 
-		//List public static methods that take a String parameter and return a
+		// List public static methods that take a String parameter and return a
 		// String
 		Method[] methods = module.getMethods();
 
 		for (int i = 0; i < methods.length; i++) {
 			if ((Modifier.isStatic(methods[i].getModifiers()))
 					&& (!methods[i].getName().equals("getModuleInfo"))) {
-				xml = xml + "    <value type=\"string\">"
-						+ methods[i].getName() + ":(";
+				xml.append("    <value type=\"string\">" + methods[i].getName()
+						+ ":(");
 
 				Class[] params = methods[i].getParameterTypes();
 				for (int j = 0; j < params.length; j++) {
 					if (params[j].getName().equals("int"))
-						xml = xml + "int;";
+						xml.append("int;");
 					if (params[j].getName().equals("double"))
-						xml = xml + "double;";
+						xml.append("double;");
 					else
-						xml = xml + "L" + params[j].getName().replace('.', '/')
-								+ ";";
+						xml.append("L" + params[j].getName().replace('.', '/')
+								+ ";");
 				}
 
-				xml = xml + "):</value>\n";
+				xml.append("):</value>\n");
 			}
 		}
 
-		xml = xml + "</value>\n";
+		xml.append("</value>\n");
 
 		if (!extendsModule.equals(""))
-			xml = xml + "  <value key=\"extends\" type=\"string\">"
-					+ extendsModule + "</value>\n";
+			xml.append("  <value key=\"extends\" type=\"string\">"
+					+ extendsModule + "</value>\n");
 
-		xml = xml + "</value>\n";
+		xml.append("</value>\n");
 
 		return prepareGrtXml(xml);
 	}
@@ -990,6 +1185,32 @@ public class Grt {
 		result.addObject("value", resultValue);
 		String xmlResult = prepareGrtXml(getObjectAsXml(result));
 
+		if (Grt.getInstance().doJava2GrtXmlLogging == 1) {
+			try {
+				boolean logDirExists = false;
+				File dir = new File(Grt.getInstance().getApplicationDataPath()
+						+ "log");
+
+				if (dir.exists() && dir.isDirectory()) {
+					logDirExists = true;
+				} else {
+					logDirExists = dir.mkdir();
+				}
+
+				if (logDirExists) {
+					FileWriter xmlFile = new FileWriter(Grt.getInstance()
+							.getApplicationDataPath()
+							+ "log/Java2Grt_"
+							+ module.getName()
+							+ "."
+							+ functionName + ".xml");
+					xmlFile.write(xmlResult);
+					xmlFile.close();
+				}
+			} catch (Exception e) {
+			}
+		}
+
 		// clear object cache
 		Grt.getInstance().clearObjectCache();
 
@@ -1012,9 +1233,11 @@ public class Grt {
 	 */
 	public Object callGrtFunction(String module, String functionName,
 			Object arguments) throws RuntimeException {
-		Object result = getObjectsFromGrtXml(getCallback().callGrtFunction(
-				getMyxGrtPointer(), module, functionName,
-				prepareGrtXml(getObjectAsXml(arguments))));
+		String argumentsXml = prepareGrtXml(getObjectAsXml(arguments));
+		String resultXml = getCallback().callGrtFunction(getMyxGrtPointer(),
+				module, functionName, argumentsXml);
+
+		Object result = getObjectsFromGrtXml(resultXml);
 
 		if (result instanceof GrtHashMap) {
 			Object value = ((GrtHashMap) result).getObject("value");
