@@ -1,3 +1,22 @@
+/*
+ Generic Runtime Library (GRT)
+ Copyright (C) 2005 MySQL AB
+
+ This library is free software; you can redistribute it and/or
+ modify it under the terms of the GNU Lesser General Public
+ License as published by the Free Software Foundation; either
+ version 2.1 of the License, or (at your option) any later version.
+
+ This library is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ Lesser General Public License for more details.
+
+ You should have received a copy of the GNU Lesser General Public
+ License along with this library; if not, write to the Free Software
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA 
+ */
+
 package com.mysql.grt;
 
 public interface GrtCallback {
@@ -55,9 +74,11 @@ public interface GrtCallback {
 
 	public abstract int getGrtGlobalDictKeyCount(long myx_grt_pointer,
 			String objectPath);
-	
+
 	public abstract String getGrtGlobalDictKey(long myx_grt_pointer,
 			String objectPath, int index);
-	
-	public abstract int processMessages(long myx_grt_pointer, String msgs);
+
+	public abstract void processMessages(long myx_grt_pointer, String msgs);
+
+	public abstract int processStatusQuery(long myx_grt_pointer);
 }
\ No newline at end of file
