@@ -1,3 +1,22 @@
+/*
+ Generic Runtime Library (GRT)
+ Copyright (C) 2005 MySQL AB
+
+ This library is free software; you can redistribute it and/or
+ modify it under the terms of the GNU Lesser General Public
+ License as published by the Free Software Foundation; either
+ version 2.1 of the License, or (at your option) any later version.
+
+ This library is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ Lesser General Public License for more details.
+
+ You should have received a copy of the GNU Lesser General Public
+ License along with this library; if not, write to the Free Software
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA 
+ */
+
 package com.mysql.grt;
 
 import java.io.*;
@@ -13,7 +32,7 @@ public class GrtCallbackNative implements GrtCallback {
 			if (f.exists()) {
 				System.load(applicationPath + "msvcr71d.dll");
 			} else {
-				//if the debug lib is not there, it has to be a release
+				// if the debug lib is not there, it has to be a release
 				System.load(applicationPath + "msvcr71.dll");
 			}
 
@@ -31,6 +50,9 @@ public class GrtCallbackNative implements GrtCallback {
 			System.load(applicationPath + "libmysqlgrtbuiltinmodule.dll");
 		} else if (System.getProperty("os.name").indexOf("Linux") > -1) {
 			System.load(applicationPath + "myx_grt_java.so");
+		} else if (System.getProperty("os.name").toLowerCase().startsWith(
+				"mac os x")) {
+			System.load(applicationPath + "myx_grt_java.dylib");
 		}
 	}
 
@@ -99,9 +121,11 @@ public class GrtCallbackNative implements GrtCallback {
 
 	public native int getGrtGlobalDictKeyCount(long myx_grt_pointer,
 			String objectPath);
-	
+
 	public native String getGrtGlobalDictKey(long myx_grt_pointer,
 			String objectPath, int index);
-	
-	public native int processMessages(long myx_grt_pointer, String msgs);
+
+	public native void processMessages(long myx_grt_pointer, String msgs);
+
+	public native int processStatusQuery(long myx_grt_pointer);
 }
