@@ -1,3 +1,22 @@
+/*
+ Generic Runtime Library (GRT)
+ Copyright (C) 2005 MySQL AB
+
+ This library is free software; you can redistribute it and/or
+ modify it under the terms of the GNU Lesser General Public
+ License as published by the Free Software Foundation; either
+ version 2.1 of the License, or (at your option) any later version.
+
+ This library is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ Lesser General Public License for more details.
+
+ You should have received a copy of the GNU Lesser General Public
+ License along with this library; if not, write to the Free Software
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA 
+ */
+
 package com.mysql.grt;
 
 import java.io.BufferedWriter;
@@ -71,9 +90,9 @@ public class GrtCallbackSnapshot implements GrtCallback {
 
 			for (int i = 0; i < methods.length; i++) {
 				try {
-					if ( (!methods[i].getName().startsWith("get")) 
-							|| (methods[i].getName().equals("getGrtXml")) 
-							|| (methods[i].getName().equals("getGrtXmlMembers")) )
+					if ((!methods[i].getName().startsWith("get"))
+							|| (methods[i].getName().equals("getGrtXml"))
+							|| (methods[i].getName().equals("getGrtXmlMembers")))
 						continue;
 
 					String path = objectPath
@@ -164,20 +183,20 @@ public class GrtCallbackSnapshot implements GrtCallback {
 			Object value) {
 
 	}
-	
-	public void setGrtGlobalFromString(long myx_grt_pointer, 
-			String objectPath, String value) {
-		
+
+	public void setGrtGlobalFromString(long myx_grt_pointer, String objectPath,
+			String value) {
+
 	}
-	
-	public void setGrtGlobalFromInt(long myx_grt_pointer, 
-			String objectPath, int value) {
-		
+
+	public void setGrtGlobalFromInt(long myx_grt_pointer, String objectPath,
+			int value) {
+
 	}
-	
-	public void setGrtGlobalFromReal(long myx_grt_pointer, 
-			String objectPath, double value) {
-		
+
+	public void setGrtGlobalFromReal(long myx_grt_pointer, String objectPath,
+			double value) {
+
 	}
 
 	public int getGrtGlobalListSize(long myx_grt_pointer, String objectPath) {
@@ -216,17 +235,20 @@ public class GrtCallbackSnapshot implements GrtCallback {
 
 	}
 
-	public int processMessages(long myx_grt_pointer, String msgs) {
+	public void processMessages(long myx_grt_pointer, String msgs) {
+
+	}
+
+	public int processStatusQuery(long myx_grt_pointer) {
 		return 0;
 	}
-	
-	public int getGrtGlobalDictKeyCount(long myx_grt_pointer,
-			String objectPath) {
+
+	public int getGrtGlobalDictKeyCount(long myx_grt_pointer, String objectPath) {
 		return 0;
 	}
-	
-	public String getGrtGlobalDictKey(long myx_grt_pointer,
-			String objectPath, int index) {
+
+	public String getGrtGlobalDictKey(long myx_grt_pointer, String objectPath,
+			int index) {
 		return "";
 	}
 }
\ No newline at end of file
