@@ -11,12 +11,14 @@
 
 #include <myx_public_interface.h>
 
+@class MConnectionInfo;
+
 @interface MMySQLDispatcher : NSObject {
   pthread_t _tid;
 
   pthread_mutex_t _mysql_mx;
   MYSQL *_mysql;
-  MYX_USER_CONNECTION *_instance;
+  MConnectionInfo *_instance;
 
   pthread_mutex_t _request_mx;
   pthread_cond_t _request_ready;
@@ -24,7 +26,9 @@
 }
 
 - (id)initWithMySQL: (MYSQL*)mysql
-		   instance: (MYX_USER_CONNECTION*)instance;
+		   instance: (MConnectionInfo*)instance;
+
+- (NSString*)lastMySQLError;
 
 - (void)performSelector:(SEL)selector
                argument:(id)argument
@@ -80,4 +84,17 @@
            waitForWindow:(id)window
                  message:(NSString*)message;
 
+- (void*)performCallback:(void*(*)(MYSQL*,void*,void*,void*,void*))cback
+                argument:(void*)arg1
+                argument:(void*)arg2
+                argument:(void*)arg3
+                argument:(void*)arg4
+           waitForWindow:(id)window
+                 message:(NSString*)message;
+
+
+- (MYX_LIB_ERROR)performQuery:(NSString*)query
+                waitForWindow:(id)window
+                      message:(NSString*)message;
+
 @end
