@@ -13,6 +13,7 @@ typedef int (*MPtrFree)(void*);
   NSString *_caption;
   void *_pointer;
   MPtrFree _destroyer;
+  int _tag;
 }
 
 - (void)setCaption:(NSString*)caption;
@@ -20,5 +21,7 @@ typedef int (*MPtrFree)(void*);
 - (void*)pointer;
 - (void)setPointer:(void*)ptr;
 - (void)setPointer:(void*)ptr destroyer:(MPtrFree)destroyer;
+- (int)tag;
+- (void)setTag:(int)tag;
 
 @end
