@@ -314,9 +314,9 @@ MYX_PUBLIC_FUNC char * myx_reconstruct_embedded_sql(MYX_Q_SQL_STRIPPED *stripped
 MYX_PUBLIC_FUNC void myx_free_stripped_sql(MYX_Q_SQL_STRIPPED *stripped_sql);
 
 /* PDF output */
-MYX_PUBLIC_FUNC int myx_write_resultset_pdf(MYX_RESULTSET *rset, MYX_RS_PRINT_PARAMS *params);
+//MYX_PUBLIC_FUNC int myx_write_resultset_pdf(MYX_RESULTSET *rset, MYX_RS_PRINT_PARAMS *params);
 
-MYX_PUBLIC_FUNC int myx_generate_pdf(MYX_RESULTSET *resultset, char *pdf_filename);
+//MYX_PUBLIC_FUNC int myx_generate_pdf(MYX_RESULTSET *resultset, char *pdf_filename);
 
 
 #ifdef __cplusplus
