@@ -113,7 +113,7 @@ MYX_EXPLAIN_RESULT *myx_query_explain(MYSQL *mysql, const char *query)
     MYSQL_FIELD * fields= mysql_fetch_fields(res);
     int fi[10];
     build_field_subst(explain_fields,explain_fields+sizeof(explain_fields)/sizeof(char*),
-                      fields,fields+num_fields,(unsigned int*)fi);
+                      fields,fields+num_fields,fi);
 
     result= g_malloc(sizeof(MYX_EXPLAIN_RESULT));
     
