@@ -55,23 +55,25 @@ MYX_BOOKMARKS * myx_bookmarks_load(const char *filename, MYX_LIB_ERROR *error_co
   xmlDocPtr doc;
   xmlNodePtr root;
   MYX_BOOKMARKS *bookmarks;
-
+  char* local_filename;
   *error_code = MYX_NO_ERROR;
 
-  if (! file_exists(filename) )  {
-    *error_code = MYX_ERROR_CANT_OPEN_FILE;
-    return NULL;
-  }
-
-  if(file_size(filename) > MAX_VALID_OPTION_FILE_SIZE)
+  if (!(local_filename= g_filename_from_utf8(filename, -1, NULL, NULL, NULL)))
   {
-    *error_code= MYX_CANT_READ_FROM_FILE;
+    *error_code= MYX_CHARSET_CONVERSION_ERROR;
+    return NULL;
+  };
+
+  if (!file_exists(local_filename))
+  {
+    *error_code = MYX_ERROR_CANT_OPEN_FILE;
     return NULL;
   }
 
   doc = myx_xmlParseFile(filename);
 
-  if (doc == NULL ) {
+  if (doc == NULL)
+  {
     *error_code = MYX_XML_PARSE_ERROR;
     return NULL;
   }
@@ -117,7 +119,13 @@ MYX_BOOKMARKS * myx_bookmarks_load(const char *filename, MYX_LIB_ERROR *error_co
 }
 
 
-
+/**
+ * Stores a bookmarks collection into a file.
+ *
+ * @param filename The name of the file to store the bookmarks in (UTF-8 encoded).
+ * @param bookmarks The bookmarks to store.
+ * @return 0 if everything went fine otherwise -1.
+ */
 MYX_LIB_ERROR myx_bookmarks_store(const char *filename, MYX_BOOKMARKS *bookmarks)
 {
   xmlDocPtr doc;
