@@ -73,11 +73,17 @@ MYX_HISTORY * myx_history_load(const char *filename, MYX_LIB_ERROR *error_code)
   xmlDocPtr doc;
   xmlNodePtr root;
   MYX_HISTORY *history;
-
-
+  char* local_filename;
   *error_code = MYX_NO_ERROR;
 
-  if (! file_exists(filename) )  {
+  if (!(local_filename= g_filename_from_utf8(filename, -1, NULL, NULL, NULL)))
+  {
+    *error_code= MYX_CHARSET_CONVERSION_ERROR;
+    return NULL;
+  };
+
+  if (!file_exists(local_filename))
+  {
     *error_code = MYX_ERROR_CANT_OPEN_FILE;
     return NULL;
   }
@@ -187,6 +193,8 @@ MYX_HISTORY_TREE *myx_history_get_tree(MYX_HISTORY *history)
   GDate cur_date;
   MYX_HISTORY_TREE *history_tree;
   unsigned int i;
+  time_t timestamp;
+  struct tm *cur_tm_date;
 
   history_tree= g_malloc0(sizeof(MYX_HISTORY_TREE));
 
@@ -195,30 +203,30 @@ MYX_HISTORY_TREE *myx_history_get_tree(MYX_HISTORY *history)
 
 
   /* get current date */
-  {
-    time_t timestamp;
-    struct tm *cur_tm_date;
-
-    timestamp= time(NULL);
-    cur_tm_date= gmtime(&timestamp);
-    g_date_set_dmy(&cur_date, cur_tm_date->tm_mday, cur_tm_date->tm_mon+1, cur_tm_date->tm_year+1900);
-  }
+  timestamp= time(NULL);
+  cur_tm_date= localtime(&timestamp); // gmtime(&timestamp); use local time for comparation, not UTC.
+  g_date_set_dmy(&cur_date, cur_tm_date->tm_mday, cur_tm_date->tm_mon+1, cur_tm_date->tm_year+1900);
 
   for (i=0; i < history->entries_num; i++)
   {
     GDate tmp_date;
-    //MYX_HISTORY_INTERVAL *last;
-    //int found;
-    int year, month, day, hour, min, sec;
-    int diff_days;    
+    int diff_days;
+    struct tm entryDate;
+    time_t dummy;
+    struct tm* localEntryDate;
 
     /* put each history_entry into
     * the appropriate category/group
     */
 
-    split_iso8601_date(history->entries[i].date_last_access, &year, &month, &day,
-      &hour, &min, &sec);
-    g_date_set_dmy(&tmp_date, day, month, year);
+    split_iso8601_date(history->entries[i].date_last_access, &entryDate.tm_year, &entryDate.tm_mon, &entryDate.tm_mday,
+      &entryDate.tm_hour, &entryDate.tm_min, &entryDate.tm_sec);
+    --entryDate.tm_mon;
+    entryDate.tm_year -= 1900;
+    entryDate.tm_isdst  = cur_tm_date->tm_isdst;
+    dummy = mktime(&entryDate);
+    localEntryDate = localtime(&dummy);
+    g_date_set_dmy(&tmp_date, localEntryDate->tm_mday, localEntryDate->tm_mon + 1, localEntryDate->tm_year + 1900);
 
     diff_days= g_date_days_between(&cur_date, &tmp_date);
 
@@ -593,23 +601,41 @@ static int history_entry_compare_asc(const void *a_,const void *b_)
 static void split_iso8601_date(const char* iso_date, int *year, int *month, 
                                int *day, int *hour, int *min, int *sec)
 {
-  assert(strlen(iso_date) >= 10);
-
-  *year= (iso_date[0]-'0') * 1000 + (iso_date[1]-'0') * 100 + (iso_date[2]-'0') * 10 + 
-    (iso_date[3]-'0'); 
-  *month= (iso_date[5]-'0') * 10 + (iso_date[6]-'0');
-  *day= (iso_date[8]-'0') * 10 + (iso_date[9]-'0');
-
-  if (strlen(iso_date) < 19)
+  //assert(strlen(iso_date) >= 10);
+  if(strlen(iso_date) >= 10)
   {
-    //old file
-    *hour= *min= *sec= 0;
-    return;
-  }
+    *year= (iso_date[0]-'0') * 1000 + (iso_date[1]-'0') * 100 + (iso_date[2]-'0') * 10 + 
+      (iso_date[3]-'0'); 
+    *month= (iso_date[5]-'0') * 10 + (iso_date[6]-'0');
+    *day= (iso_date[8]-'0') * 10 + (iso_date[9]-'0');
 
-  *hour= (iso_date[11]-'0') * 10 + (iso_date[12]-'0');
-  *min= (iso_date[14]-'0') * 10 + (iso_date[15]-'0');
-  *sec= (iso_date[17]-'0') * 10 + (iso_date[18]-'0');
+    if (strlen(iso_date) < 19)
+    {
+      //old file
+      *hour= *min= *sec= 0;
+      return;
+    }
+
+    *hour= (iso_date[11]-'0') * 10 + (iso_date[12]-'0');
+    *min= (iso_date[14]-'0') * 10 + (iso_date[15]-'0');
+    *sec= (iso_date[17]-'0') * 10 + (iso_date[18]-'0');
+
+    if((*hour < 0) || (*hour > 23) || (*min < 0) || (*min > 59) || (*sec < 0) || (*sec > 59) 
+      || (*month < 1) || (*month > 12) || (*day < 1) || (*day > 31)) 
+    {
+      *year= 2004;
+      *month= 2;
+      *day= 1;
+      *hour= *min= *sec= 0;
+    }
+  }
+  else  // invalid date format
+  {
+    *year= 2004;
+    *month= 2;
+    *day= 1;
+    *hour= *min= *sec= 0;
+  }
 }
 
 static void store_history_entry(MYX_HISTORY_ENTRY *he, xmlNodePtr parent)
