@@ -38,7 +38,7 @@ class MGResultSetModel : public Glib::Object, public Gtk::TreeModel, public MYXR
     int _stamp;
 
     bool _edit_enabled;
-    bool _disable_checks;
+    mutable bool _disable_checks;
 
     Gtk::TreeModel::ColumnRecord _column_record;
 
@@ -48,22 +48,34 @@ class MGResultSetModel : public Glib::Object, public Gtk::TreeModel, public MYXR
     virtual Gtk::TreeModelFlags get_flags_vfunc();
     virtual int get_n_columns_vfunc();
     virtual GType get_column_type_vfunc(int index);
-    virtual bool iter_next_vfunc(GtkTreeIter* iter);
-    virtual bool iter_children_vfunc(GtkTreeIter* iter, const GtkTreeIter* parent);
-    virtual bool iter_has_child_vfunc(const GtkTreeIter* iter);
-    virtual int iter_n_children_vfunc(const GtkTreeIter* iter);
-    virtual bool iter_nth_child_vfunc(GtkTreeIter* iter, const GtkTreeIter* parent, int n);
-    virtual bool iter_parent_vfunc(GtkTreeIter* iter, const GtkTreeIter* child);
-    virtual void ref_node_vfunc(GtkTreeIter* iter);
-    virtual void unref_node_vfunc(GtkTreeIter* iter);
-    virtual Gtk::TreeModel::Path get_path_vfunc(const Gtk::TreeModel::iterator& iter);
-    virtual bool get_iter_vfunc(GtkTreeIter* iter, const Gtk::TreeModel::Path& path);
-    virtual void get_value_vfunc(const Gtk::TreeModel::iterator& iter, int column, GValue* value);
   
+    //virtual bool iter_next_vfunc(GtkTreeIter* iter);
+    virtual bool iter_next_vfunc (const Gtk::TreeModel::iterator& iter, Gtk::TreeModel::iterator& iter_next) const;
+    //virtual bool iter_children_vfunc(GtkTreeIter* iter, const GtkTreeIter* parent);
+    virtual bool iter_children_vfunc (const Gtk::TreeModel::iterator& parent, Gtk::TreeModel::iterator& iter) const;
+    //virtual bool iter_has_child_vfunc(const GtkTreeIter* iter);
+    virtual bool iter_has_child_vfunc (const Gtk::TreeModel::iterator& iter) const;
+    //virtual int iter_n_children_vfunc(const GtkTreeIter* iter);
+    virtual int iter_n_children_vfunc (const Gtk::TreeModel::iterator& iter) const;
+    //virtual bool iter_nth_child_vfunc(GtkTreeIter* iter, const GtkTreeIter* parent, int n);
+    virtual bool iter_nth_child_vfunc (const Gtk::TreeModel::iterator& parent, int n, Gtk::TreeModel::iterator& iter) const;
+    //virtual bool iter_parent_vfunc(GtkTreeIter* iter, const GtkTreeIter* child);
+    virtual bool 	iter_parent_vfunc (const Gtk::TreeModel::iterator& child, Gtk::TreeModel::iterator& iter) const;
+    //virtual void ref_node_vfunc(GtkTreeIter* iter);
+    virtual void 	ref_node_vfunc (const Gtk::TreeModel::iterator& iter) const;
+    //virtual void unref_node_vfunc(GtkTreeIter* iter);
+    virtual void 	unref_node_vfunc (const Gtk::TreeModel::iterator& iter) const;
+    //virtual Gtk::TreeModel::Path get_path_vfunc(const Gtk::TreeModel::iterator& iter);
+    virtual Gtk::TreeModel::Path get_path_vfunc(const Gtk::TreeModel::iterator& iter) const;
+    //virtual bool get_iter_vfunc(GtkTreeIter* iter, const Gtk::TreeModel::Path& path);
+    virtual bool get_iter_vfunc (const Gtk::TreeModel::Path& path, Gtk::TreeModel::iterator& iter) const;
+    //virtual void get_value_vfunc(const Gtk::TreeModel::iterator& iter, int column, GValue* value);
+    virtual void get_value_vfunc (const Gtk::TreeModel::iterator& iter, int column, Glib::ValueBase& value) const;
+    // unchanged
     virtual void set_value_impl(const Gtk::TreeModel::iterator& row, int column, const Glib::ValueBase& value);
     
-    bool treeiter_valid(const Gtk::TreeModel::iterator& iter);
-    bool treeiter_valid(const GtkTreeIter *iter);
+    bool treeiter_valid(const Gtk::TreeModel::iterator& iter) const;
+    bool treeiter_valid(const GtkTreeIter *iter) const;
 
     MGResultSetModel(const Gtk::TreeModelColumnRecord& columns,
                      MYX_RESULTSET *resultset);
@@ -78,7 +90,7 @@ class MGResultSetModel : public Glib::Object, public Gtk::TreeModel, public MYXR
                                                  MYX_RESULTSET *resultset);
 
     void set_value(const Gtk::TreeModel::iterator& row, int column, const gpointer value, guint value_length);
-    bool get_value(const Gtk::TreeModel::iterator& row, unsigned int column, gpointer &value, guint &value_length);
+    bool get_value(const Gtk::TreeModel::iterator& row, unsigned int column, gpointer &value, guint &value_length) const;
 
     void set_state_colors(const std::vector<Glib::ustring> &colors);
     void set_diff_colors(const std::vector<Glib::ustring> &colors);
