@@ -44,7 +44,7 @@ MQMainWindow::MQMainWindow(GtkWindow *win)
                               "add_bookmark_dialog")),
     _has_status(false)
 {
-  _bookmarks.signal_changed().connect(SigC::slot(*this,&MQMainWindow::bookmarks_changed));
+  _bookmarks.signal_changed().connect(sigc::mem_fun(*this,&MQMainWindow::bookmarks_changed));
   
   _state= new MQSavedState();
 }
@@ -75,7 +75,7 @@ bool MQMainWindow::init(MYSQL *mysql, const MYX::UserConnection &conn)
 
   _workarea->set_dispatcher(_dispatcher);
 
-  signal_delete_event().connect(SigC::slot(*this,&MQMainWindow::close_clicked));
+  signal_delete_event().connect(sigc::mem_fun(*this,&MQMainWindow::close_clicked));
 
   _workarea->add_result_view("");
   if (_state->first_time)
@@ -102,13 +102,13 @@ bool MQMainWindow::init(MYSQL *mysql, const MYX::UserConnection &conn)
 
   
   // bind menu items
-  _xml->get_menu_item("new_connection2")->signal_activate().connect(SigC::slot(*this,&MQMainWindow::new_connection));
-  _xml->get_menu_item("manage_connections1")->signal_activate().connect(SigC::bind<bool>(SigC::slot(*this,&MQMainWindow::show_preferences),true));
-  _xml->get_menu_item("preferences1")->signal_activate().connect(SigC::bind<bool>(SigC::slot(*this,&MQMainWindow::show_preferences),false));
+  _xml->get_menu_item("new_connection2")->signal_activate().connect(sigc::mem_fun(*this,&MQMainWindow::new_connection));
+  _xml->get_menu_item("manage_connections1")->signal_activate().connect(sigc::bind<bool>(sigc::mem_fun(*this,&MQMainWindow::show_preferences),true));
+  _xml->get_menu_item("preferences1")->signal_activate().connect(sigc::bind<bool>(sigc::mem_fun(*this,&MQMainWindow::show_preferences),false));
 
-  _xml->get_menu_item("about1")->signal_activate().connect(SigC::slot(*this,&MQMainWindow::about));
+  _xml->get_menu_item("about1")->signal_activate().connect(sigc::mem_fun(*this,&MQMainWindow::about));
 
-  _xml->get_menu_item("mysql_text_console1")->signal_activate().connect(SigC::slot(*this,&MQMainWindow::start_console));
+  _xml->get_menu_item("mysql_text_console1")->signal_activate().connect(sigc::mem_fun(*this,&MQMainWindow::start_console));
   
   return true;
 }
@@ -213,14 +213,14 @@ void MQMainWindow::bookmarks_changed()
 {
   /*
   _bookmarks.update_bookmark_menu(_workarea->get_bookmark_menu(), 3,
-                                  SigC::slot(*_workarea,&MQWorkArea::open_bookmark));
+                                  sigc::mem_fun(*_workarea,&MQWorkArea::open_bookmark));
    */
 }
 
 void MQMainWindow::show_preferences(bool conn_only)
 {
   MGPreferencesEditor *editor= MGPreferencesEditor::instance();
-//  _prefs_editor->signal_changed().connect(SigC::slot(*this,&MAdministrator::prefs_changed));
+//  _prefs_editor->signal_changed().connect(sigc::mem_fun(*this,&MAdministrator::prefs_changed));
 
   editor->show(conn_only);
 }
