@@ -170,7 +170,7 @@ MQResultSetView::MQResultSetView(MQQueryDispatcher *disp,
   add(_box);
   _box.show();
   
-  signal_button_press_event().connect(SigC::slot(*this,&MQResultSetView::show_menu));
+  signal_button_press_event().connect(sigc::mem_fun(*this,&MQResultSetView::show_menu));
 
   _params= new MQQueryParameters(params);
   
@@ -193,7 +193,7 @@ MQResultSetView::MQResultSetView(MQQueryDispatcher *disp,
 
   _ebox.set_name("resultset_bar");
   
-  _ebox.signal_button_press_event().connect(SigC::slot(*this,&MQResultSetView::editbar_clicked));
+  _ebox.signal_button_press_event().connect(sigc::mem_fun(*this,&MQResultSetView::editbar_clicked));
 
   _edit_btn.property_can_focus()= false;
   _apply_btn.property_can_focus()= false;
@@ -202,10 +202,10 @@ MQResultSetView::MQResultSetView(MQQueryDispatcher *disp,
   _last_btn.property_can_focus()= false;
   _search_btn.property_can_focus()= false;
   
-  _edit_btn.signal_clicked().connect(SigC::slot(*this,&MQResultSetView::edit_start));
-  _apply_btn.signal_clicked().connect(SigC::slot(*this,&MQResultSetView::edit_apply));
-  _first_btn.signal_clicked().connect(SigC::slot(*this,&MQResultSetView::go_first));
-  _last_btn.signal_clicked().connect(SigC::slot(*this,&MQResultSetView::go_last));
+  _edit_btn.signal_clicked().connect(sigc::mem_fun(*this,&MQResultSetView::edit_start));
+  _apply_btn.signal_clicked().connect(sigc::mem_fun(*this,&MQResultSetView::edit_apply));
+  _first_btn.signal_clicked().connect(sigc::mem_fun(*this,&MQResultSetView::go_first));
+  _last_btn.signal_clicked().connect(sigc::mem_fun(*this,&MQResultSetView::go_last));
     
   _box.pack_start(_ebox, false, false);
   _ebox.add(_hbox);
@@ -245,37 +245,37 @@ MQResultSetView::MQResultSetView(MQQueryDispatcher *disp,
 
   // result menu
   myg_menu_add(_edit_menu, Gtk::Stock::ADD, _("Add Row"),
-               SigC::slot(*this,&MQResultSetView::add_row_mi),
+               sigc::mem_fun(*this,&MQResultSetView::add_row_mi),
                "add_row");
 //  myg_menu_add(_edit_menu, Gtk::Stock::UNDO, _("Undo Last Edition"),
-//               SigC::slot(*this,&MQResultSetView::undo_row_mi),
+//               sigc::mem_fun(*this,&MQResultSetView::undo_row_mi),
 //               "undo");
 
   myg_menu_add(_edit_menu, Gtk::Stock::REMOVE, _("Delete Row"),
-               SigC::slot(*this,&MQResultSetView::delete_row_mi),
+               sigc::mem_fun(*this,&MQResultSetView::delete_row_mi),
                "delete_row");
 
   myg_menu_add(_edit_menu);
 
   // column stuff
   myg_menu_add(_edit_menu, Gtk::Stock::OPEN, _("_Load Field Content"),
-               SigC::bind<FieldAction>(SigC::slot(*this,&MQResultSetView::column_action_mi), FieldLoad),
+               sigc::bind<FieldAction>(sigc::mem_fun(*this,&MQResultSetView::column_action_mi), FieldLoad),
                "load_field");
   myg_menu_add(_edit_menu, Gtk::Stock::SAVE, _("_Save Field Content"),
-               SigC::bind<FieldAction>(SigC::slot(*this,&MQResultSetView::column_action_mi), FieldSave),
+               sigc::bind<FieldAction>(sigc::mem_fun(*this,&MQResultSetView::column_action_mi), FieldSave),
                "save_field");
   myg_menu_add(_edit_menu, Gtk::Stock::COPY, _("_Copy Field Content"),
-               SigC::bind<FieldAction>(SigC::slot(*this,&MQResultSetView::column_action_mi), FieldCopy),
+               sigc::bind<FieldAction>(sigc::mem_fun(*this,&MQResultSetView::column_action_mi), FieldCopy),
                "copy_field");
   myg_menu_add(_edit_menu, Gtk::Stock::CLEAR, _("C_lear Field Content"),
-               SigC::bind<FieldAction>(SigC::slot(*this,&MQResultSetView::column_action_mi), FieldClear),
+               sigc::bind<FieldAction>(sigc::mem_fun(*this,&MQResultSetView::column_action_mi), FieldClear),
                "clear_field");
   myg_menu_add(_edit_menu);
   myg_menu_add(_edit_menu, _("_View Field in Popup Editor"),
-               SigC::bind<FieldAction>(SigC::slot(*this,&MQResultSetView::column_action_mi), FieldView),
+               sigc::bind<FieldAction>(sigc::mem_fun(*this,&MQResultSetView::column_action_mi), FieldView),
                "view_field");
   myg_menu_add(_edit_menu, _("_Edit Field in Popup Editor"),
-               SigC::bind<FieldAction>(SigC::slot(*this,&MQResultSetView::column_action_mi), FieldEdit),
+               sigc::bind<FieldAction>(sigc::mem_fun(*this,&MQResultSetView::column_action_mi), FieldEdit),
                "edit_field");
   
   
@@ -292,7 +292,7 @@ MQResultSetView::MQResultSetView(MQQueryDispatcher *disp,
   _msgtree.get_column(1)->set_resizable();
   _msgtree.get_column(2)->set_resizable();
   _msgtree.set_model(_msgstore);
-  _msgtree.get_selection()->signal_changed().connect(SigC::slot(*this,&MQResultSetView::selected_error));
+  _msgtree.get_selection()->signal_changed().connect(sigc::mem_fun(*this,&MQResultSetView::selected_error));
 
   _box.pack_start(_msgscroll, false, false, 2);
 }
@@ -415,7 +415,7 @@ void MQResultSetView::display_result(MYX_RESULTSET *result)
 
     _tree= new PopupTreeView(&_edit_menu);
     
-    _tree->signal_key_release_event().connect(SigC::slot(*this,&MQResultSetView::tree_key_released));
+    _tree->signal_key_release_event().connect(sigc::mem_fun(*this,&MQResultSetView::tree_key_released));
     _tree->set_data("owner", this);
 
     _model= MGResultSetModel::create(*_columns, result);
@@ -437,12 +437,11 @@ void MQResultSetView::display_result(MYX_RESULTSET *result)
 
     _scroll.add(*_tree);
 
+    _tree->set_model(_model);
     _tree->show();
 
     _tree->set_rules_hint();
 
-    _tree->set_model(_model);
-
     // add indicator column
     Gtk::TreeViewColumn *col;
     MQIndicatorCellRenderer *irend= Gtk::manage(new MQIndicatorCellRenderer);
@@ -495,7 +494,7 @@ void MQResultSetView::display_result(MYX_RESULTSET *result)
         _tree->append_column("", *rend);
         col= _tree->get_column(i+1);
 
-        rend->signal_edited().connect(SigC::bind<int>(SigC::slot(*this,&MQResultSetView::string_data_edited), (i)*2));
+        rend->signal_edited().connect(sigc::bind<int>(sigc::mem_fun(*this,&MQResultSetView::string_data_edited), (i)*2));
 
         col->add_attribute(rend->property_text(), *_columns->columns[text_idx++]);
         col->add_attribute(rend->property_background(), *_columns->bgcolors[i]);
@@ -532,13 +531,13 @@ void MQResultSetView::display_result(MYX_RESULTSET *result)
       col->get_widget()->show();
     }
     _tree->get_selection()->set_mode(Gtk::SELECTION_BROWSE);
-    _tree->get_selection()->signal_changed().connect(SigC::slot(*this,&MQResultSetView::selected_row));
+    _tree->get_selection()->signal_changed().connect(sigc::mem_fun(*this,&MQResultSetView::selected_row));
     go_first();
     selected_row();
 
-    _tree->signal_will_popup().connect(SigC::slot(*this,&MQResultSetView::update_menu));
+    _tree->signal_will_popup().connect(sigc::mem_fun(*this,&MQResultSetView::update_menu));
     
-    _tree->signal_focus_in_event().connect(SigC::slot(*this,&MQResultSetView::focused));
+    _tree->signal_focus_in_event().connect(sigc::mem_fun(*this,&MQResultSetView::focused));
   }
 }
 
@@ -598,12 +597,12 @@ void MQResultSetView::execute_query(const Glib::ustring &query,
   set_sensitive(false);
 
   _query_started_signal.emit(this, false);
-  
+
   _dispatcher->start_query(query, _params->get_all(),
-                           SigC::slot(*this,&MQResultSetView::query_finished),
-                           SigC::slot(*this,&MQResultSetView::query_progress),
-                           SigC::slot(*this,&MQResultSetView::query_error),
-                           SigC::slot(*this,&MQResultSetView::query_fetch_more),
+                           sigc::mem_fun(*this,&MQResultSetView::query_finished),
+                           sigc::mem_fun(*this,&MQResultSetView::query_progress),
+                           sigc::mem_fun(*this,&MQResultSetView::query_error),
+                           sigc::mem_fun(*this,&MQResultSetView::query_fetch_more),
                            NULL);
 }
 
@@ -622,10 +621,10 @@ void MQResultSetView::fetch_more(MYSQL *mysql)
   _query_started_signal.emit(this, false);
 
   _dispatcher->start_query("", NULL,
-                           SigC::slot(*this,&MQResultSetView::query_finished),
-                           SigC::slot(*this,&MQResultSetView::query_progress),
-                           SigC::slot(*this,&MQResultSetView::query_error),
-                           SigC::slot(*this,&MQResultSetView::query_fetch_more),
+                           sigc::mem_fun(*this,&MQResultSetView::query_finished),
+                           sigc::mem_fun(*this,&MQResultSetView::query_progress),
+                           sigc::mem_fun(*this,&MQResultSetView::query_error),
+                           sigc::mem_fun(*this,&MQResultSetView::query_fetch_more),
                            mysql);
 }
 
@@ -724,8 +723,8 @@ bool MQResultSetView::explain_query()
     vbox->pack_start(*expscroll, true, true);
     
     button= Gtk::manage(new Gtk::Button(Gtk::Stock::CLOSE));
-    button->signal_clicked().connect(SigC::slot(*this,&MQResultSetView::hide_explain));
-    button->signal_delete_event().connect(SigC::slot(*this,&MQResultSetView::hide_explain_d));
+    button->signal_clicked().connect(sigc::mem_fun(*this,&MQResultSetView::hide_explain));
+    button->signal_delete_event().connect(sigc::mem_fun(*this,&MQResultSetView::hide_explain_d));
     {
       Gtk::HBox *hbox= Gtk::manage(new Gtk::HBox(false, 0));
       hbox->pack_end(*button, false, false);
@@ -975,22 +974,34 @@ bool MQResultSetView::query_progress(MYX_RESULTSET *result,
 }
 
 
-void MQResultSetView::query_error(MYX_RESULTSET *result, MYX_MYSQL_ERROR_MSGS *errors)
+void MQResultSetView::query_error(MYX_RESULTSET *result, MYX_MYSQL_ERROR_MSGS *errors, MYX_LIB_ERROR merror)
 {
   if (!errors)
   {
-    if (result)
-      _status_label.set_text(ufmt(_("Query cancelled. %i rows were fetched."),
-                                  result->rows_num));
+    if (merror == MYX_MEMORY_LIMIT_EXCEEDED)
+    {
+      if (result)
+        _status_label.set_text(ufmt(_("Memory limit exceeded. %i rows fetched."),
+                                    result->rows_num));
+      else
+        _status_label.set_text(_("Memory limit exceeded."));
+    }
     else
-      _status_label.set_text(_("Query cancelled."));
+    {
+      if (result)
+        _status_label.set_text(ufmt(_("Query cancelled. %i rows fetched."),
+                                    result->rows_num));
+      else
+        _status_label.set_text(_("Query cancelled."));
+    }
   }
   else
+  {
     _status_label.set_text(_("Error executing query"));
-
+  }
   _is_busy= false;
   set_sensitive(false);
-  
+
   display_result(result);
 
   _query_finished_signal.emit(this);
@@ -1096,12 +1107,11 @@ void MQResultSetView::save_finished(MYX_RS_ACTION_ERRORS *errors)
 
     // refresh
     _tree->queue_draw();
-    
     _query_finished_signal.emit(this);
     
     _msgscroll.hide();
   }
-  
+
   _tree->set_model(Gtk::ListStore::create(_msg_columns));
   _tree->set_model(_model);
 }
@@ -1205,21 +1215,18 @@ void MQResultSetView::edit_apply()
   _model->set_editable(false);
   _model->pre_commit();
   update_edit_button_sensitivity();
-
   _tree->set_model(Gtk::ListStore::create(_msg_columns));
   _tree->set_model(_model);
 
   if (_result->actions && _result->actions->actions_num > 0)
   {
     _query_started_signal.emit(this, true);
-    
     _dispatcher->save_edits(_result,
-                            SigC::slot(*this,&MQResultSetView::save_finished));
+                            sigc::mem_fun(*this,&MQResultSetView::save_finished));
   }
   else
   {
     _query_started_signal.emit(this, true);
-    
     save_finished(NULL);
   }
   _editable_change_signal.emit();
@@ -1465,7 +1472,7 @@ void MQResultSetView::column_action_mi(FieldAction action)
       cpath= g_strdup(path.to_string().c_str());
       ((Glib::Object*)editor)->set_data("path", cpath, (Glib::Object::DestroyNotify)g_free);
       
-      editor->signal_save().connect(SigC::bind<MGBlobEditor*,unsigned int>(SigC::slot(*this,&MQResultSetView::save_cell_change),
+      editor->signal_save().connect(sigc::bind<MGBlobEditor*,unsigned int>(sigc::mem_fun(*this,&MQResultSetView::save_cell_change),
                                                                                        (MGBlobEditor*)editor,(unsigned int)column_index));
     }
     break;
@@ -1789,9 +1796,9 @@ void MQResultSetView::set_selected_row_path(const Gtk::TreePath &path)
 
 
 
-void MQResultSetView::set_search_func(const SigC::Slot1<void,MQResultSetView*> &slot)
+void MQResultSetView::set_search_func(const sigc::slot1<void,MQResultSetView*> &slot)
 {
-  _search_btn.signal_clicked().connect(SigC::bind<MQResultSetView*>(slot,this));
+  _search_btn.signal_clicked().connect(sigc::bind<MQResultSetView*>(slot,this));
 }
 
 
@@ -1836,6 +1843,7 @@ void MQResultSetView::set_compact_editbar(bool flag)
 
 void MQResultSetView::editor_save(MGBlobEditor *editor)
 {
+  int column;
   gpointer buffer;
   gsize size;
   Glib::ustring path;
@@ -1843,7 +1851,12 @@ void MQResultSetView::editor_save(MGBlobEditor *editor)
   editor->get_data(buffer, size);
 
   path= (char*)((Glib::Object*)editor)->get_data("path");
+  column= (int)((Glib::Object*)editor)->get_data("column");
 
+  Gtk::TreeIter iter= _model->get_iter(Gtk::TreePath(path));
+  _model->set_value(iter, column*2, buffer, size);
+  _tree->queue_draw();
+  
   g_free(buffer);
 }
 
@@ -1869,19 +1882,20 @@ void MQResultSetView::blob_edit(CellRendererBlob *cell, Gtk::Widget *widget, con
   gsize fsize; 
   Gtk::TreeIter iter= _model->get_iter(Gtk::TreePath(path));
 
-  if (_model->get_value(iter, column, fdata, fsize))
+  if (_model->get_value(iter, column*2, fdata, fsize))
   {
     MGBlobEditor *editor= new MGBlobEditor(get_flags_for_data((char*)fdata, fsize, cell->is_binary()), true);
     editor->set_data(fdata, fsize);
     editor->resize(400, 300);
     editor->set_delete_on_close();
-    
+
     ((Glib::Object*)editor)->set_data("path", g_strdup(path.c_str()), g_free);
+    ((Glib::Object*)editor)->set_data("column", (char *)column, NULL);
     
     editor->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
     editor->set_transient_for(*(Gtk::Window*)widget->get_toplevel());
     editor->property_destroy_with_parent()=true;
-    editor->signal_save().connect(SigC::bind<MGBlobEditor*>(SigC::slot(*this,&MQResultSetView::editor_save),editor));
+    editor->signal_save().connect(sigc::bind<MGBlobEditor*>(sigc::mem_fun(*this,&MQResultSetView::editor_save),editor));
     
     editor->show();
   }
@@ -1894,7 +1908,7 @@ void MQResultSetView::blob_save(CellRendererBlob *cell, Gtk::Widget *widget, con
   gsize fsize;
   Gtk::TreeIter iter= _model->get_iter(Gtk::TreePath(path));
 
-  if (_model->get_value(iter, column, fdata, fsize))
+  if (_model->get_value(iter, column*2, fdata, fsize))
   {
     Gtk::FileSelection fsel(_("Save Field Data"));
     
@@ -1943,7 +1957,7 @@ void MQResultSetView::blob_load(CellRendererBlob *cell, Gtk::Widget *widget, con
 
       channel->read(buffer, size, size);
 
-      blob_data_edited(path, buffer, size, column);
+      blob_data_edited(path, buffer, size, column*2);
       
       g_free(buffer);
     }
@@ -1958,7 +1972,7 @@ void MQResultSetView::blob_load(CellRendererBlob *cell, Gtk::Widget *widget, con
 
 void MQResultSetView::blob_clear(CellRendererBlob *cell, Gtk::Widget *widget, const Glib::ustring &path, int column)
 {
-  blob_data_edited(path, NULL, 0, column);
+  blob_data_edited(path, NULL, 0, column*2);
 }
 
 
@@ -1968,7 +1982,7 @@ void MQResultSetView::blob_view(CellRendererBlob *cell, Gtk::Widget *widget, con
   gsize fsize; 
   Gtk::TreeIter iter= _model->get_iter(Gtk::TreePath(path));
 
-  if (_model->get_value(iter, column, fdata, fsize))
+  if (_model->get_value(iter, column*2, fdata, fsize))
   {
     MGBlobEditor *editor= new MGBlobEditor(get_flags_for_data((char*)fdata, fsize, cell->is_binary()));
     editor->set_data(fdata, fsize);
@@ -1994,4 +2008,3 @@ void MQResultSetView::string_data_edited(const Glib::ustring& path, const Glib::
 {
   _model->set_value(_model->get_iter(path), column, (const gpointer)new_text.data(), new_text.size());
 }
-
