@@ -194,7 +194,7 @@ void MQResultTab::add_resultset(MQResultSetView *rsv, bool vertical)
   item->frame.show();
   paned->show();
 
-  rsv->signal_activate().connect(SigC::slot(*this,&MQResultTab::activated));
+  rsv->signal_activate().connect(sigc::mem_fun(*this,&MQResultTab::activated));
 
   if (_rs_items.empty()) // first one
   {    
@@ -215,8 +215,8 @@ void MQResultTab::add_resultset(MQResultSetView *rsv, bool vertical)
 
   rsv->show();
 
-  rsv->signal_editable_changed().connect(SigC::slot(*this,&MQResultTab::update_cmp_toolbar));
-  rsv->signal_row_changed().connect(SigC::slot(*this,&MQResultTab::row_changed));
+  rsv->signal_editable_changed().connect(sigc::mem_fun(*this,&MQResultTab::update_cmp_toolbar));
+  rsv->signal_row_changed().connect(sigc::mem_fun(*this,&MQResultTab::row_changed));
 }
 
 
@@ -394,7 +394,7 @@ void MQResultTab::set_sync_scrolling(bool flag)
     for (unsigned int i= 0; i < _rs_items.size(); i++)
     {
       _rs_items[i]->scroll_con=
-        _rs_items[i]->rs->get_scrollwin()->get_vscrollbar()->signal_value_changed().connect(SigC::bind<MQResultSetView*>(SigC::slot(*this,&MQResultTab::scrolled),_rs_items[i]->rs));
+        _rs_items[i]->rs->get_scrollwin()->get_vscrollbar()->signal_value_changed().connect(sigc::bind<MQResultSetView*>(sigc::mem_fun(*this,&MQResultTab::scrolled),_rs_items[i]->rs));
     }
   }
   else
@@ -433,18 +433,18 @@ void MQResultTab::show_compare_toolbar()
     _topBox.pack_start(*toolbar, false, false);
     toolbar->show();
 
-    _toolbar_xml->get_button("actions")->signal_clicked().connect(SigC::slot(*this,&MQResultTab::cmp_actions));
-    _toolbar_xml->get_button("close")->signal_clicked().connect(SigC::slot(*this,&MQResultTab::cmp_close));
+    _toolbar_xml->get_button("actions")->signal_clicked().connect(sigc::mem_fun(*this,&MQResultTab::cmp_actions));
+    _toolbar_xml->get_button("close")->signal_clicked().connect(sigc::mem_fun(*this,&MQResultTab::cmp_close));
 
-    _toolbar_xml->get_button("next")->signal_clicked().connect(SigC::slot(*this,&MQResultTab::cmp_go_next));
-    _toolbar_xml->get_button("previous")->signal_clicked().connect(SigC::slot(*this,&MQResultTab::cmp_go_back));
+    _toolbar_xml->get_button("next")->signal_clicked().connect(sigc::mem_fun(*this,&MQResultTab::cmp_go_next));
+    _toolbar_xml->get_button("previous")->signal_clicked().connect(sigc::mem_fun(*this,&MQResultTab::cmp_go_back));
     
     for (unsigned int i= 0; setup_data[i].widget; i++)
     {
       if (setup_data[i].menu)
-        ((Gtk::MenuItem*)_toolmenu_xml->get_widget(setup_data[i].widget))->signal_activate().connect(SigC::bind<CompareAction>(SigC::slot(*this,&MQResultTab::cmp_do_action),setup_data[i].action));
+        ((Gtk::MenuItem*)_toolmenu_xml->get_widget(setup_data[i].widget))->signal_activate().connect(sigc::bind<CompareAction>(sigc::mem_fun(*this,&MQResultTab::cmp_do_action),setup_data[i].action));
       else
-        _toolbar_xml->get_button(setup_data[i].widget)->signal_clicked().connect(SigC::bind<CompareAction>(SigC::slot(*this,&MQResultTab::cmp_do_action),setup_data[i].action));
+        _toolbar_xml->get_button(setup_data[i].widget)->signal_clicked().connect(sigc::bind<CompareAction>(sigc::mem_fun(*this,&MQResultTab::cmp_do_action),setup_data[i].action));
     }
     
     update_cmp_toolbar();
@@ -555,7 +555,7 @@ void MQResultTab::cmp_actions()
 {
   Gtk::Menu *menu= (Gtk::Menu*)_toolmenu_xml->get_widget("actions_menu");
   
-  menu->popup(SigC::slot(*this,&MQResultTab::cmp_get_menu_pos),0, 0);
+  menu->popup(sigc::mem_fun(*this,&MQResultTab::cmp_get_menu_pos),0, 0);
 }
 
 
