@@ -34,8 +34,9 @@ MQSQLCodeEditor::MQSQLCodeEditor()
 {  
   _sql_text= myx_init_mysql_text();
   
-  get_wrapper()->get_buffer()->signal_changed().connect(SigC::slot(*this,&MQSQLCodeEditor::text_changed));
-  get_wrapper()->signal_move_cursor().connect(SigC::slot(*this,&MQSQLCodeEditor::move_cursor));
+  get_wrapper()->get_buffer()->signal_changed().connect(sigc::mem_fun(*this,&MQSQLCodeEditor::text_changed));
+  //get_wrapper()->signal_move_cursor().connect(sigc::mem_fun(*this,&MQSQLCodeEditor::move_cursor));
+  get_wrapper()->get_buffer()->signal_mark_set().connect(sigc::mem_fun(*this,&MQSQLCodeEditor::move_cursor));
 }
 
 
@@ -158,6 +159,7 @@ void MQSQLCodeEditor::text_changed()
     l1= std::max(cur_line, prev_line);
   }
   invalidate_lines(l0, l1);
+  set_dirty(true);
 }
 
 
@@ -193,6 +195,8 @@ void MQSQLCodeEditor::set_text(const Glib::ustring &text, bool no_undo)
   
   if (no_undo)
     end_no_undo();
+  
+  set_dirty(!no_undo);
 }
 
 
@@ -243,7 +247,8 @@ bool MQSQLCodeEditor::get_statement(unsigned int i, Glib::ustring &text,
   return false;
 }
 
-void MQSQLCodeEditor::move_cursor(Gtk::MovementStep step, int count, bool extend_selection)
+//void MQSQLCodeEditor::move_cursor(Gtk::MovementStep step, int count, bool extend_selection)
+void MQSQLCodeEditor::move_cursor(const Gtk::TextBuffer::iterator&, const Glib::RefPtr<Gtk::TextBuffer::Mark>&)
 {
   _cursor_moved_signal.emit();
 }
@@ -268,7 +273,7 @@ MQScriptEditorTab::MQScriptEditorTab(MQQueryDispatcher *disp)
   _msg_tree.append_column(_("errno"), _msg_columns.errornum);
   _msg_tree.append_column(_("Row"), _msg_columns.row);
   _msg_tree.set_model(_msg_store);
-//  _msg_tree.get_selection()->signal_changed().connect(SigC::slot(*this,&MQResultSetView::selected_error));
+//  _msg_tree.get_selection()->signal_changed().connect(sigc::mem_fun(*this,&MQResultSetView::selected_error));
   for (unsigned int i= 0; i < 4; i++)
     _msg_tree.get_column(i)->set_resizable();
   
@@ -417,6 +422,7 @@ bool MQScriptEditorTab::execute_statement(const Glib::ustring &s)
                                     "Click 'Ignore' if you'd like to have this error ignored until the end of the script."),
                                   s.c_str(),
                                   mysql_error(my), myx_mysql_errno(my)),
+                             false,
                              Gtk::MESSAGE_ERROR,
                              Gtk::BUTTONS_NONE, true);
       
@@ -522,6 +528,10 @@ void MQScriptEditorTab::do_execute_script(bool cont, int until)
     change_state(SIdle);
     _script_finished_signal.emit(this);
   }
+  if(!error)
+  {
+    _editor->set_dirty(false);
+  }  
 }
 
 
@@ -679,3 +689,19 @@ void MQScriptEditorTab::set_text(const Glib::ustring &text)
   _editor->set_text(text, true);
   reset();
 }
+
+bool MQScriptEditorTab::tab_will_close()
+{
+  if(_editor->get_dirty())
+  {
+    Gtk::MessageDialog dlg(
+      *static_cast<Gtk::Window*>(get_toplevel()),
+      Glib::ustring(_("The script is modified, are you sure you want to close the editor?")),
+      false,
+      Gtk::MESSAGE_QUESTION,
+      Gtk::BUTTONS_OK_CANCEL);
+    
+    return (dlg.run() == GTK_RESPONSE_OK);
+  }
+  return true;
+}
