@@ -324,8 +324,10 @@ int main(int argc, char **argv)
 
     if (!mainw->init(mysql, user_connection))
     {
-      Gtk::MessageDialog dlg(_("Error initializing GUI"), Gtk::MESSAGE_ERROR,
-                             Gtk::BUTTONS_OK);
+      Gtk::MessageDialog dlg(_("Error initializing GUI"), 
+                              false,
+                              Gtk::MESSAGE_ERROR,
+                              Gtk::BUTTONS_OK);
       dlg.run();
       return 1;
     }
