@@ -33,6 +33,10 @@
 - (void)setActiveResultSet:(MQResultSetView*)rsview;
 - (MQResultSetView*)activeResultSet;
 
+- (BOOL)hasUnsavedChanges;
+
+- (void)exportResultSet:(const char*)format;
+
 - (NSArray*)resultSetsWithMaster:(MQResultSetView*)rsview;
 
 - (void)setDefaultSchema:(NSString*)schema;
